<?php
/**
 * Clipart Meta Boxes
 *
 * Registers clipart post type metaboxes.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Admin_Menus Class.
 */
class VAM_UDesign_Admin_Menus {

  /**
   * Constructor.
   */
  public function __construct() {
    add_action( 'admin_menu', array( $this, 'admin_menu' ), 8 );
    add_action( 'admin_menu', array( $this, 'settings_menu' ), 40 );
    add_action( 'admin_head', array( $this, 'menu_order_count' ) );
    add_filter( 'menu_order', array( $this, 'menu_order' ) );
    add_filter( 'custom_menu_order', array( $this, 'custom_menu_order' ) );
  }

  /**
   * Add menu items.
   */
  public function admin_menu() {
    global $menu;

    if ( current_user_can( 'manage_woocommerce' ) ) {
      $menu[] = array( '', 'read', 'separator-udesign', '', 'wp-menu-separator udesign' );
    }

    add_menu_page( __( 'uDesign', 'vam-udesign' ), __( 'uDesign', 'vam-udesign' ), 'manage_woocommerce', 'udesign', null, null, '50.5' );
  }


  /**
   * Add menu item.
   */
  public function settings_menu() {
   add_submenu_page( 'udesign', __( 'uDesign Settings', 'vam-udesign' ),  __( 'Settings', 'vam-udesign' ) , 'manage_woocommerce', 'udesign-settings', array( $this, 'settings_page' ) );
  }

  /**
   * Init the settings page.
   */
  public function settings_page() {
    VAM_UDesign_Admin_Settings::output();
  }

  /**
   * Custom menu order.
   *
   * @return bool
   */
  public function custom_menu_order() {
    return current_user_can( 'manage_woocommerce' );
  }

  /**
   * Reorder the WC menu items in admin.
   *
   * @param mixed $menu_order
   * @return array
   */
  public function menu_order( $menu_order ) {
    // Initialize our custom order array
    $udesign_menu_order = array();

    // Get the index of our custom separator
    $udesign_separator = array_search( 'separator-udesign', $menu_order );

    // Get index of clipart menu
    $udesign_clipart = array_search( 'edit.php?post_type=clipart', $menu_order );

    // Get index of design menu
    $udesign_design = array_search( 'edit.php?post_type=design', $menu_order );

    // Loop through menu order and do some rearranging
    foreach ( $menu_order as $index => $item ) {

      if ( ( ( 'udesign' ) == $item ) ) {
        $udesign_menu_order[] = 'separator-woocommerce';
        $udesign_menu_order[] = $item;
        $udesign_menu_order[] = 'edit.php?post_type=clipart';
        $udesign_menu_order[] = 'edit.php?post_type=design';
        unset( $menu_order[ $udesign_separator ] );
        unset( $menu_order[ $udesign_clipart ] );
        unset( $menu_order[ $udesign_design ] );
      } elseif ( ! in_array( $item, array( 'separator-udesign' ) ) ) {
        $udesign_menu_order[] = $item;
      }

    }

    // Return order
    return $udesign_menu_order;
  }

  /**
   * Remove 'uDesign' sub menu item.
   */
  public function menu_order_count() {
    global $submenu;

    if ( isset( $submenu['udesign'] ) ) {
      unset( $submenu['udesign'][0] );
    }
  }

}

new VAM_UDesign_Admin_Menus();
