<?php
/**
 * Post Types Admin
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Admin_Post_Types Class.
 */
class VAM_UDesign_Admin_Post_Types {

  /**
   * Constructor.
   */
  public function __construct() {
    add_filter( 'manage_design_posts_columns', array( $this, 'design_columns' ) );
    add_action( 'manage_design_posts_custom_column', array( $this, 'render_design_columns' ), 2 );
    add_filter( 'manage_edit-design_sortable_columns', array( $this, 'design_sortable_columns' ) );
    add_filter( 'manage_clipart_posts_columns', array( $this, 'clipart_columns' ) );
    add_action( 'manage_clipart_posts_custom_column', array( $this, 'render_clipart_columns' ), 2 );
    add_filter( 'manage_edit-clipart_sortable_columns', array( $this, 'clipart_sortable_columns' ) );
    add_filter( 'list_table_primary_column', array( $this, 'list_table_primary_column' ), 10, 2 );
    add_action( 'post_submitbox_misc_actions', array( $this, 'design_featured' ) );
    add_filter( 'woocommerce_screen_ids', array( $this, 'screen_ids' ) );
    add_action( 'before_delete_post', array( $this, 'before_delete_post' ), 1 );
  }

  /**
   * Define custom columns for designs.
   * @param  array $existing_columns
   * @return array
   */
  public function design_columns( $existing_columns ) {
    if ( empty( $existing_columns ) && ! is_array( $existing_columns ) ) {
      $existing_columns = array();
    }

    unset( $existing_columns['title'], $existing_columns['comments'], $existing_columns['date'], $existing_columns['author'] );

    $columns          = array();
    $columns['cb']    = '<input type="checkbox" />';
    $columns['thumb'] = '<span class="wc-image tips" data-tip="' . esc_attr__( 'Image', 'vam-udesign' ) . '">' . __( 'Image', 'vam-udesign' ) . '</span>';
    $columns['name']  = __( 'Name', 'vam-udesign' );
    $columns['taxonomy-design_cat']  = __( 'Categories', 'vam-udesign' );
    $columns['taxonomy-design_tag']  = __( 'Tags', 'vam-udesign' );
    $columns['featured']     = '<span class="wc-featured tips" data-tip="' . esc_attr__( 'Featured', 'vam-udesign' ) . '">' . __( 'Featured', 'vam-udesign' ) . '</span>';
    $columns['date']         = __( 'Date', 'vam-udesign' );

    return array_merge( $columns, $existing_columns );
  }

  /**
   * Define custom columns for cliparts.
   * @param  array $existing_columns
   * @return array
   */
  public function clipart_columns( $existing_columns ) {
    if ( empty( $existing_columns ) && ! is_array( $existing_columns ) ) {
      $existing_columns = array();
    }

    unset( $existing_columns['title'], $existing_columns['comments'], $existing_columns['date'], $existing_columns['author'] );

    $columns          = array();
    $columns['cb']    = '<input type="checkbox" />';
    $columns['thumb'] = '<span class="wc-image tips" data-tip="' . esc_attr__( 'Image', 'vam-udesign' ) . '">' . __( 'Image', 'vam-udesign' ) . '</span>';
    $columns['name']  = __( 'Name', 'vam-udesign' );
    $columns['taxonomy-clipart_cat']  = __( 'Categories', 'vam-udesign' );
    $columns['taxonomy-clipart_tag']  = __( 'Tags', 'vam-udesign' );
    $columns['date']         = __( 'Date', 'vam-udesign' );

    return array_merge( $columns, $existing_columns );
  }

  /**
   * Ouput custom columns for designs.
   *
   * @param string $column
   */
  public function render_design_columns( $column ) {
    global $post, $the_design;

    if ( empty( $the_design ) || $the_design->get_id() != $post->ID ) {
      $the_design = vam_udesign_get_design( $post );
    }

    // Only continue if we have a design.
    if ( empty( $the_design ) ) {
      return;
    }

    switch ( $column ) {
      case 'thumb' :
        echo '<a href="' . get_edit_post_link( $post->ID ) . '"><img src="' . $the_design->get_thumbnail_image() . '" style="background-color:' . $the_design->get_background_color() . ';"></a>';
        break;
      break;
      case 'name' :
        $edit_link = get_edit_post_link( $post->ID );
        $title     = _draft_or_post_title();

        echo '<strong><a class="row-title" href="' . esc_url( $edit_link ) . '">' . esc_html( $title ) . '</a>';

        _post_states( $post );

        echo '</strong>';

        if ( $post->post_parent > 0 ) {
          echo '&nbsp;&nbsp;&larr; <a href="'. get_edit_post_link( $post->post_parent ) .'">'. get_the_title( $post->post_parent ) .'</a>';
        }

        // Excerpt view
        if ( isset( $_GET['mode'] ) && 'excerpt' == $_GET['mode'] ) {
          echo apply_filters( 'the_excerpt', $post->post_excerpt );
        }

        get_inline_data( $post );
      break;
      case 'featured' :
        $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=udesign_feature_design&design_id=' . $post->ID ), 'udesign-feature-design' );
        echo '<a href="' . esc_url( $url ) . '" aria-label="' . __( 'Toggle featured', 'vam-udesign' ) . '">';
        if ( $the_design->is_featured() ) {
          echo '<span class="wc-featured tips" data-tip="' . esc_attr__( 'Yes', 'vam-udesign' ) . '">' . __( 'Yes', 'vam-udesign' ) . '</span>';
        } else {
          echo '<span class="wc-featured not-featured tips" data-tip="' . esc_attr__( 'No', 'vam-udesign' ) . '">' . __( 'No', 'vam-udesign' ) . '</span>';
        }
        echo '</a>';
        break;
      default :
        break;
    }
  }

  /**
   * Ouput custom columns for cliparts.
   *
   * @param string $column
   */
  public function render_clipart_columns( $column ) {
    global $post;

    // Only continue if we have a post.
    if ( empty( $post ) ) {
      return;
    }

    switch ( $column ) {
      case 'thumb' :
        echo '<a href="' . get_edit_post_link( $post->ID ) . '">' . get_the_post_thumbnail( $post->ID, 'thumbnail' ) . '</a>';
        break;
      break;
      case 'name' :
        $edit_link = get_edit_post_link( $post->ID );
        $title     = _draft_or_post_title();

        echo '<strong><a class="row-title" href="' . esc_url( $edit_link ) . '">' . esc_html( $title ) . '</a>';

        _post_states( $post );

        echo '</strong>';

        if ( $post->post_parent > 0 ) {
          echo '&nbsp;&nbsp;&larr; <a href="'. get_edit_post_link( $post->post_parent ) .'">'. get_the_title( $post->post_parent ) .'</a>';
        }

        // Excerpt view
        if ( isset( $_GET['mode'] ) && 'excerpt' == $_GET['mode'] ) {
          echo apply_filters( 'the_excerpt', $post->post_excerpt );
        }

        get_inline_data( $post );
      break;
      default :
        break;
    }
  }

  /**
   * Make columns sortable
   *
   * @param  array $columns
   * @return array
   */
  public function design_sortable_columns( $columns ) {
    $custom = array(
      'name'     => 'title'
    );

    return wp_parse_args( $custom, $columns );
  }

  /**
   * Make columns sortable
   *
   * @param  array $columns
   * @return array
   */
  public function clipart_sortable_columns( $columns ) {
    $custom = array(
      'name'     => 'title'
    );

    return wp_parse_args( $custom, $columns );
  }

  /**
   * Set list table primary column for products and orders.
   * Support for WordPress 4.3.
   *
   * @param  string $default
   * @param  string $screen_id
   *
   * @return string
   */
  public function list_table_primary_column( $default, $screen_id ) {
    if ( 'edit-design' === $screen_id ) {
      return 'name';
    }

    if ( 'edit-clipart' === $screen_id ) {
      return 'name';
    }

    return $default;
  }

  /**
   * Get all WooCommerce screen ids.
   *
   * @return array
   */
  public function screen_ids( $screen_ids ) {
    $custom = array(
      'design',
      'edit-design',
      'clipart',
      'edit-clipart',
    );

    return wp_parse_args( $custom, $screen_ids );
  }

  /**
   * Output design featured checkbox.
   */
  public function design_featured() {
    global $post;

    if ( 'design' !== $post->post_type ) {
      return;
    }

    $post_id          = $post->ID;
    $design_object    = $post_id ? vam_udesign_get_design( $post_id ) : new VAM_UDesign_Design();
    $current_featured = $design_object->get_featured() === 'yes' ? 'yes' : 'no';

    echo '<div class="misc-pub-section"><input type="checkbox" name="design_featured" id="design_featured" ' . checked( $current_featured, 'yes', false ) . ' /> <label for="design_featured">' . __( 'Featured design', 'vam-udesign' ) . '</label></div>';
  }

  public function before_delete_post( $post_id ) {
    if ( empty( $post_id ) ) {
      return;
    }

    global $post_type;
    switch ( $post_type ) {
    case 'clipart':
      $this->before_delete_clipart( $post_id );
      break;
    case 'design':
      $this->before_delete_design( $post_id );
      break;
    }
  }

  public function before_delete_clipart( $post_id ) {
    $attachment_ids = array(
      get_post_thumbnail_id( $post_id ),
      get_post_meta( $post_id, '_clipart_image_id', true )
    );

    vam_udesign_delete_attachments( $attachment_ids );
  }

  public function before_delete_design( $post_id ) {
    $attachment_ids = array( get_post_thumbnail_id( $post_id ) );
    vam_udesign_delete_attachments( $attachment_ids );

    $design_views = get_post_meta( $post_id, '_design_thumbnails', true );
    if ( !empty( $design_views ) ) {
      foreach( $design_views as &$design_view ) {
        foreach( $design_view as $url ) {
          vam_udesign_delete_url( $url );
        }
      }
    }
  }
}

new VAM_UDesign_Admin_Post_Types();
