<?php
/**
 * WooCommerce API Settings
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Admin_Settings_Designs.
 */
class VAM_UDesign_Settings_Designs extends VAM_UDesign_Settings_Page {

  /**
   * Constructor.
   */
  public function __construct() {

    $this->id    = 'designs';
    $this->label = __( 'Designs', 'vam-udesign' );

    add_filter( 'vam_udesign_settings_tabs_array', array( $this, 'add_settings_page' ), 30 );
    add_action( 'vam_udesign_settings_' . $this->id, array( $this, 'output' ) );
    add_action( 'vam_udesign_settings_save_' . $this->id, array( $this, 'save' ) );
  }

  /**
   * Get settings array.
   *
   * @return array
   */
  public function get_settings() {
    $settings = array(
      array(
        'title'   => __( 'Archive Pages', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'archive_pages',
      ),

      array(
        'title'    => __( 'Archive Page', 'vam-udesign' ),
        'id'       => 'udesign_designs_page_id',
        'type'     => 'single_select_page',
        'default'  => '',
        'class'    => 'enhanced-select',
        'css'      => 'min-width:350px;',
        'desc_tip' => __( 'Default archive page for designs.', 'vam-udesign' ),
      ),

      array( 'type' => 'sectionend', 'id' => 'archive_pages' ),

      array(
        'title'   => __( 'Design Pages', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'design_pages',
      ),

      array(
        'title'   => __( 'Products Layout', 'vam-udesign' ),
        'desc'    => __( 'This changes the layout of the products list on the single design page.', 'vam-udesign' ),
        'id'      => 'udesign_products_list_layout',
        'default' => 'carousel',
        'type'    => 'radio',
        'options' => array(
          'carousel'     => __( 'Default to carousel layout.', 'vam-udesign' ),
          'masonry'      => __( 'Default to masonry layout.', 'vam-udesign' ),
        ),
        'desc_tip'        =>  true,
        'show_if_checked' => 'option',
      ),

      array( 'type' => 'sectionend', 'id' => 'design_pages' ),

      array(
        'title' => __( 'Design Images', 'vam-udesign' ),
        'type'  => 'title',
        'desc'  => __( 'These settings affect the display and dimensions of images in your catalog - the display on the front-end will still be affected by CSS styles.', 'vam-udesign' ),
        'id'    => 'design_image_options'
      ),

      array(
        'title'    => __( 'Catalog Images', 'vam-udesign' ),
        'desc'     => __( 'This size is usually used in design listings', 'vam-udesign' ),
        'id'       => 'udesign_design_catalog_image_size',
        'css'      => '',
        'type'     => 'image_width',
        'default'  => array(
          'width'  => '200',
          'height' => '260',
          'crop'   => 1
        ),
        'desc_tip' =>  true,
      ),

      array(
        'title'    => __( 'Single Design Image', 'vam-udesign' ),
        'desc'     => __( 'This is the size used by the main image on the design page.', 'vam-udesign' ),
        'id'       => 'udesign_design_single_image_size',
        'css'      => '',
        'type'     => 'image_width',
        'default'  => array(
          'width'  => '360',
          'height' => '460',
          'crop'   => 1
        ),
        'desc_tip' =>  true,
      ),

      array(
        'title'    => __( 'Design Thumbnails', 'vam-udesign' ),
        'desc'     => __( 'This size is usually used for the gallery of images on the design page.', 'vam-udesign' ),
        'id'       => 'udesign_design_thumbnail_image_size',
        'css'      => '',
        'type'     => 'image_width',
        'default'  => array(
          'width'  => '200',
          'height' => '260',
          'crop'   => 1
        ),
        'desc_tip' =>  true,
      ),

      array( 'type' => 'sectionend', 'id' => 'design_image_options' ),
    );

    return $settings;
  }

  /**
   * Save settings.
   */
  public function save() {
    $settings = $this->get_settings();

    VAM_UDesign_Admin_Settings::save_fields( $settings );
  }

}

return new VAM_UDesign_Settings_Designs();
