<?php
/**
 * WooCommerce General Settings
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Admin_Settings_General.
 */
class VAM_UDesign_Settings_General extends VAM_UDesign_Settings_Page {

  /**
   * Constructor.
   */
  public function __construct() {

    $this->id    = 'general';
    $this->label = __( 'General', 'vam-udesign' );

    add_filter( 'vam_udesign_settings_tabs_array', array( $this, 'add_settings_page' ), 10 );
    add_action( 'vam_udesign_settings_' . $this->id, array( $this, 'output' ) );
    add_action( 'vam_udesign_settings_save_' . $this->id, array( $this, 'save' ) );
  }

  /**
   * Get settings array.
   *
   * @return array
   */
  public function get_settings() {
    $products       = array(0 => __( 'Select a default designer product&hellip;', 'vam-udesign' ) );
    $products_query = vam_udesign_get_products( array(
      'designer_products' => true
    ) );

    foreach ( $products_query->posts as $key => $product_id ) {
      $products[$product_id]   = get_the_title( $product_id );
    }

    $settings = array(
      array(
        'title'   => __( 'API', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'api_options',
      ),

      array(
        'title'    => __( 'API Key', 'vam-udesign' ),
        'desc'     => __( 'Product Designer API Key.', 'vam-udesign' ),
        'id'       => 'udesign_api_key',
        'css'      => 'min-width:350px;',
        'default'  => '',
        'type'     => 'text',
        'desc_tip' =>  true,
      ),

      array( 'type' => 'sectionend', 'id' => 'api_options' ),

      array(
        'title'   => __( 'Designer Pages', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'designer_pages',
      ),

      array(
        'title'    => __( 'Designer Page', 'vam-udesign' ),
        'id'       => 'udesign_designer_page_id',
        'type'     => 'single_select_page',
        'default'  => '',
        'class'    => 'enhanced-select',
        'css'      => 'min-width:350px;',
        'desc_tip' => __( 'Page where the product designer shortcode is inserted.', 'vam-udesign' ),
      ),

      array( 'type' => 'sectionend', 'id' => 'designer_pages' ),

      array(
        'title'   => __( 'Designer Products', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'designer_products',
      ),

      array(
        'title'    => __( 'Default Product', 'vam-udesign' ),
        'desc'     => __( 'Default product to show in designer.', 'vam-udesign' ),
        'id'       => 'udesign_default_product_id',
        'css'      => 'min-width:350px;',
        'default'  => '',
        'type'     => 'select',
        'class'    => 'enhanced-select',
        'desc_tip' =>  true,
        'options'  => $products
      ),

      array( 'type' => 'sectionend', 'id' => 'designer_products' ),

      array(
        'title'   => __( 'Designer Options', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'designer_options',
      ),

      array(
        'title'      => __( 'Customer Clipart Upload', 'vam-udesign' ),
        'desc'       => __( 'Enable/Disable designer clipart upload for customers.', 'vam-udesign' ),
        'id'         => 'udesign_customer_clipart_upload',
        'css'        => 'min-width:350px;',
        'default'    => 'enabled',
        'type'       => 'select',
        'class'      => 'enhanced-select',
        'desc_tip'   =>  true,
        'options'    =>  array(
          'enabled'  => __( 'Enabled', 'vam-udesign' ),
          'disabled' => __( 'Disabled', 'vam-udesign' ),
        ),
      ),

      array(
        'title'      => __( 'Save Design for Later', 'vam-udesign' ),
        'desc'       => __( 'Enable/Disable customer save design for later functionality.', 'vam-udesign' ),
        'id'         => 'udesign_customer_save_design_for_later',
        'css'        => 'min-width:350px;',
        'default'    => 'enabled',
        'type'       => 'select',
        'class'      => 'enhanced-select',
        'desc_tip'   =>  true,
        'options'    =>  array(
          'enabled'  => __( 'Enabled', 'vam-udesign' ),
          'disabled' => __( 'Disabled', 'vam-udesign' ),
        ),
      ),

      array(
        'title'      => __( 'Footer Branding', 'vam-udesign' ),
        'desc'       => __( 'Show/Hide powered by uDesign message.', 'vam-udesign' ),
        'id'         => 'udesign_branding_message',
        'css'        => 'min-width:350px;',
        'default'    => 'enabled',
        'type'       => 'select',
        'class'      => 'enhanced-select',
        'desc_tip'   =>  true,
        'options'    =>  array(
          'enabled'  => __( 'Enabled', 'vam-udesign' ),
          'disabled' => __( 'Disabled', 'vam-udesign' ),
        ),
      ),

      array( 'type' => 'sectionend', 'id' => 'designer_options' ),

      array(
        'title'   => __( 'Cart Options', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'cart_options',
      ),

      array(
        'title'       => __( 'Add to Cart Style', 'vam-udesign' ),
        'desc'        => __( 'The type of add to cart dialog to show on the designer page.', 'vam-udesign' ),
        'id'          => 'udesign_add_to_cart_style',
        'css'         => 'min-width:350px;',
        'default'     => 'bulk',
        'type'        => 'select',
        'class'       => 'enhanced-select',
        'desc_tip'    =>  true,
        'options'     =>  array(
          'classic'   => __( 'Classic', 'vam-udesign' ),
          'bulk'      => __( 'Bulk', 'vam-udesign' ),
        ),
      ),

      array( 'type' => 'sectionend', 'id' => 'cart_options' ),
    );

    return $settings;
  }

  /**
   * Save settings.
   */
  public function save() {
    $settings = $this->get_settings();

    VAM_UDesign_Admin_Settings::save_fields( $settings );
  }

}

return new VAM_UDesign_Settings_General();
