<?php
/**
 * WooCommerce Settings Page/Tab
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Settings_Page.
 */
abstract class VAM_UDesign_Settings_Page {

  /**
   * Setting page id.
   *
   * @var string
   */
  protected $id = '';

  /**
   * Setting page label.
   *
   * @var string
   */
  protected $label = '';

  /**
   * Constructor.
   */
  public function __construct() {
    add_action( 'vam_udesign_settings_' . $this->id, array( $this, 'output' ) );
    add_action( 'vam_udesign_settings_save_' . $this->id, array( $this, 'save' ) );
  }

  /**
   * Add this page to settings.
   */
  public function add_settings_page( $pages ) {
    $pages[ $this->id ] = $this->label;

    return $pages;
  }

  /**
   * Get settings array.
   *
   * @return array
   */
  public function get_settings() {
    return apply_filters( 'vam_udesign_get_settings_' . $this->id, array() );
  }

  /**
   * Output the settings.
   */
  public function output() {
    $settings = $this->get_settings();

    VAM_UDesign_Admin_Settings::output_fields( $settings );
  }

  /**
   * Save settings.
   */
  public function save() {
    $settings = $this->get_settings();
    VAM_UDesign_Admin_Settings::save_fields( $settings );

    do_action( 'vam_udesign_update_options_' . $this->id );
  }
}
