<?php
/**
 * WooCommerce API Settings
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Admin_Settings_Products.
 */
class VAM_UDesign_Settings_Products extends VAM_UDesign_Settings_Page {

  /**
   * Constructor.
   */
  public function __construct() {

    $this->id    = 'products';
    $this->label = __( 'Products', 'vam-udesign' );

    add_filter( 'vam_udesign_settings_tabs_array', array( $this, 'add_settings_page' ), 20 );
    add_action( 'vam_udesign_settings_' . $this->id, array( $this, 'output' ) );
    add_action( 'vam_udesign_settings_save_' . $this->id, array( $this, 'save' ) );
  }

  /**
   * Get settings array.
   *
   * @return array
   */
  public function get_settings() {
    $settings = array(
      array(
        'title'   => __( 'Remote API', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'remote_products_api_options',
      ),

      array(
        'title'    => __( 'Consumer Key', 'vam-udesign' ),
        'desc'     => __( 'Products API consumer key.', 'vam-udesign' ),
        'id'       => 'udesign_remote_products_api_consumer_key',
        'css'      => 'min-width:350px;',
        'default'  => '',
        'type'     => 'text',
        'desc_tip' =>  true,
      ),

      array(
        'title'    => __( 'Consumer Secret', 'vam-udesign' ),
        'desc'     => __( 'Products API consumer secret.', 'vam-udesign' ),
        'id'       => 'udesign_remote_products_api_consumer_secret',
        'css'      => 'min-width:350px;',
        'default'  => '',
        'type'     => 'text',
        'desc_tip' =>  true,
      ),

      array( 'type' => 'sectionend', 'id' => 'remote_products_api_options' ),

      array(
        'title'   => __( 'Local API', 'vam-udesign' ),
        'type'    => 'title',
        'id'      => 'local_products_api_options',
      ),

      array(
        'title'    => __( 'Consumer Key', 'vam-udesign' ),
        'desc'     => __( 'Products API consumer key.', 'vam-udesign' ),
        'id'       => 'udesign_local_products_api_consumer_key',
        'css'      => 'min-width:350px;',
        'default'  => '',
        'type'     => 'text',
        'desc_tip' =>  true,
      ),

      array(
        'title'    => __( 'Consumer Secret', 'vam-udesign' ),
        'desc'     => __( 'Products API consumer secret.', 'vam-udesign' ),
        'id'       => 'udesign_local_products_api_consumer_secret',
        'css'      => 'min-width:350px;',
        'default'  => '',
        'type'     => 'text',
        'desc_tip' =>  true,
      ),

      array( 'type' => 'sectionend', 'id' => 'local_products_api_options' ),
    );

    return $settings;
  }

  /**
   * Save settings.
   */
  public function save() {
    $settings = $this->get_settings();

    VAM_UDesign_Admin_Settings::save_fields( $settings );
  }

}

return new VAM_UDesign_Settings_Products();
