<?php
/**
 * Clipart Meta Boxes
 *
 * Registers clipart post type metaboxes.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Clipart_Meta_Boxes Class.
 */
class VAM_UDesign_Clipart_Meta_Boxes {
  /**
   * Is meta boxes saved once?
   *
   * @var boolean
   */
  private static $saved_meta_boxes = false;

  /**
   * Meta box error messages.
   *
   * @var array
   */
  public static $meta_box_errors  = array();

  /**
   * Constructor.
   */
  public function __construct() {
    add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ), 30 );
    add_action( 'save_post', array( $this, 'save_meta_boxes' ), 1, 2 );
  }

  /**
   * Add Clipart Meta boxes.
   */
  public static function add_meta_boxes() {
    add_meta_box( 'clipart-image', __( 'Clipart for Product Designer', 'vam-udesign' ), __CLASS__ . '::clipart_image_meta_box', 'clipart', 'normal', 'high' );
  }

  public static function clipart_image_meta_box() {
    include 'partials/html-clipart-image-meta-box.php';
  }

  /**
   * Check if we're saving, the trigger an action based on the post type.
   *
   * @param  int $post_id
   * @param  object $post
   */
  public function save_meta_boxes( $post_id, $post ) {
    // $post_id and $post are required
    if ( empty( $post_id ) || empty( $post ) || self::$saved_meta_boxes ) {
      return;
    }

    // Dont' save meta boxes for revisions or autosaves
    if ( defined( 'DOING_AUTOSAVE' ) || is_int( wp_is_post_revision( $post ) ) || is_int( wp_is_post_autosave( $post ) ) ) {
      return;
    }

    // Check the post being saved == the $post_id to prevent triggering this call for other save_post events
    if ( empty( $_POST['post_ID'] ) || $_POST['post_ID'] != $post_id ) {
      return;
    }

    // Check user has permission to edit
    if ( ! current_user_can( 'edit_post', $post_id ) ) {
      return;
    }

    self::$saved_meta_boxes = true;

    // Save meta
    if ( ! empty( $_POST['_clipart_image_id'] ) ) {
      update_post_meta( $post_id, '_clipart_image_id', absint( $_POST['_clipart_image_id'] ) );
    }
  }
}

new VAM_UDesign_Clipart_Meta_Boxes();
