<?php
/**
 * Design Import Upload
 *
 * Registers design import metaboxes.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Clipart_Import Class.
 */
class VAM_UDesign_Design_Import {

  /**
   * Constructor.
   */
  public function __construct() {
    add_action( 'admin_menu', array( $this, 'add_design_import_submenu_page' ), 30 );
  }

  /**
   * Add Clipart bulk upload page.
   */
  public static function add_design_import_submenu_page() {
    add_submenu_page( 'edit.php?post_type=design', __( 'Import', 'vam-udesign' ), __( 'Import', 'vam-udesign' ), 'manage_options', 'design_import',  __CLASS__ . '::design_import_page' );
  }

  public static function design_import_page() {
    $post = new WP_Post( (object) array(
      'post_type' => 'design'
    ) );

    // get api key
    $api_key = get_option( 'udesign_api_key', '' );
    $base_params = array(
      'api_key' => $api_key
    );

    // get design categories
    $design_categories = self::get_design_categories( $base_params );
    if ( empty( $design_categories ) ) {
      $design_categories = array();
    }

    // get design dates
    $design_dates = self::get_design_dates( $base_params );
    if ( empty( $design_dates ) ) {
      $design_dates = array();
    }

    // get design list
    $query_params = array_merge( self::parse_query_params(), $base_params );

    $design_list = self::get_design_list( $query_params );
    if ( empty( $design_list ) ) {
      $design_list = array(
        'items' => 0,
        'total_items' => 0,
        'total_pages' => 0
      );
    }

    include 'partials/html-design-import-page.php';
  }

  public static function parse_query_params() {
    $query_params = array();

    parse_str( $_SERVER['QUERY_STRING'], $query_params );
    if ( empty( $query_params ) ) {
      $query_params = array();
    }

    $params = array(
      'paged' => 1,
      'category_id' => 0,
      'month' => 0,
      'query' => '',
      'orderby' => 'date',
      'order' => 'DESC',
      'limit' => 25
    );

    foreach( $query_params as $key => $value ) {
      switch ( $key ) {
      case 'paged':
        $params['paged'] = absint( $value );
        if ( empty( $params['paged'] ) ) {
          $params['paged'] = 1;
        }
        break;

      case 'category-filter':
        $params['category_id'] = $value;
        break;

      case 'm':
        $params['month'] = $value;
        break;

      case 's':
        $params['query'] = $value;
        break;

      case 'posts_per_page':
        $params['limit'] = $value;

      default:
        continue;
      }
    }

    return $params;
  }

  public static function get_design_list( $args = array() ) {
    $url = sprintf( '%s/%s?%s', VAM_UDESIGN_API_URL, 'designs', http_build_query( $args ) );
    return vam_udesign_remote_request( $url, $args, true );
  }

  public static function get_design_categories( $args = array() ) {
    $url = sprintf( '%s/%s?%s', VAM_UDESIGN_API_URL, 'designs/categories', http_build_query( $args ) );
    return vam_udesign_remote_request( $url, $args, true );
  }

  public static function get_design_dates( $args = array() ) {
    $url = sprintf( '%s/%s?%s', VAM_UDESIGN_API_URL, 'designs/dates', http_build_query( $args ) );
    return vam_udesign_remote_request( $url, $args, true );
  }

  /**
   * Check if we're saving, the trigger an action based on the post type.
   *
   * @param  int $post_id
   * @param  object $post
   */
  public function process_design_import() {
  }

}

new VAM_UDesign_Design_Import();
