<?php
/**
 * Design Meta Boxes
 *
 * Registers design post type metaboxes.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/admin
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Design_Meta_Boxes Class.
 */
class VAM_UDesign_Design_Meta_Boxes {
  /**
   * Is meta boxes saved once?
   *
   * @var boolean
   */
  private static $saved_meta_boxes = false;

  /**
   * Meta box error messages.
   *
   * @var array
   */
  public static $meta_box_errors  = array();

  /**
   * Constructor.
   */
  public function __construct() {
    add_action( 'add_meta_boxes', array( $this, 'remove_meta_boxes' ), 10 );
    add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ), 20 );
    add_action( 'save_post', array( $this, 'save_meta_boxes' ), 1, 2 );
  }

  /**
   * Add Design Meta boxes.
   */
  public static function add_meta_boxes() {
    add_meta_box( 'udesign-designer', __( 'Designer', 'vam-udesign' ), __CLASS__ . '::designer_meta_box', 'design', 'normal', 'high' );
    add_meta_box( 'udesign-designer-excluded-colors', __( 'Design Excluded Colors', 'vam-udesign' ), __CLASS__ . '::design_excluded_colors_meta_box', 'design', 'normal', 'high' );
    add_meta_box( 'udesign-design-options', __( 'Design Options', 'vam-udesign' ), __CLASS__ . '::design_options_meta_box', 'design', 'side', 'default' );
  }

  public static function designer_meta_box() {
    global $post;

    include 'partials/html-design-designer-meta-box.php';
  }

  public static function design_excluded_colors_meta_box() {
    global $post;

    include 'partials/html-design-excluded-colors-meta-box.php';
  }

  public static function design_options_meta_box() {
    global $post;

    include 'partials/html-design-options-meta-box.php';
  }

  /**
   * Remove metaboxes.
   */
  public function remove_meta_boxes() {
  }

  /**
   * Check if we're saving, the trigger an action based on the post type.
   *
   * @param  int $post_id
   * @param  object $post
   */
  public function save_meta_boxes( $post_id, $post ) {
    // $post_id and $post are required
    if ( empty( $post_id ) || empty( $post ) || self::$saved_meta_boxes ) {
      return;
    }

    // Dont' save meta boxes for revisions or autosaves
    if ( defined( 'DOING_AUTOSAVE' ) || is_int( wp_is_post_revision( $post ) ) || is_int( wp_is_post_autosave( $post ) ) ) {
      return;
    }

    // Check the post being saved == the $post_id to prevent triggering this call for other save_post events
    if ( empty( $_POST['post_ID'] ) || $_POST['post_ID'] != $post_id ) {
      return;
    }

    // Check user has permission to edit
    if ( ! current_user_can( 'edit_post', $post_id ) ) {
      return;
    }

    self::$saved_meta_boxes = true;

    // Save meta
    if ( !empty( $_POST['design_meta'] ) ) {
      update_post_meta( $post_id, '_design_meta', $_POST['design_meta'] );
    }

    if ( !empty( $_POST['design_default_product_id'] ) ) {
      update_post_meta( $post_id, '_design_default_product_id', $_POST['design_default_product_id'] );
    }

    if ( !empty( $_POST['design_background_color'] ) ) {
      update_post_meta( $post_id, '_design_background_color', $_POST['design_background_color'] );
    }

    update_post_meta( $post_id, '_design_excluded_colors', empty( $_POST['design_excluded_colors'] ) ? null : $_POST['design_excluded_colors'] );
    update_post_meta( $post_id, '_design_featured', empty( $_POST['design_featured'] ) ? 'no' : 'yes' );
    update_post_meta( $post_id, '_design_disable_customize', empty( $_POST['design_disable_customize'] ) ? 0 : 1 );
  }
}

new VAM_UDesign_Design_Meta_Boxes();
