<?php
/**
 * VAM_UDesign_WC_Meta_Box_Product_Data class
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_WC_Meta_Box_Product_Data Class.
 */
class VAM_UDesign_WC_Meta_Box_Product_Data {

  /**
   * Constructor.
   */
  public function __construct() {
    add_filter( 'product_type_options', array( $this, 'vam_udesign_product_type_options' ), 10, 1 );
    add_filter( 'woocommerce_product_data_tabs', array( $this, 'vam_udesign_woocommerce_product_data_tabs' ), 10, 1 );
    add_action( 'woocommerce_product_data_panels', array( $this, 'vam_udesign_woocommerce_product_data_panels' ));
    add_action( 'woocommerce_process_product_meta_variable', array( $this, 'save' ), 10, 1 );
  }

  public static function vam_udesign_product_type_options( $product_type_options ) {
    return (array) wp_parse_args( array(
      'designer' => array(
        'id'            => '_designer',
        'wrapper_class' => 'show_if_variable',
        'label'         => __( 'Designer Product', 'vam-udesign' ),
        'description'   => __( 'Designer products are variable product with customization options.', 'vam-udesign' ),
        'default'       => 'no'
      )
    ), $product_type_options );
  }

  public static function vam_udesign_woocommerce_product_data_tabs( $product_data_tabs ) {
    return (array) wp_parse_args( array(
      'designer' => array(
        'label'  => __( 'Designer', 'vam-udesign' ),
        'target' => 'designer_product_data',
        'class'  => array( 'designer_views_tab', 'show_if_designer' ),
      )
    ), $product_data_tabs );
  }

  public static function vam_udesign_woocommerce_product_data_panels() {
    global $post, $thepostid;

    ?>
    <div id="designer_product_data" class="panel wc-metaboxes-wrapper hidden">
      <div id="designer_product_data_inner">

        <div class="toolbar toolbar-top">
          <select id="designer_field_to_edit" class="designer_views_actions">
            <option data-global="true" value="link_all_designer_variations" selected><?php _e( 'Create designer variations from all attributes', 'vam-udesign' ); ?></option>
          </select>
          <a class="button bulk_edit do_designer_views_action"><?php _e( 'Go', 'vam-udesign' ); ?></a>
          <div class="designer-variations-pagenav">
            <span class="displaying-num"><?php printf( _n( '%s view', '%s views', 0, 'vam-udesign' ), 0 ); ?></span>
            <span class="expand-close">
              (<a href="#" class="expand_all"><?php _e( 'Expand', 'vam-udesign' ); ?></a> / <a href="#" class="close_all"><?php _e( 'Close', 'vam-udesign' ); ?></a>)
            </span>
          </div>
          <div class="clear"></div>
        </div>

        <div class="designer_views wc-metaboxes" data-attributes="<?php
            $attributes = maybe_unserialize( get_post_meta( $thepostid, '_product_attributes', true ) );
            // esc_attr does not double encode - htmlspecialchars does
            echo htmlspecialchars( json_encode( $attributes ) );
          ?>" data-total="0" data-total_pages="1" data-page="1" data-edited="false">
        </div>

        <div class="toolbar">
          <button type="button" class="button-primary save-designer-views-changes" disabled="disabled"><?php _e( 'Save changes', 'vam-udesign' ); ?></button>
          <button type="button" class="button cancel-designer-views-changes" disabled="disabled"><?php _e( 'Cancel', 'vam-udesign' ); ?></button>
          <div class="designer-variations-pagenav">
            <span class="displaying-num"><?php printf( _n( '%s view', '%s views', 0, 'vam-udesign' ), 0 ); ?></span>
            <span class="expand-close">
              (<a href="#" class="expand_all"><?php _e( 'Expand', 'vam-udesign' ); ?></a> / <a href="#" class="close_all"><?php _e( 'Close', 'vam-udesign' ); ?></a>)
            </span>
          </div>
          <div class="clear"></div>
        </div>

      </div>
    </div>
    <?php
  }

  public static function save( $post_id ) {

    $is_designer = isset( $_POST['_designer'] ) ? 'yes' : 'no';
    update_post_meta( $post_id, '_designer', $is_designer );

    if ( isset( $_POST['_vam_udesign_views'] ) ) {
      $views = $_POST['_vam_udesign_views'];

      update_post_meta( $post_id, '_vam_udesign_views', $views );
    }

  }

}

new VAM_UDesign_WC_Meta_Box_Product_Data();
