<?php
/**
 * VAM_UDesign_WC_Product_Attribute_Swatches class
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_WC_Product_Attribute_Swatches Class.
 */
class VAM_UDesign_WC_Product_Attribute_Swatches {

  private $taxonomy;
  private $meta_key     = 'swatches_id';
  private $image_size   = 'swatches_image_size';
  private $image_width  = 26;
  private $image_height = 26;

  /**
   * Constructor.
   */
  public function __construct() {
    add_filter( 'product_attributes_type_selector', array( $this, 'vam_udesign_wc_product_attributes_type_selector' ), 10, 1 );
    add_action( 'woocommerce_product_option_terms', array( $this,'vam_udesign_woocommerce_product_option_terms' ), 10, 2 );

    if ( is_admin() ) {
      add_action( 'current_screen', array( $this, 'init_attribute_image_selector' ) );
      add_action( 'created_term', array( $this, 'woocommerce_attribute_thumbnail_field_save' ), 10, 3 );
      add_action( 'edit_term', array( $this, 'woocommerce_attribute_thumbnail_field_save' ), 10, 3 );
    }

    add_action( 'admin_init', array( $this, 'on_admin_init') );
  }

  /**
   * Add Custom product attribute type
   *
   * @param array $types
   * @since 1.0.0
   */
  public function vam_udesign_wc_product_attributes_type_selector( $types ) {
    return (array) wp_parse_args( array(
      'custom'   => __( 'Custom', 'vam-udesign' ),
    ), $types );
  }

  /**
   * Add Custom product attribute selector
   *
   * @param $attribute_taxonomy
   * @param $i
   * @since 1.0.0
   */
  public function vam_udesign_woocommerce_product_option_terms( $attribute_taxonomy, $i ) {
    global $thepostid;
    $taxonomy = wc_attribute_taxonomy_name( $attribute_taxonomy->attribute_name );

    if ( 'custom' === $attribute_taxonomy->attribute_type ) {
    ?>
      <select multiple="multiple" data-placeholder="<?php esc_attr_e( 'Select terms', 'vam-udesign' ); ?>" class="multiselect attribute_values wc-enhanced-select" name="attribute_values[<?php echo $i; ?>][]">
        <?php
        $args = array(
          'orderby'    => 'id',
          'hide_empty' => 0
        );
        $all_terms = get_terms( $taxonomy, apply_filters( 'woocommerce_product_attribute_terms', $args ) );

        if ( $all_terms ) {
          foreach ( $all_terms as $term ) {
            $term_value = VAM_UDesign_WC_Compat::is_wc_version_gte( '3.0.0' ) ? esc_attr( $term->term_id ) : esc_attr( $term->slug );
            echo '<option value="' . $term_value . '" ' . selected( has_term( absint( $term->term_id ), $taxonomy, $thepostid ), true, false ) . '>' . esc_attr( apply_filters( 'woocommerce_product_attribute_term_name', $term->name, $term ) ) . '</option>';
          }
        }
        ?>
      </select>
      <button type="button" class="button plus select_all_attributes"><?php _e( 'Select all', 'vam-udesign' ); ?></button>
      <button type="button" class="button minus select_no_attributes"><?php _e( 'Select none', 'vam-udesign' ); ?></button>
      <button type="button" class="button fr plus add_new<?php echo ( 'pa_color' === $taxonomy ) ? '_color' : ''; ?>_attribute"><?php _e( 'Add new', 'vam-udesign' ); ?></button>
    <?php
    }
  }

  public function on_admin_init() {
    if (isset($_REQUEST['taxonomy'])){
      $this->taxonomy = $_REQUEST['taxonomy'];
    }

    $attribute_taxonomies = wc_get_attribute_taxonomies();
    if ( $attribute_taxonomies ) {
      foreach ( $attribute_taxonomies as $tax ) {
        add_action( 'pa_' . $tax->attribute_name . '_add_form_fields', array( $this, 'woocommerce_add_attribute_thumbnail_field' ) );
        add_action( 'pa_' . $tax->attribute_name . '_edit_form_fields', array( $this, 'woocommerce_edit_attributre_thumbnail_field' ), 10, 2 );
        add_filter( 'manage_edit-pa_' . $tax->attribute_name . '_columns', array( $this, 'woocommerce_product_attribute_columns' ) );
        add_filter( 'manage_pa_' . $tax->attribute_name . '_custom_column', array( $this, 'woocommerce_product_attribute_column' ), 10, 3 );
      }
    }
  }

  //Initalize the actions for all product attribute taxonomoies
  public function init_attribute_image_selector() {
    global $_wp_additional_image_sizes;
    $screen = get_current_screen();

    if ( strpos( $screen->id, 'pa_' ) !== false ) {
      if ( taxonomy_exists( $_REQUEST['taxonomy'] ) ) {
        $term_id = term_exists( isset( $_REQUEST['tag_ID'] ) ? $_REQUEST['tag_ID'] : 0, $_REQUEST['taxonomy'] );
        $term = 0;
        if ( $term_id ) {
          $term = get_term( $term_id, $_REQUEST['taxonomy'] );
        }

        $this->image_size = apply_filters( 'vam_udesign_swatch_image_size', $this->image_size, $_REQUEST['taxonomy'], $term_id );
      }

      $the_size = isset( $_wp_additional_image_sizes[$this->image_size] ) ? $_wp_additional_image_sizes[$this->image_size] : $_wp_additional_image_sizes['shop_thumbnail'];

      if ( isset( $the_size['width'] ) && isset( $the_size['height'] ) ) {
        $this->image_width = $the_size['width'];
        $this->image_height = $the_size['height'];
      }
    }
  }

  //The field used when adding a new term to an attribute taxonomy
  public function woocommerce_add_attribute_thumbnail_field() {
    ?>
    <div class="form-field ">
      <label for="product_attribute_swatchtype_<?php echo $this->meta_key; ?>"><?php _e( 'Swatch Type', 'vam-udesign' ); ?></label>
      <select name="product_attribute_meta[<?php echo $this->meta_key; ?>][type]" id="product_attribute_swatchtype_<?php echo $this->meta_key; ?>" class="postform">
        <option value="-1"><?php _e( 'None', 'vam-udesign' ); ?></option>
        <option value="label"><?php _e( 'Label', 'vam-udesign' ); ?></option>
        <option value="color"><?php _e( 'Color', 'vam-udesign' ); ?></option>
        <option value="image"><?php _e( 'Image', 'vam-udesign' ); ?></option>
      </select>
      <script type="text/javascript">
        jQuery(document).ready(function ($) {
          $('#product_attribute_swatchtype_<?php echo $this->meta_key; ?>').change(function() {
            $('.swatch-field-active').hide().removeClass('swatch-field-active');
            $('.swatch-field-' + $(this).val()).slideDown().addClass('swatch-field-active');
          });
        });
      </script>
    </div>
    <div class="form-field swatch-field swatch-field-color section-color-swatch" style="overflow:visible;display:none;">
      <div id="swatch-color" class="<?php echo sanitize_title( $this->meta_key ); ?>-color">
        <label><?php _e( 'Color', 'vam-udesign' ); ?></label>
        <div id="product_attribute_color_<?php echo $this->meta_key; ?>_picker" class="colorSelector"><div></div></div>
        <input class="woo-color"
        id="product_attribute_color_<?php echo $this->meta_key; ?>"
        type="text" class="text"
        name="product_attribute_meta[<?php echo $this->meta_key; ?>][color]"
        value="#ffffff" data-type="colorpicker" />
      </div>
    </div>
    <div class="sub_field form-field swatch-field swatch-field-image" style="overflow:visible;display:none;">
      <div id="swatch-image" class="<?php echo sanitize_title( $this->meta_key ); ?>-image">
        <label><?php _e( 'Thumbnail', 'vam-udesign' ); ?></label>
        <div id="product_attribute_thumbnail_<?php echo $this->meta_key; ?>" style="float:left;margin-right:10px;">
          <img src="<?php echo apply_filters( 'woocommerce_placeholder_img_src', WC()->plugin_url() . '/assets/images/placeholder.png' ); ?>" width="<?php echo $this->image_width; ?>px" height="<?php echo $this->image_height; ?>px" />
        </div>
        <div style="line-height:60px;">
          <input type="hidden"  class="upload_image_id" id="product_attribute_<?php echo $this->meta_key; ?>" name="product_attribute_meta[<?php echo $this->meta_key; ?>][image]" />
          <button type="submit" class="upload_swatch_image_button button"><?php _e( 'Upload/Add image', 'vam-udesign' ); ?></button>
          <button type="submit" class="remove_swatch_image_button button"><?php _e( 'Remove image', 'vam-udesign' ); ?></button>
        </div>
        <div class="clear"></div>
      </div>
    </div>
    <?php
  }

  //The field used when editing an existing product attribute taxonomy term
  public function woocommerce_edit_attributre_thumbnail_field( $term, $taxonomy ) {
    $swatch_term = new VAM_UDesign_WC_Swatch_Term( $term->term_id, $taxonomy, false, $this->image_size );
    ?>
    <tr class="form-field ">
      <th scope="row" valign="top"><label for="product_attribute_swatchtype_<?php echo $this->meta_key; ?>"><?php _e( 'Swatch Type', 'vam-udesign' ); ?></label></th>
      <td>
        <select name="product_attribute_meta[<?php echo $this->meta_key; ?>][type]" id="product_attribute_swatchtype_<?php echo $this->meta_key; ?>" class="postform">
          <option <?php selected( 'none', $swatch_term->get_type() ); ?> value="-1"><?php _e( 'None', 'vam-udesign' ); ?></option>
          <option <?php selected( 'label', $swatch_term->get_type() ); ?> value="label"><?php _e( 'Label', 'vam-udesign' ); ?></option>
          <option <?php selected( 'color', $swatch_term->get_type() ); ?> value="color"><?php _e( 'Color', 'vam-udesign' ); ?></option>
          <option <?php selected( 'image', $swatch_term->get_type() ); ?> value="image"><?php _e( 'Image', 'vam-udesign' ); ?></option>
        </select>
        <script type="text/javascript">
          jQuery(document).ready(function ($) {
            $('#product_attribute_swatchtype_<?php echo $this->meta_key; ?>').change(function() {
              $('.swatch-field-active').hide().removeClass('swatch-field-active');
              $('.swatch-field-' + $(this).val()).show().addClass('swatch-field-active');
            });
          });
        </script>
      </td>
    </tr>
    <?php $class = $swatch_term->get_type() == 'color' ? 'swatch-field-active' : ''; ?>
    <tr class="form-field swatch-field swatch-field-color section-color-swatch <?php echo $class; ?>" style="overflow:visible;<?php echo $class ? '' : 'display:none;'; ?>">
      <th scope="row" valign="top"><label><?php _e( 'Color', 'vam-udesign' ); ?></label></th>
      <td>
        <div id="swatch-color" class="<?php echo sanitize_title( $this->meta_key ); ?>-color">
          <div id="product_attribute_color_<?php echo $this->meta_key; ?>_picker" class="colorSelector"><div></div></div>
          <input class="woo-color"
          id="product_attribute_color_<?php echo $this->meta_key; ?>"
          type="text" class="text"
          name="product_attribute_meta[<?php echo $this->meta_key; ?>][color]"
          value="<?php echo $swatch_term->get_color(); ?>" data-type="colorpicker" />
        </div>
      </td>
    </tr>
    <?php $class = $swatch_term->get_type() == 'image' ? 'swatch-field-active' : ''; ?>
    <tr class="form-field sub_field swatch-field swatch-field-image <?php echo $class; ?>" style="overflow:visible;<?php echo $class ? '' : 'display:none;'; ?>">
      <th scope="row" valign="top"><label><?php _e( 'Image', 'vam-udesign' ); ?></label></th>
      <td>
        <div id="product_attribute_thumbnail_<?php echo $this->meta_key; ?>" style="float:left;margin-right:10px;">
          <img src="<?php echo $swatch_term->get_image_src(); ?>"  width="<?php echo $swatch_term->get_width(); ?>px" height="<?php echo $swatch_term->get_height(); ?>px" />
        </div>
        <div style="line-height:60px;">
          <input class="upload_image_id" type="hidden" id="product_attribute_<?php echo $this->meta_key; ?>" name="product_attribute_meta[<?php echo $this->meta_key; ?>][image]" value="<?php echo $swatch_term->get_image_id(); ?>" />
          <button type="submit" class="upload_swatch_image_button button"><?php _e( 'Upload/Add image', 'vam-udesign' ); ?></button>
          <button type="submit" class="remove_swatch_image_button button"><?php _e( 'Remove image', 'vam-udesign' ); ?></button>
        </div>
        <div class="clear"></div>
      </td>
    </tr>
    <?php
  }

  //Registers a column for this attribute taxonomy for this image
  public function woocommerce_product_attribute_columns( $columns ) {
    $new_columns = array();

    if ( isset( $columns['cb'] ) ) {
      $new_columns['cb'] = $columns['cb'];
      unset( $columns['cb'] );
    }

    $new_columns[$this->meta_key] = __( 'Thumbnail', 'vam-udesign' );

    return array_merge( $new_columns, $columns );
  }

  //Renders the custom column as defined in woocommerce_product_attribute_columns
  public function woocommerce_product_attribute_column( $columns, $column, $id ) {
    if ( $column == $this->meta_key ) {
      $swatch_term = new VAM_UDesign_WC_Swatch_Term( $id, $this->taxonomy, false, $this->image_size );
      $columns .= $swatch_term->get_html();
    }

    return $columns;
  }

  //Saves the product attribute taxonomy term data
  public function woocommerce_attribute_thumbnail_field_save( $term_id, $tt_id, $taxonomy ) {
    if ( isset( $_POST['product_attribute_meta'] ) ) {

      $metas = $_POST['product_attribute_meta'];
      if ( isset( $metas[$this->meta_key] ) ) {
        $data = $metas[$this->meta_key];

        $image = isset( $data['image'] ) ? $data['image'] : '';
        $color = isset( $data['color'] ) ? strtolower( $data['color'] ) : '';
        $type = isset( $data['type'] ) ? strtolower( $data['type'] ) : '';

        update_woocommerce_term_meta( $term_id, $taxonomy . '_' . $this->meta_key . '_type', $type );
        update_woocommerce_term_meta( $term_id, $taxonomy . '_' . $this->meta_key . '_image', $image );
        update_woocommerce_term_meta( $term_id, $taxonomy . '_' . $this->meta_key . '_color', $color );
      }
    } else {
      if ( $taxonomy == 'pa_size' ) {
        update_woocommerce_term_meta( $term_id, $taxonomy . '_' . $this->meta_key . '_type', 'label' );
      }
    }
  }

}

new VAM_UDesign_WC_Product_Attribute_Swatches();
