<?php

class VAM_UDesign_REST_API {

  public static function init() {
    // Clipart methods
    register_rest_route( 'udesign/v1', '/cliparts', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'cliparts' ),
    ) );

    register_rest_route( 'udesign/v1', '/cliparts/(?P<id>\d+)', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'clipart' ),
    ) );

    register_rest_route( 'udesign/v1', '/cliparts/categories', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'clipart_categories' ),
    ) );

    register_rest_route( 'udesign/v1', '/cliparts/dates', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'clipart_dates' ),
    ) );

    // Design methods
    register_rest_route( 'udesign/v1', '/designs', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'designs' ),
    ) );

    register_rest_route( 'udesign/v1', '/designs/(?P<id>\d+)', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'design' ),
    ) );

    register_rest_route( 'udesign/v1', '/designs/categories', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'design_categories' ),
    ) );

    register_rest_route( 'udesign/v1', '/designs/dates', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'design_dates' ),
    ) );

    register_rest_route( 'udesign/v1', '/designs/import', array(
      'methods' => 'POST',
      'callback' => array( __CLASS__, 'import_design' ),
    ) );

    // Product methods
    register_rest_route( 'udesign/v1', '/products', array(
      'methods'  => 'GET',
      'callback' => array( __CLASS__, 'products' ),
    ) );

    register_rest_route( 'udesign/v1', '/products/(?P<id>\d+)', array(
      'methods' => 'GET',
      'callback' => array( __CLASS__, 'product' ),
    ) );

    register_rest_route( 'udesign/v1', '/products/categories', array(
      'methods'  => 'GET',
      'callback' => array( __CLASS__, 'product_categories' ),
    ) );
  }

  public static function auth( $request ) {
    if ( empty( VAM_UDESIGN_REQUIRE_AUTH ) ) {
      return true;
    }

    if ( empty( VAM_UDESIGN_API_USERNAME ) || empty( VAM_UDESIGN_API_PASSWORD ) ) {
      return false;
    }

    $api_key = $request->get_param( 'api_key' );
    if ( empty( $api_key ) ) {
      return false;
    }

    $url = sprintf( 'https:%s/api/licenses/%s', UDESIGN_URL, trim( $api_key ) );
    $args = array(
      'headers' => array(
        'Authorization' => 'Basic ' . base64_encode( VAM_UDESIGN_API_USERNAME . ':' . VAM_UDESIGN_API_PASSWORD )
      )
    );

    $response = vam_udesign_remote_request( $url, $args, true );
    if ( empty( $response['id'] ) || $response['id'] != $api_key ) {
      return false;
    }

    $request['license'] = $response;
    return true;
  }

  public static function verify_license( $request ) {
    // check license
    if ( empty( $request['license'] ) ) {
      return false;
    }
    $license = $request['license'];

    // check license type
    if ( !isset( $license['type'] ) ) {
      return false;
    }

    $license_type = $license['type'];
    if ( empty( $license_type ) ) {
      return false;
    }

    // check license status
    if ( !isset( $license['status'] ) ) {
      return false;
    }

    $license_status = $license['status'];
    if ( !empty( $license_status ) ) {
      return false;
    }

    // check license date
    if ( empty( $license['expires_at'] ) ) {
      return false;
    }

    $expires_at = new DateTime($license['expires_at']);
    if ( empty( $expires_at ) ) {
      return false;
    }

    $now = new DateTime();
    if ( $now->diff($expires_at)->days > 7 && $now > $expires_at ) {
      return false;
    }

    return true;
  }

  public static function error( $message, $status = 500 ) {
    return new WP_Error(
      'udesign-api-error',
      __( $message, 'vam-udesign' ),
      array( 'status' => $status )
    );
  }

  public static function parse_list_query_params( $valid_license, $request ) {
    if ( empty( $valid_license ) ) {
      return self::inactive_license_list_query_params();
    }

    $args = array();

    $category_id = $request->get_param( 'category_id' );
    if ( !empty( $category_id ) ) {
      $args['category_id'] = absint( $category_id );
    }

    $search_query = $request->get_param( 'query' );
    if ( !empty( $search_query ) ) {
      $args['search_query'] = $search_query;
    }

    $limit = $request->get_param( 'limit' );
    if ( !empty( $limit ) ) {
      $args['posts_per_page'] = $limit;
    }

    $page = $request->get_param( 'paged' );
    if ( !empty( $page ) ) {
      $args['paged'] = $page;
    }

    $month = $request->get_param( 'month' );
    if ( !empty( $month ) ) {
      $args['m'] = $month;
    }

    $order_by = $request->get_param( 'orderby' );
    if ( !empty( $order_by ) ) {
      $args['orderby'] = $order_by;
    }

    $order = $request->get_param( 'order' );
    if ( !empty( $order ) ) {
      $args['order'] = $order;
    }

    return $args;
  }

  public static function inactive_license_list_query_params() {
    return array(
      'posts_per_page' => 25,
      'paged' => 1,
      'orderby' => 'date',
      'order' => 'asc',
    );
  }

  public static function list_query_result( $valid_license, &$query, $items ) {
    $found_posts = min( 25, $query->found_posts );
    $total_pages = 1;
    if ( $valid_license ) {
      $found_posts = $query->found_posts;
      $total_pages = $query->max_num_pages ? $query->max_num_pages : 1;
    }

    return array(
      'items' => $items,
      'total_items' => $found_posts,
      'total_pages' => $total_pages,
    );
  }

  public static function object_terms( $args ) {
    $wp_terms = get_terms( $args );

    $terms = array();
    if ( !empty( $wp_terms ) ) {
      foreach( $wp_terms as &$wp_term ) {
        $terms[] = array(
          'id' => $wp_term->term_id,
          'slug' => $wp_term->slug,
          'name' => $wp_term->name
        );
      }
    }

    return $terms;
  }

  public static function cliparts( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }
    $valid_license = !empty( self::verify_license( $request ) );

    global $post;
    $args = self::parse_list_query_params( $valid_license, $request );
    $query = vam_udesign_get_cliparts( $args );
    $cliparts = $query->posts;

    $items = array();
    foreach ( $cliparts as $clipart_id ) {
      $thumb_id = get_post_thumbnail_id( $clipart_id );
      $thumb = wp_get_attachment_image_src( $thumb_id, 'full' );

      if ( ! empty( $thumb ) ) {
        $items[] = array(
          'id' => $clipart_id,
          'name' => get_the_title( $clipart_id ),
          'thumb' => $thumb[0],
        );
      }
    }

    $data = self::list_query_result( $valid_license, $query, $items );
    return new WP_REST_Response( $data );
  }

  public static function clipart( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    // Check parameters
    $clipart_id = $request->get_param( 'id' );
    if ( empty( $clipart_id ) ) {
      return self::error( 'Must provide clipart ID', 400 );
    }

    // Get clipart thumb
    $thumb_id = get_post_thumbnail_id( $clipart_id );
    $thumb = wp_get_attachment_image_src( $thumb_id, 'full' );
    if ( empty( $thumb ) ) {
      return self::error( 'Could not get clipart thumb' );
    }

    // Get clipart image
    $attachment_id = get_post_meta( $clipart_id, '_clipart_image_id', true );
    if ( empty( $attachment_id ) ) {
      return self::error( 'Could not get clipart image id' );
    }

    $image_src = wp_get_attachment_image_src( $attachment_id, 'full' );
    if ( empty( $image_src ) ) {
      return self::error( 'Could not get clipart image' );
    }

    // Get clipart categories
    $categories = self::object_terms( array(
      'taxonomy' => 'clipart_cat',
      'object_ids' => array( $clipart_id )
    ) );

    // Get clipart tags
    $tags = self::object_terms( array(
      'taxonomy' => 'clipart_tag',
      'object_ids' => array( $clipart_id )
    ) );

    // Create result set
    $data = array(
      'id' => $clipart_id,
      'name' => get_the_title( $clipart_id ),
      'src' => $image_src[0],
      'thumb' => $thumb[0],
      'search_field' => get_post_meta( $clipart_id, '_vam_udesign_search_field', true ),
      'categories' => $categories,
      'tags' => $tags
    );

    return new WP_REST_Response( $data );
  }

  public static function clipart_categories( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    return new WP_REST_Response( self::object_terms( array(
      'taxonomy' => 'clipart_cat'
    ) ) );
  }

  public static function clipart_dates( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    return new WP_REST_Response( vam_udesign_post_dates( 'clipart' ) );
  }

  public static function designs( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }
    $valid_license = !empty( self::verify_license( $request ) );

    global $post;
    $args = self::parse_list_query_params( $valid_license, $request );
    $query = vam_udesign_get_designs( $args );
    $designs = $query->posts;

    $items = array();
    foreach ( $designs as $design_id ) {
      $thumb_id = get_post_thumbnail_id( $design_id );
      $thumb = wp_get_attachment_image_src( $thumb_id, 'full' );

      if ( ! empty( $thumb ) ) {
        $items[] = array(
          'id' => $design_id,
          'name' => get_the_title( $design_id ),
          'thumb' => $thumb[0],
          'background' => get_post_meta( $design_id, '_design_background_color', true )
        );
      }
    }

    $data = self::list_query_result( $valid_license, $query, $items );
    return new WP_REST_Response( $data );
  }

  public static function design( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    // Check params
    $design_id = $request->get_param( 'id' );
    if ( empty( $design_id ) ) {
      return self::error( 'Must provide design ID', 400 );
    }

    // Get design thumb
    $thumb = wp_get_attachment_image_src( get_post_thumbnail_id( $design_id ), 'full' );
    if ( empty( $thumb ) ) {
      return self::error( 'Could not get design thumb' );
    }

    // Get design thubms
    $thumbs = get_post_meta( $design_id, '_design_thumbnails', true );
    if ( empty( $thumbs ) ) {
      return self::error( 'Could not get design thumbnails' );
    }

    $views = self::object_terms( array( 'taxonomy' => 'pa_view' ) );
    if ( empty( $views ) ) {
      return self::error( 'Could not get design views' );
    }

    // Get design metadata
    $design_meta = get_post_meta( $design_id, '_design_meta', true );
    if ( empty( $design_meta ) ) {
      return self::error( 'Could not get design metadata' );
    }

    // Get design categories
    $categories = self::object_terms( array(
      'taxonomy' => 'design_cat',
      'object_ids' => array( $design_id )
    ) );

    // Get design tags
    $tags = self::object_terms( array(
      'taxonomy' => 'design_tag',
      'object_ids' => array( $design_id )
    ) );

    // Create result set
    $data = array(
      'id' => $design_id,
      'name' => get_the_title( $design_id ),
      'thumb' => $thumb[0],
      'thumbs' => $thumbs,
      'views' => $views,
      'metadata' => $design_meta,
      'background' => get_post_meta( $design_id, '_design_background_color', true ),
      'search_field' => get_post_meta( $design_id, '_vam_udesign_search_field', true ),
      'categories' => $categories,
      'tags' => $tags
    );

    return new WP_REST_Response( $data );
  }

  public static function design_categories( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    return new WP_REST_Response( self::object_terms( array(
      'taxonomy' => 'design_cat'
    ) ) );
  }

  public static function design_dates( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    return new WP_REST_Response( vam_udesign_post_dates( 'design' ) );
  }

  public static function import_design( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    // process design data
    $product = $request->get_param( 'product' );
    if ( empty( $product ) ) {
      return self::error( 'Empty design information', 400 );
    }

    if ( empty( $product['name'] ) ) {
      return self::error( 'Empty design name', 400 );
    }
    $design_name = $product['name'];

    // process design image
    $image = $request->get_param( 'image' );
    if ( empty( $image ) ) {
      return self::error( 'Empty design image', 400 );
    }

    if ( empty( $image['data'] ) ) {
      return self::error( 'Empty design image data', 400 );
    }

    $image_data = base64_decode( $image['data'] );
    if ( empty( $image_data ) ) {
      return self::error( 'Invalid design image data provided', 400 );
    }

    if ( empty( $image['name'] ) ) {
      return self::error( 'Empty design image name', 400 );
    }
    $image_name = $image['name'];

    $uploaded_file = vam_udesign_upload_image( $image_data, true, $image_name );
    if ( empty( $uploaded_file ) ) {
      return self::error( 'Could not upload design image', 400 );
    }

    if ( empty( $uploaded_file['url'] ) ) {
      return self::error( 'Could not get design image url', 400 );
    }
    $uploaded_url = $uploaded_file['url'];

    if ( empty( $uploaded_file['file'] ) ) {
      return self::error( 'Could not get design image path', 400 );
    }
    $uploaded_path = $uploaded_file['file'];

    $uploaded_path_info = pathinfo( $uploaded_path );
    if ( empty( $uploaded_path_info ) || empty( $uploaded_path_info['filename'] ) ) {
      return self::error( 'Could not get design image path info', 400 );
    }

    if ( empty( $uploaded_file['type'] ) ) {
      return self::error( 'Could not get design image type', 400 );
    }
    $uploaded_type = $uploaded_file['type'];

    $uploaded_image_size = getimagesize( $uploaded_path );
    if ( empty( $uploaded_image_size ) ) {
      return self::error( 'Could not get design image size', 400 );
    }

    // create design categories
    $category_ids = array();

    if ( !empty( $product['categories'] ) ) {
      $categories = $product['categories'];

      foreach( $categories as &$category ) {
        if ( empty( $category['name'] ) ) {
          continue;
        }

        $category_id = vam_udesign_create_term( $category['name'], 'design_cat' );
        if ( !empty( $category_id ) ) {
          $category_ids[] = $category_id;
        }
      }
    }

    // create design tags
    $tag_ids = array();
    $search_field = $design_name;

    if ( !empty( $product['tags'] ) ) {
      $tags = $product['tags'];

      foreach( $tags as &$tag ) {
        if ( empty( $tag['name'] ) ) {
          continue;
        }
        $tag_name = $tag['name'];

        $tag_id = vam_udesign_create_term( $tag_name, 'design_tag' );
        if ( !empty( $tag_id ) ) {
          $tag_ids[] = $tag_id;
          $search_field .= ', ' . $tag_name;
        }
      }
    }

    // create design metadata
    $front_view = get_term_by( 'slug', 'front', 'pa_view', 'ARRAY_A' );
    if ( empty( $front_view ) || empty( $front_view['term_id'] ) ) {
      return self::error( 'Could not get design views', 400 );
    }
    $front_view_id = $front_view['term_id'];

    $design_meta = array(
       array(
          "view" => $front_view_id,
          "layers" => array(
             array(
                "rotation" => 0,
                "flipX" => false,
                "flipY" => false,
                "hidden" => false,
                "locked" => false,
                "selected" => false,
                "id" => intval( microtime( true ) * 1000000 ),
                "type" => "image",
                "name" => $uploaded_path_info['filename'],
                "thumb" => $uploaded_url,
                "src" => $uploaded_url,
                "width" => $uploaded_image_size[0],
                "height" => $uploaded_image_size[1],
                "mime" => $uploaded_type,
                "x" => 0,
                "y" => 0,
                "cX" => 0,
                "cY" => 0
              )
          ),
          "export" => array(
             "bounds" => false,
             "data" => false
          )
       )
    );

    // create design
    $design_id = vam_udesign_create_design( array(
      'name' => $design_name,
      'metadata' => json_encode( $design_meta ),
      'category_ids' => $category_ids,
      'tag_ids' => $tag_ids,
      'search_field' => $search_field,
      'disable_customize' => true
    ) );

    if ( empty( $design_id ) ) {
      self::error( 'Could not create design post', 400 );
    }

    // generate design thumbnails
    $views = array( array(
      'name' => $front_view_id,
      'url' => $uploaded_url,
      'tmp_name' => $uploaded_path,
      'type' => $uploaded_type
    ) );

    $thumbnails = vam_udesign_generate_design_thumbnails( $design_id, $views );
    if ( empty( $thumbnails ) ) {
      self::send_json_error( 'Could not generate design thumbnails' );
    }

    $attach_id = vam_udesign_create_attachment( $thumbnails['featured_image'], $design_id );
    if ( empty( $attach_id ) ) {
      self::send_json_error( 'Could not create attachment' );
    }

    $old_attach_id = get_post_thumbnail_id( $design_id );
    if ( !empty( $old_attach_id ) ) {
      wp_delete_attachment( $old_attach_id, true );
    }

    set_post_thumbnail( $design_id, $attach_id );

    return new WP_REST_Response( 'Design successfully imported' );
  }

  public static function products( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }
    $valid_license = !empty( self::verify_license( $request ) );

    global $post;
    $args = self::parse_list_query_params( $valid_license, $request );
    $args['designer_products'] = true;
    $query = vam_udesign_get_products( $args );
    $products = $query->posts;

    $items = array();
    foreach ( $products as $product_id ) {
      $thumb_id = get_post_thumbnail_id( $product_id );
      $thumb = wp_get_attachment_image_src( $thumb_id, 'full' );

      if ( ! empty( $thumb ) ) {
        $items[] = array(
          'id' => $product_id,
          'name' => get_the_title( $product_id ),
          'thumb' => $thumb[0],
        );
      }
    }

    $data = self::list_query_result( $valid_license, $query, $items );
    return new WP_REST_Response( $data );
  }

  public static function product( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    // Check parameters
    $product_id = $request->get_param( 'id' );
    if ( empty( $product_id ) ) {
      return self::error( 'Must provide product ID', 400 );
    }

    $product = wc_get_product( $product_id );
    if ( empty( $product ) ) {
      return self::error( 'Could not retrieve product', 400 );
    }
    if ( 'variable' != $product->get_type() ) {
      return self::error( 'Only variable products allowed', 400 );
    }
    if ( get_post_meta( $product_id, '_designer', true ) != 'yes' ) {
      return self::error( 'Only designer products allowed', 400 );
    }

    // Get product colors
    $color_terms = VAM_UDesign_WC_Compat::wc_get_product_terms( $product_id,
      'pa_color', array( 'fields' => 'all' ) );

    $colors = array();
    if ( !empty( $color_terms ) ) {
      foreach( $color_terms as &$color_term ) {
        $hex = get_woocommerce_term_meta( $color_term->term_id, 'pa_color_swatches_id_color', true );
        if ( empty( $hex ) ) {
          $hex = '#ffffff';
        }

        $colors[] = array(
          'id' => $color_term->term_id,
          'slug' => $color_term->slug,
          'name' => $color_term->name,
          'color' => $hex
        );
      }
    }

    // Get product views
    $view_terms = VAM_UDesign_WC_Compat::wc_get_product_terms( $product_id,
      'pa_view', array( 'fields' => 'all' ) );

    $views = array();
    if ( !empty( $view_terms ) ) {
      foreach( $view_terms as &$view_term ) {
        $views[] = array(
          'id' => $view_term->term_id,
          'slug' => $view_term->slug,
          'name' => $view_term->name
        );
      }
    }

    // Get views meta
    $designer_views = maybe_unserialize( get_post_meta( $product_id, '_vam_udesign_views', true ) );

    $view_metas = array();
    if ( !empty( $designer_views ) ) {
      foreach( $designer_views as $view_id => &$designer_view ) {
        if ( empty( $view_id ) ) {
          return self::error( 'Product view not properly configured', 400 );
        }
        if ( empty( $designer_view['image_id'] ) ) {
          return self::error( 'Product view does not have an image', 400 );
        }
        if ( empty( $designer_view['colors'] ) ) {
          return self::error( 'Product view does not have colors', 400 );
        }

        $view_image_id = $designer_view['image_id'];

        $image_src = wp_get_attachment_image_src( $view_image_id, 'full' );
        if ( empty( $image_src ) ) {
          return self::error( 'Could not get product view image', 400 );
        }

        $view_meta = array(
          'image_id' => $view_image_id,
          'image' => $image_src[0],
          'colors' => array()
        );

        $view_colors = &$designer_view['colors'];
        foreach( $view_colors as $color_id => $image_id ) {
          $image_src = wp_get_attachment_image_src( $image_id, 'full' );
          if ( empty( $image_src ) ) {
            return self::error( 'Could not get product view color image', 400 );
          }

          $view_meta['colors'][$color_id] = array(
            'image_id' => $image_id,
            'image' => $image_src[0]
          );
        }

        $view_metas[$view_id] = $view_meta;
      }
    }

    // Create result set
    $data = array(
      'id' => $product_id,
      'colors' => $colors,
      'views' => $views,
      'view_metas' => $view_metas
    );

    return new WP_REST_Response( $data );
  }

  public static function product_categories( $request ) {
    if ( empty( self::auth( $request ) ) ) {
      return self::error( 'Unauthorized request', 403 );
    }

    $query = vam_udesign_get_products( array(
      'designer_products' => true
    ) );

    if ( empty( $query ) ) {
      return array();
    }

    return new WP_REST_Response( self::object_terms( array(
      'taxonomy' => 'product_cat',
      'object_ids' => $query->posts
    ) ) );
  }
};
