<?php

use Automattic\WooCommerce\Client;

class VAM_UDesign_Wordpress_REST_API {

  private $api_client;

  public function __construct( $api_client ) {
    $this->api_client = $api_client;
  }

  public function get( $endpoint, $params = array() ) {
    $ret = false;

    try {
      $ret = $this->api_client->get( $endpoint, $params );
    } catch ( Exception $e ) {
      return vam_udesign_error_response( $e->getMessage() );
    }

    return vam_udesign_success_response( $ret );
  }

  public function post( $endpoint, $data ) {
    $ret = false;

    try {
      $ret = $this->api_client->post( $endpoint, $data );
    } catch ( Exception $e ) {
      return vam_udesign_error_response( $e->getMessage() );
    }

    return vam_udesign_success_response( $ret );
  }

  public function put( $endpoint, $data ) {
    $ret = false;

    try {
      $ret = $this->api_client->put( $endpoint, $data );
    } catch ( Exception $e ) {
      return vam_udesign_error_response( $e->getMessage() );
    }

    return vam_udesign_success_response( $ret );
  }

  public function delete( $endpoint, $params = array() ) {
    $ret = false;

    try {
      $ret = $this->api_client->delete( $endpoint, $params );
    } catch ( Exception $e ) {
      return vam_udesign_error_response( $e->getMessage() );
    }

    return vam_udesign_success_response( $ret );
  }
}

function vam_udesign_new_wordpress_api_client( $api_url, $consumer_key, $consumer_secret ) {
  if ( empty( $api_url ) ) {
    return false;
  }
  if ( empty( $consumer_key ) ) {
    return false;
  }
  if ( empty( $consumer_secret ) ) {
    return false;
  }

  $api_client = new Client(
    $api_url,
    $consumer_key,
    $consumer_secret,
    array(
      'verify_ssl' => false,
      'timeout' => 300,
      'query_string_auth' => true,
      'wp_api' => true,
      'version' => 'wc/v2'
    )
  );

  return new VAM_UDesign_Wordpress_REST_API( $api_client );
}
