/* global wp */

/**
 * uDesign clipart bulk upload
 */
jQuery( function ( $ ) {
  'use strict';

  $('#start-upload').on('click', function() {
    var container = $('#postbox-container-2 .postbox');
    var logsContainer = $('.logs-container');

    udesign.ui.block(container);

    var categoryIDs = [];
    $('input[name="tax_input[clipart_cat][]"]:checked').each(function(i, val) {
      var categoryID = parseInt($(val).val(), 10);
      if (isNaN(categoryID)) {
        return;
      }

      categoryIDs.push(categoryID);
    });

    if (!categoryIDs.length) {
      udesign.ui.showNotice(logsContainer, 'Must select at least one category', 'error', false);
      udesign.ui.unblock(container);
      return;
    }

    var files = $('.input-bulk-upload-cliparts').prop('files');

    var filesLen = files.length;
    if (!filesLen) {
      udesign.ui.showNotice(logsContainer, 'Must select at least one file to upload', 'error', false);
      udesign.ui.unblock(container);
      return;
    }

    var uploadClipart = function(file) {
      setTimeout(function() {
        var form = new FormData();
        form.append('action', 'udesign_create_clipart');
        form.append('security', bulk_upload_data.security.clipart_nonce);
        form.append('image', file);
        form.append('category_ids', JSON.stringify(categoryIDs));

        jQuery.ajax({
          url: bulk_upload_data.ajax_url,
          data: form,
          type: 'POST',
          processData: false,
          contentType: false,
        }).always(function(response) {
          if (file.type != 'image/svg+xml') {
            udesign.ui.showNotice(logsContainer, file.name + ': ' + response.data.message,
              response.success !== true ? 'error' : 'success', false);

            filesLen--;
            if (!filesLen) {
              udesign.ui.showNotice(logsContainer, 'All done', 'success', false);
              udesign.ui.unblock(container);
            }

            return;
          }

          udesign.blob.fromSVG(response.data.attachment_url, 400, function(blob) {
            udesign.blob.upload({
              security: bulk_upload_data.security.blob_attachment_nonce,
              postID: response.data.clipart_id,
              postType: 'clipart',
              setThumbnail: true,
              image: {
                name: file.name,
                data: blob
              }
            }, function(response) {
              udesign.ui.showNotice(logsContainer, file.name + ': ' + response.data.message,
                response.success !== true ? 'error' : 'success', false);

              filesLen--;
              if (!filesLen) {
                udesign.ui.showNotice(logsContainer, 'All done!', 'success', false);
                udesign.ui.unblock(container);
              }
            });
          });
        });
      }, 100);
    };

    for (var i = 0; i < files.length; i++) {
      uploadClipart(files[i]);
    }
  });

});
