jQuery(function($) {
  'use strict';

  var designs_import = {
    container_selector: '.design-items',
    item_selector: '.design-item',
    item_width: 200,

    init: function() {
      $(document).ready(this.ready);
      $(window).on('resize', this.resize);
    },

    ready: function() {
      $(document).on('click', designs_import.item_selector, function(e) {
        designs_import.select_item($(this));
        designs_import.check_selection();
      });

      $(document).on('click', '.design-checkbox', function(e) {
        e.stopPropagation();

        var el = $(this).closest(designs_import.item_selector);
        designs_import.select_item(el);
        designs_import.check_selection();
      });

      $(document).on('click', '#select', function(e) {
        designs_import.select_items();
        designs_import.check_selection();
      });

      $(document).on('click', '.pagination-link', designs_import.change_page);
      $(document).on('click', '#import', designs_import.import_designs);

      designs_import.resize();
    },

    resize: function() {
      var grid = $(designs_import.container_selector);
      if (!grid.length) {
        return;
      }

      var grid_items = grid.find(designs_import.item_selector);
      if (!grid_items.length) {
        return;
      }

      var grid_width = Math.ceil(grid.width());
      if (grid_width < 500) {
        grid_items.css({
          'width': '98%',
          'margin': '1%'
        });

        return;
      }

      var items = Math.floor(grid_width / designs_import.item_width);
      grid_items.css({
        'width': (96.0 / items) + '%',
        'margin': (2 / items) + '%'
      });
    },

    select_item: function(el, value) {
      if (!value) {
        value = !el.hasClass('selected');
      }

      el.toggleClass('selected', value);
      el.find('.design-checkbox').prop('checked', value);
    },

    select_items: function() {
      var items = $(designs_import.item_selector);

      var select_all = items.length != items.filter('.selected').length;
      for (var i = 0; i < items.length; i++) {
        designs_import.select_item($(items[i]), select_all);
      }
    },

    check_selection: function () {
      var items = $(designs_import.item_selector);
      if (!items.length) {
        return;
      }

      var count_selected = items.filter('.selected').length;
      var select_msg = designs_import_data.messages.select;
      if (items.length == count_selected) {
        select_msg = designs_import_data.messages.deselect;
      }

      $('#select').text(select_msg);
      $('#import').attr('disabled', !count_selected);
    },

    change_page: function() {
      var value = $(this).data('page');
      if (!value) {
        return;
      }

      $('input[name="paged"]').val(value);
      $('#posts-filter').submit();
    },

    import_designs: function() {
      var queue = [];
      $(designs_import.item_selector + '.selected').each(function(i, val) {
        var id = $(val).data('id');
        if (!id) {
          return;
        }

        queue.push(id);
      });

      designs_import.import_design(queue);
    },

    import_design: function(queue, cb) {
      if (!queue || !queue.length) {
        if (cb) {
          cb();
        }

        return;
      }

      var id = queue.shift();
      if (!id) {
        designs_import.import_design(queue);
        return;
      }

      var el = $(designs_import.item_selector + '[data-id="' + id + '"]');

      udesign.ui.block(el, designs_import_data.messages.importing);
      jQuery.ajax({
        url: designs_import_data.ajax_url,
        type: 'POST',
        data: {
          design_id: id,
          action: 'udesign_import_design',
          security: designs_import_data.security.design_import_nonce
        }
      }).always(function(response) {
        var statusCls = 'status-success';
        if ( !response || !response.success ) {
          statusCls = 'status-failed';
        }

        if (el.hasClass('selected')) {
          el.trigger('click');
        }

        el.toggleClass(statusCls, true);
        el.find('.' + statusCls).toggleClass('hidden', false);
        udesign.ui.unblock(el);

        designs_import.import_design(queue);
      });
    }
  };

  designs_import.init();
});
