/* global wp, designer_data */
jQuery(function($) {
  'use strict';

  /**
   * Designer actions
   */
  var designer_actions = {

    variations: null,
    selected: {
      color: null,
      size: null,
      variation: null
    },

    /**
     * Initialize designer actions
     */
    init: function() {
      $(window).on('beforeunload', this.beforeunload);
      $(document).ready(this.load);
      $(document).on('click', 'a', this.checkUnload);

      $(document).on('designer-load-product', function(event, data) {
          designer_actions.load_designer(data.product_id, data.design);
      });

      $(document).on('designer-update-design-price', function(event, data) {
        if (!data) {
          return;
        }

        var cb = (typeof data.callback === 'function') ? data.callback : function(){};

        var variations = data.variations;
        if (!variations || !variations.length) {
          cb(false);
          return;
        }

        designer_actions.update_design_price(variations, cb);
      });

      $(document).on('designer-add-to-cart', function(event, data) {
        designer_actions.reset_ui_cart();

        if (!data) {
          return;
        }

        var cb = (typeof data.callback === 'function') ? data.callback : function(){};

        var variations = data.variations;
        if (!variations || !variations.length) {
          cb(false);
          return;
        }

        designer_actions.create_cart_design(function(designID) {
          if (!designID) {
            cb(false);
            return;
          }

          designer_actions.add_to_cart(designID, variations, data.button);
        });
      });

      if (parseInt(designer_data.allow_clipart_upload, 10)) {
        Dropzone.autoDiscover = false;
        Dropzone.options.designerUploadDropzone = {
          maxFilesize: 10,
          maxFiles: 6,
          parallelUploads: 6,
          uploadMultiple: false,
          addRemoveLinks: true,
          paramName: 'image',
          acceptedFiles: 'image/*',
          autoProcessQueue: false,
          init: function() {
            this.on('processing', function(file) {
              this.options.url = designer_data.ajax_url;
            }).on('maxfilesexceeded', function(file) {
              this.removeFile(file);
            }).on('queuecomplete', function (file) {
              this.removeAllFiles();
              jQuery('.udesign .upload-popup').hide();
            }).on('complete', function(file) {
              pd.setLoading(false);

              if (file.status == 'success') {
                var response = JSON.parse(file.xhr.response);
                if (response) {
                  pd.loadClipart(response);
                }
              }
            });
          }
        };

        $('#designer-upload-image').click(function(event) {
          event.preventDefault();

          pd.getProduct(function(product) {
            var view_width = 300;
            var view_height = 300;
            for (var i = 0; i < product.views.length; i++) {
              var view = product.views[i];
              if (view.name != product.view) {
                continue;
              }

              view_width = view.bounds.width;
              view_height = view.bounds.height;
              break;
            }

            var dropzone = Dropzone.forElement("#designer-upload-dropzone");
            dropzone.on('sending', function(file, xhr, formData) {
              var previewEl = $(file.previewElement).find('img')[0];

              var image_width = previewEl.naturalWidth;
              if (!image_width) {
                image_width = previewEl.width;
              }

              var image_height = previewEl.naturalHeight;
              if (!image_height) {
                image_height = previewEl.height;
              }

              formData.append('action', 'udesign_upload_clipart');
              formData.append('view_height', view_width);
              formData.append('view_width', view_height);
              formData.append('image_width', image_width);
              formData.append('image_height', image_height);
              formData.append('security', designer_data.security.designer_nonce);
            });
            dropzone.processQueue();
          });
        });
      }

      $('.designer-option-color').on('click', '.option.color', function(event) {
        event.preventDefault();

        var el = $(this);
        var color = el.data('color');
        var value = el.data('value');
        var title = el.attr('title');

        el.closest('.swatch-dropdown').find('.selected-option').
          attr('data-color', color).attr('data-value', value).attr('title', title).
          css('background-color', color);

        designer_actions.set_designer_variation(color, designer_actions.selected.size);
      });

      $('.designer-option-size').on('click', '.option.size', function(event) {
        event.preventDefault();

        var el = $(this);
        var size = el.data('size');
        var value = el.data('value');
        var title = el.attr('title');

        el.closest('.swatch-dropdown').find('.selected-option').text(size).
          attr('data-size', size).attr('data-value', size).attr('title', title);

        designer_actions.set_designer_variation(designer_actions.selected.color, size);
      });

      $('.designer-wrapper')
      .on('click', '.designer-popup-close', function(event) {
        event.preventDefault();

        $(this).closest('.designer-popup').hide();
        pd.setLoading(false);
      })
      .on('click', '.swatch-dropdown', function(event) {
        event.preventDefault();

        var dropdown = $(this).find('.dropdown-list');
        if (!dropdown.is(':visible')) {
          $('.designer-wrapper').find('.dropdown-list').toggle(false);
        }

        dropdown.toggle();
      })
      .on('click', function(event) {
        if (event && $(event.target).parents('.swatch-dropdown').length) {
          return;
        }

        $('.designer-wrapper').find('.dropdown-list').toggle(false);
      })
      .on('click', '.designer-image-item', function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.setLoading(true);
          $('.designer-popup.upload-popup').show();
        }
      })
      .on('click', '.designer-save-design', function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.setLoading(true);

          $('.save-design-popup .msg-save-design').hide();
          $('.designer-popup.save-design-popup').show();
        }
      })
      .on('click', '#designer-save-design-button', function(event) {
        $('#designer-save-design-button').toggleClass('loading', true);

        designer_actions.create_user_design(function(designID) {
          designer_actions.reset_ui_button($('#designer-save-design-button'));

          if (!designID) {
            return;
          }

          $('.designer-save-design-name').val('');
          $('.msg-save-design-success').show();
        });
      })
      .on('click', '.designer-add-to-cart', function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.setLoading(true);
          designer_actions.set_variation();
          designer_actions.reset_ui_cart(false, true);
          $('.designer-popup.add-to-cart-popup').show();
        }
      })
      .on('click', '#designer-add-to-cart-button', function(event) {
        designer_actions.reset_ui_cart();

        var variations = designer_actions.get_cart_variations();
        if (!variations || !variations.length) {
          return;
        }

        $('#designer-add-to-cart-button').toggleClass('loading', true);
        designer_actions.create_cart_design(function(designID) {
          designer_actions.add_to_cart(designID, variations);
        });
      })
      .on('click', '#designer-update-price-button', function(event) {
        designer_actions.reset_ui_cart();

        var variations = designer_actions.get_cart_variations();
        if (variations && variations.length) {
          $('#designer-update-price-button').toggleClass('loading', true);
          designer_actions.update_design_price(variations);
        } else {
          designer_actions.check_variations();
        }
      })
      .on('change', '.add-to-cart-popup .variations-container .designer-cart-quantity', function(event) {
        var el = $(this);
        if (!el || !el.length) {
          return;
        }

        el.removeClass('valid invalid');

        var val = parseInt(el.val(), 10);
        if (!isFinite(val) || val < 0) {
          el.addClass('invalid');
        }
        if (val > 0) {
          el.addClass('valid');
        }
      });
    },

    set_variation: function() {
      var color = $('.selected-option.color').attr('data-value');
      if (color) {
        $('.select-option[data-attribute="pa_color"][data-value="' + color + '"]')
          .not('.selected').trigger('click');
      }

      var size = $('.selected-option.size').attr('data-value');
      if (size) {
        $('.select-option[data-attribute="pa_size"][data-value="' + size + '"]')
          .not('.selected').trigger('click');
      }

      designer_actions.check_variations();
    },

    check_variations: function() {
      var variationForm = $('.design_variations_form');
      if (variationForm.length) {
        variationForm.trigger('check_variations',
          {data: {variationForm: variationForm}});
      }
    },

    set_designer_variation: function(color, size) {
      if (!size || !color || !designer_actions.variations) {
        return;
      }

      color = color.toLowerCase();
      size = size.toLowerCase();

      var match = false;
      $.each(designer_actions.variations, function(i, variation) {
        if (variation.size === size && variation.color === color) {
          match = variation;
        }
      });

      if (!match) {
        return;
      }

      designer_actions.selected.color = color;
      designer_actions.selected.size = size;
      designer_actions.selected.variation = match;

      pd.setVariation(match, function() {
        designer_actions.set_variation();
      });
    },

    beforeunload: function(e) {
      if (designer_data.is_admin) {
        return;
      }

      return designer_data.i18n_edited_design;
    },

    checkUnload: function(e) {
      var href = $(this).attr('href');
      if (!href) {
        return;
      }

      var unloadExcluded = ['/cart/', '/checkout/'];
      for (var i = 0; i < unloadExcluded.length; i++) {
        if (href.indexOf(unloadExcluded[i]) != -1) {
          $(window).off('beforeunload');
          return;
        }
      }
    },

    load: function(event) {
      var handlers = {
        changed: designer_actions.designer_changed,
        loaded: function() {
          designer_actions.load_designer(
            designer_data.product_id,
            designer_data.design
          );
        },
        products: {
          categories: designer_ajax.get_product_categories,
          list: designer_ajax.list_products,
          query: designer_ajax.search_products,
          details: designer_ajax.get_product_details,
          load: designer_ajax.load_product,
        },
        cliparts: {
          categories: designer_ajax.get_clipart_categories,
          list: designer_ajax.list_cliparts,
          query: designer_ajax.search_cliparts,
          load: designer_ajax.load_clipart,
        },
        designs: {
          categories: designer_ajax.get_design_categories,
          list: designer_ajax.list_designs,
          query: designer_ajax.search_designs,
          load: designer_ajax.load_design,
          delete: designer_ajax.delete_design,
        }
      };

      if (designer_actions.check_product()) {
        var pd = new ProductDesigner('#designer-main-region',
          JSON.parse(designer_data.designer_config), handlers);
        window.pd = pd;
      }

      $('#designer-upload-dropzone').dropzone({
        url: designer_data.ajax_url
      });
    },

    check_product: function() {
      if (!parseInt(designer_data.product_id)) {
        alert('No product!');
        return false;
      }

      return true;
    },

    load_designer: function(productID, design) {
      design = (design && design.length) ? design : null;
      pd.loadProduct(designer_ajax.load_product(productID), design);

      $('.designer-product-item').click(function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.selectProduct();
        }
      });

      $('.designer-text-item').click(function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.addText();
        }
      });

      $('.designer-clipart-item').click(function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.addClipart();
        }
      });

      $('.designer-design-item').click(function(event) {
        event.preventDefault();

        if (designer_actions.check_product()) {
          pd.addDesign();
        }
      });
    },

    designer_changed: function(event) {
    },

    /* UI functions */
    load_ui: function(product) {
      designer_actions.load_ui_colors(product);
      designer_actions.load_ui_sizes(product);
      designer_actions.load_ui_cart(product);
    },

    load_ui_colors: function(product) {
      var colorHTML = '';
      var selectedColor = {};
      var colorList = $('.designer-option-list-colors');

      if (product.colors) {
        product.colors.forEach(function(color) {
          colorHTML += '<div style="background-color:' + color.color +
            '" class="option color" data-color="' + color.color +
            '" title="' + color.name + '" data-value="' + color.slug +
            '">&nbsp;</div>';

          if (product.color === color.color) {
            selectedColor = color;
          }
        });

        var lenColors = product.colors.length;

        var cls = '';
        if (lenColors > 35) {
            cls = 'items-10';
        } else if (lenColors > 25) {
            cls = 'items-7';
        } else if (lenColors > 15) {
            cls = 'items-5';
        }

        colorList.addClass(cls);
      }

      colorList.html(colorHTML);
      $('.selected-option.color').css('background-color', product.color).
        attr('data-color', product.color).
        attr('data-value', selectedColor.slug).
        attr('title', selectedColor.name);
    },

    load_ui_sizes: function(product) {
      var sizeHTML = '';
      var selectedSize = {};

      if (product.sizes) {
        product.sizes.forEach(function(size) {
          sizeHTML += '<div class="option size" data-size="' + size.slug +
            '" data-value="' + size.slug + '" title="' + size.name +
            '">' + size.name + '</div>';

          if (product.size == size.slug) {
            selectedSize = size;
          }
        });
      }

      $('.designer-option-list-sizes').empty().html(sizeHTML);
      $('.selected-option.size').text(product.size).
        attr('data-size', product.size).
        attr('data-value', selectedSize.slug).
        attr('title', selectedSize.name);
    },

    load_ui_cart: function(product) {
      var addToCartEl = $('.add-to-cart-popup');
      if (!addToCartEl.length) {
        return;
      }

      addToCartEl.data('product', product.id);
      addToCartEl.find('.designer-popup-heading').text(product.name);

      var tableHTML = '';
      if (product.colors && product.sizes && product.variations) {
        var lenColors = product.colors.length;
        var lenSizes = product.sizes.length;
        var lenVariations = product.variations.length;

        var tableBody = '';

        for (var i = 0; i < lenColors; i++) {
          var color = product.colors[i];

          var row = '<tr><td class="designer-cart-cell">' +
            '<p class="designer-cart-color-label">&nbsp;</p>' +
            '<div class="designer-cart-color" ' +
            'style="background-color:' + color.color + '">&nbsp;</div>' +
            '</td>';

          for (var j = 0; j < lenSizes; j++) {
            var size = product.sizes[j];

            var variationID = 0;
            for (var k = 0; k < lenVariations; k++) {
              var variation = product.variations[k];
              if (!variation || !variation.id) {
                continue;
              }

              if (variation.color_slug == color.slug &&
                variation.size_slug == size.slug) {
                variationID = variation.id;
                break;
              }
            }

            row += '<td class="designer-cart-cell">' +
              '<p class="designer-cart-size-label">' + size.name + '</p>' +
              '<input type="number" min="0" step="1" ' +
              'pattern="[0-9]*" inputmode="numeric" value="0" ' +
              'class="designer-cart-quantity" ' +
              'data-variation="' + variationID + '" ' +
              'title="' + size.name + ' - ' + color.name + '"/>' +
              '</td>';
          }

          tableBody += row + '</tr>';
        }

        tableHTML = '<table class="shop_table shop_table_responsive">';
        tableHTML += '<tbody>' + tableBody + '</tbody>';
        tableHTML += '</table>';
      }

      addToCartEl.find('.variations-container').html(tableHTML);
    },

    show_ui_error: function(msgSelector, response) {
      var msgEl = $(msgSelector);
      if (!msgEl || !msgEl.length) {
        return;
      }

      var msg = (response && response.data && response.data.message) ?
        response.data.message : msgEl.data('default-error');

      msgEl.find('.msg').text(msg);
      msgEl.show();
    },

    show_ui_cart_error: function(response, defaultKey) {
      $('#designer-add-to-cart-button').toggleClass('loading', false);
      $('#designer-update-price-button').toggleClass('loading', false);

      var msgEl = $('.msg-cart-error');
      var msg = (response && response.data && response.data.message) ?
        response.data.message : msgEl.data(defaultKey + '-default-error');

      msgEl.find('.msg').text(msg);
      msgEl.show();
    },

    show_ui_cart_success: function() {
      var addToCartBtn = $('#designer-add-to-cart-button');
      addToCartBtn.toggleClass('loading', false);

      designer_actions.reset_ui_cart_quantities();
      $('.msg-cart-success').show();
    },

    reset_ui_cart: function(resetQuantities, resetPrice) {
      designer_actions.reset_ui_cart_button();
      designer_actions.reset_ui_price_button();

      if (resetQuantities) {
        designer_actions.reset_ui_cart_quantities();
      }

      if (resetPrice) {
        var priceEl = $('.add-to-cart-popup .cart-price .price-value');
        priceEl.html(priceEl.data('default-value'));
      }

      $('.msg-cart').hide();
    },

    reset_ui_button: function(button) {
      if (!button || !button.length) {
        return;
      }

      button.toggleClass('loading', false);
      button.toggleClass('added', false);
    },

    reset_ui_cart_button: function() {
      designer_actions.reset_ui_button($('#designer-add-to-cart-button'));
    },

    reset_ui_price_button: function() {
      designer_actions.reset_ui_button($('#designer-update-price-button'));
    },

    reset_ui_cart_quantities: function() {
      var quantityEls = $('.add-to-cart-popup .variations-container .designer-cart-quantity');
      quantityEls.val(0);
      quantityEls.removeClass('valid invalid');
    },

    /* Add to cart */
    get_cart_variations: function() {
      var items = $('.add-to-cart-popup .variations-container input.designer-cart-quantity');
      if (!items.length) {
        return false;
      }

      var variations = [];
      var invalidItems = false;
      for(var i = 0; i < items.length; i++) {
        var item = $(items[i]);
        if (!item.length) {
          continue;
        }

        var id = item.data('variation');
        if (!id) {
          continue;
        }

        var quantity = parseInt(item.val(), 10);
        if (!isFinite(quantity) || quantity < 0) {
          invalidItems = true;
          continue;
        }
        if (quantity === 0) {
          continue;
        }

        variations.push({
          id: id,
          quantity: quantity
        });
      }

      if (invalidItems) {
        $('.msg-cart-invalid-items').show();
        return false;
      }
      if (!variations.length) {
        $('.msg-cart-no-items').show();
        return false;
      }

      return variations;
    },

    create_user_design: function(cb) {
      cb = (typeof cb === 'function') ? cb : function(){};

      var designName = $('#designer-save-design-name').val();
      if (!designName) {
        designName = '';
      }

      designName = designName.trim();
      if (!designName) {
          $('.msg-save-design-invalid-name').show();
          cb(false);
          return;
      }

      pd.getDesigns(function(designs) {
        pd.exportDesigns(function(views) {
          var validDesign = false;
          for (var view in views) {
            if (views[view]) {
              validDesign = true;
              break;
            }
          }

          if (!validDesign) {
            $('.msg-save-design-invalid-design').show();
            cb(false);
            return;
          }

          $.ajax({
            url: designer_data.ajax_url,
            data: {
              action: 'udesign_create_user_design',
              security: designer_data.security.designer_nonce,
              design_name: designName,
              design_meta: JSON.stringify(designs),
              default_product_id: designer_actions.variation_id,
              background: designer_actions.selected.color
            },
            type: 'POST',
          }).always(function(response) {
            if (!response || !response.success ||
                !response.data || !response.data.id) {
              designer_actions.show_ui_error('.msg-save-design-error', response);
              cb(false);
              return;
            }

            var designID = response.data.id;

            udesign.designs.upload({
              id: designID,
              views: views,
              security: designer_data.security.designer_nonce
            }, function(response) {
              if (!response || !response.success) {
                designer_actions.show_ui_error('.msg-save-design-error', response);
                cb(false);
                return;
              }

              if (cb) {
                cb(designID);
              }
            });
          });
        }, {
          size: 600
        });
      });
    },

    create_cart_design: function(cb) {
      cb = (typeof cb === 'function') ? cb : function(){};

      pd.getDesigns(function(designs) {
        pd.exportDesigns(function(views) {
          var validDesign = false;
          for (var view in views) {
            if (views[view]) {
              validDesign = true;
              break;
            }
          }

          if (!validDesign) {
            $('.msg-cart-invalid-design').show();
            designer_actions.reset_ui_cart_button();
            cb(false);
            return;
          }

          $.ajax({
            url: designer_data.ajax_url,
            data: {
              action: 'udesign_create_cart_design',
              security: designer_data.security.designer_nonce,
              design_meta: JSON.stringify(designs),
              default_product_id: designer_actions.variation_id,
              background: designer_actions.selected.color
            },
            type: 'POST',
          }).always(function(response) {
            if (!response || !response.success ||
                !response.data || !response.data.id) {
              designer_actions.show_ui_cart_error(response, 'cart');
              cb(false);
              return;
            }

            var designID = response.data.id;

            udesign.designs.upload({
              id: designID,
              views: views,
              security: designer_data.security.designer_nonce
            }, function(response) {
              if (!response || !response.success) {
                designer_actions.show_ui_cart_error(response, 'cart');
                cb(false);
                return;
              }

              if (cb) {
                cb(designID);
              }
            });
          });
        }, {
          size: 600
        });
      });
    },

    add_to_cart: function(designID, variations, button) {
      if (!designID) {
        $('.msg-cart-invalid-design').show();
        designer_actions.reset_ui_cart_button();
        return;
      }
      if (!variations || !variations.length) {
        $('.msg-cart-no-items').show();
        designer_actions.reset_ui_cart_button();
        return;
      }

      designer_actions.add_to_cart_variation(designer_actions.product_id,
        designID, variations, button, function() {
        designer_actions.show_ui_cart_success();
      });
    },

    add_to_cart_variation: function(productID, designID, variations, button, cb) {
      if (!variations || !variations.length) {
        if (cb) {
          cb();
        }
        return;
      }

      var variation = variations.shift();
      if (!variation) {
        designer_actions.add_to_cart_variation(productID, designID, variations, button, cb);
        return;
      }

      var data = {
        action: 'udesign_add_to_cart',
        product_id: productID,
        design_id: designID,
        quantity: variation.quantity,
        variation_id: variation.id
      };

      udesign.designs.add_to_cart(data, button, function(response) {
        if (!response || !response.cart_hash || !response.fragments) {
          designer_actions.show_ui_cart_error(false, 'cart');
          return;
        }

        designer_actions.add_to_cart_variation(productID, designID, variations, button, cb);
      });
    },

    update_design_price: function(variations, cb) {
      cb = (typeof cb === 'function') ? cb : function(){};

      if (!variations || !variations.length) {
        $('.msg-cart-no-items').show();
        designer_actions.reset_ui_price_button();
        cb(false);
        return;
      }

      pd.getDesigns(function(designs) {
        $.ajax({
          url: designer_data.ajax_url,
          type: 'POST',
          data: {
            action: 'udesign_get_design_price',
            security: designer_data.security.designer_nonce,
            design_meta: JSON.stringify(designs),
            variations: variations
          },
        }).always(function(response) {
          if (!response || !response.success ||
              !response.data || !response.data.price_html) {

            designer_actions.show_ui_cart_error(response, 'price');
            cb(false);
            return;
          }

          cb(response);
          designer_actions.reset_ui_price_button();
          $('.add-to-cart-popup .cart-price .price-value').html(response.data.price_html);
        });
      });
    }
  };

  /**
   * Designer ajax methods
   */
  var designer_ajax = {

    /**
     * Initialize designer ajax methods
     */
    init: function() {
    },

    /* Load product categories */
    get_product_categories: function() {
      return JSON.parse(designer_data.config.product_categories);
    },

    /* Load clipart categories */
    get_clipart_categories: function() {
      return JSON.parse(designer_data.config.clipart_categories);
    },

    /* Load design categories */
    get_design_categories: function() {
      return JSON.parse(designer_data.config.design_categories);
    },

    /* List products in a category */
    list_products: function(categoryID, page) {
      if (!page) {
        page = 1;
      }

      var products = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:      'udesign_get_products',
          security:    designer_data.security.designer_nonce,
          category_id: categoryID,
          page:        page,
          data:        {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return products;
    },

    /* Search for products */
    search_products: function(query, page) {
      if (!page) {
        page = 1;
      }

      var products = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:   'udesign_get_products',
          security: designer_data.security.designer_nonce,
          query:    query,
          page:     page,
          data:     {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return products;
    },

    /* Get product details */
    get_product_details: function(productID) {
      var product = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:     'udesign_get_product_details',
          security:   designer_data.security.designer_nonce,
          product_id: productID,
          data:       {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return product;
    },

    /* Get product by id */
    load_product: function(productID) {
      var product = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:     'udesign_get_product',
          security:   designer_data.security.designer_nonce,
          product_id: productID,
          data:       {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      $('.designer-wrapper').data({
          product_id: product.id,
          pa_size: product.size,
          pa_color: product.color
      }).trigger('load_designer_product');

      designer_actions.variation_id = product.id;
      designer_actions.product_id = product.product_id;
      designer_actions.variations = product.variations;
      designer_actions.selected.size = product.size;
      designer_actions.selected.color = product.color;
      designer_actions.selected.variation = product;
      designer_actions.load_ui(product);

      return product;
    },

    /* List all cliparts in a category */
    list_cliparts: function(categoryID, page) {
      if (!page) {
        page = 1;
      }

      var cliparts = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:      'udesign_get_cliparts',
          security:    designer_data.security.designer_nonce,
          category_id: categoryID,
          page:        page,
          data:        {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return cliparts;
    },

    /* Search for cliparts */
    search_cliparts: function(query, page) {
      if (!page) {
        page = 1;
      }

      var cliparts = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:   'udesign_get_cliparts',
          security: designer_data.security.designer_nonce,
          query:    query,
          page:     page,
          data:     {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return cliparts;
    },

    /* Get clipart by id */
    load_clipart: function(clipartID, view) {
      var clipart = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:      'udesign_get_clipart',
          security:    designer_data.security.designer_nonce,
          clipart_id:  clipartID,
          view_width:  view.bounds.width,
          view_height: view.bounds.height,
          data:        {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return clipart;
    },

    /* List all designs in a category */
    list_designs: function(categoryID, page) {
      if (!page) {
        page = 1;
      }

      var designs = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:      'udesign_get_designs',
          security:    designer_data.security.designer_nonce,
          category_id: categoryID,
          page:        page,
          data:        {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return designs;
    },

    /* Search for designs */
    search_designs: function(query, page) {
      if (!page) {
        page = 1;
      }

      var designs = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:   'udesign_get_designs',
          security: designer_data.security.designer_nonce,
          query:    query,
          page:     page,
          data:     {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return designs;
    },

    /* Get design by ID */
    load_design: function(designID, view, designs) {
      var design = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:          'udesign_get_design',
          security:        designer_data.security.designer_nonce,
          design_id:       designID,
          current_view:    view.name,
          current_designs: JSON.stringify(designs),
          data:            {}
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return design;
    },

    /* Delete design by ID */
    delete_design: function(designID) {
      var response = $.ajax({
        url: designer_data.ajax_url,
        data: {
          action:          'udesign_delete_my_design',
          security:        designer_data.security.designer_nonce,
          design_id:       designID,
        },
        type: 'POST',
        async: false
      }).responseJSON;

      return response;
    },
  };

  designer_actions.init();
  designer_ajax.init();

});
