/* global designer_data */
jQuery( function( $ ) {
  'use strict';

  if ( typeof designer_order_data === 'undefined' ) {
    return false;
  }

  var designer_actions = {
    /**
     * Designer metabox init
     */
    init: function() {
      $( '.meta-box-sortables' ).sortable( { cancel: '.not-sortable' } );
      $( '.woocommerce_order_items .item-designer .hndle' ).on( 'click', designer_actions.toggle_item_designer );
      $( '.designer-download-view, .designer-download-all' ).on( 'click', designer_actions.download_views );
    },

    toggle_item_designer: function( event ) {
      var toggle = $( event.currentTarget );
      var item = toggle.closest( '.item-designer' );
      var container = item.find( '.inside' );
      var designer = $( '.item-designer-container' );

      var openTabs = $( ".item-designer .not-sortable:not('.closed')" ).
        not( item.find( '.not-sortable' ) );

      var isClosed = designer.is( '.invisible' );
      if ( !isClosed && !openTabs.length ) {
        designer.toggleClass( 'invisible', true );
        return;
      }

      designer.toggleClass( 'invisible', true );
      openTabs.toggleClass( 'closed', true );

      container.height( designer.outerHeight( true ) );
      designer.css( 'top', ( item.position().top + item.find( '.hndle' ).outerHeight() ) + 'px' );
      designer.toggleClass( 'invisible', false );

      var orderItemID = item.parent().data( 'order_item_id' );
      var orderItem = designer_order_data.order_items[orderItemID];
      if ( !orderItem ) {
        return;
      }

      $( document ).trigger( 'designer-load-product', {
        'product_id': orderItem.product_id,
        'design': orderItem.design
      } );
    },

    download_views: function( event ) {
      var pd = window.pd;
      if ( !pd ) {
        return;
      }

      var minWidth = parseInt( $( '#designer-download-width' ).val(), 10 );
      if ( isNaN( minWidth ) ) {
        minWidth = 1920;
      }

      var downloadAll = $( event.target ).is( '.designer-download-all' );

      pd.setLoading( true );
      pd.getProduct( function( product ) {
        if ( !product ) {
          pd.setLoading( false );
          return;
        }

        var views = [];
        if ( !downloadAll ) {
          views.push( product.view );
        }

        pd.getDesigns(function(designs) {
          if ( !designs ) {
            pd.setLoading( false );
            return;
          }

          pd.exportDesigns( function( views ) {
            if ( !views ) {
              pd.setLoading( false );
              return;
            }

            udesign.designs.download( {
              id: product.id,
              views: views,
              designs: JSON.stringify(designs),
              security: designer_order_data.security.designer_download_nonce,
            }, function( response ) {
              if ( !response || !response.success || !response.data.file ) {
                pd.setLoading( false );
                return;
              }
              var data = response.data.file;

              var blob = window.dataURLtoBlob( data.blob );
              if ( !blob ) {
                pd.setLoading( false );
                return;
              }

              var file = new File( [blob], data.name, {type: data.mime} );
              saveAs( file );

              pd.setLoading( false );
            } );
          }, {
            views: views,
            minWidth: minWidth
          } );
        } );
      } );
    }
  };

  designer_actions.init();
});
