/* global wp */
jQuery(function($) {
  'use strict';

  var designer_actions = {
    /**
     * Designer metabox init
     */
    init: function() {
      $('#publish').on('click', this.save);
      $('document').ready(function() {
        designer_actions.init_color_picker();
        designer_actions.excluded_colors_check_selection(false);
      });

      // Excluded colors
      $('#excluded-colors-select-btn').on('click', function() {
          designer_actions.excluded_colors_check_selection(true);
      });
      $('.design-excluded-colors .excluded-color-input').on('change', function() {
          designer_actions.excluded_colors_check_selection(false);
      });
    },

    /**
     * Init color picker
     */
    init_color_picker: function() {
      $('[data-type="colorpicker"]').each(function() {
        $( this ).wpColorPicker();
      });
    },

    /**
     * Save on post form submit
     */
    save: function(e, submit) {
      if (submit) {
        udesign.ui.unblock('#udesign-designer');
        return true;
      }

      e.preventDefault();
      udesign.ui.block('#udesign-designer');

      pd.getProduct(function(product) {
        if (!product) {
          designer_actions.submit();
        }

        $('input[name="design_default_product_id"]').val(product.id);

        var bgInput = $('input[name="design_background_color"]');
        if (!bgInput.val()) {
          bgInput.val(product.color);
        }

        pd.getDesigns(function(designs) {
          $('input[name="design_meta"]').val(JSON.stringify(designs));

          pd.exportDesigns(function(views) {
            udesign.designs.upload({
              id: designer_data.post_id,
              views: views,
              security: designer_data.security.designer_nonce,
            }, function(response) {
              if (response.success !== true) {
                udesign.ui.unblock('#udesign-designer');
                return;
              }

              $('#_thumbnail_id').val(response.data.attachment_id);
              designer_actions.submit();
            });
          }, {
            size: 600
          });
        });
      });
    },

    submit: function() {
      $('#publish').trigger('click', true);
    },

    excluded_colors_check_selection: function(do_selection) {
      var container = $('.design-excluded-colors');

      var els = container.find('.excluded-color-input');
      if (!els || !els.length) {
        return;
      }

      var checked_els = container.find('.excluded-color-input:checked');
      var select_all = els.length != checked_els.length;

      if (do_selection) {
        els.prop('checked', select_all);
        select_all = !select_all;
      }

      var btn = $('#excluded-colors-select-btn');
      var title = select_all ? btn.data('select-all') : btn.data('deselect-all');
      btn.text(title);
    }
  };

  designer_actions.init();
});
