/* global wp, woocommerce_admin_meta_boxes_variations, woocommerce_admin, admin_meta_boxes_designer_views, accounting */
jQuery( function( $ ) {
  'use strict';

  /**
   * Designer views actions
   */
  var designer_views_actions = {

    /**
     * Initialize designer views actions
     */
    init: function() {
      var wrapper = $( '#woocommerce-product-data' );
      wrapper
        .on( 'change', 'input#_designer', this.variable_is_designer )
        .on( 'designer_views_loaded', this.designer_views_loaded )
        .on( 'reload', '#variable_product_options', this.reload )
        .on( 'click', '.add_new_color_attribute', this.add_new_color_attribute )
        .on( 'change', 'select.attribute_values', this.init_edit_color_attribute )
        .on( 'init_edit_color_attribute', this.init_edit_color_attribute )
        .trigger( 'init_edit_color_attribute' );

      wrapper.find( '.show_if_designer' )
        .toggle( $( 'input#_designer' ).is( ':checked' ) );

      $( 'select#product-type' ).change( function() {
        var select_val = $( this ).val();
        if ( 'variable' !== select_val ) {
          $( 'input#_designer' ).removeAttr( 'checked' ).change();
        }
      }).change();

      $( document.body )
        .on( 'designer_views_added', this.designer_views_added )
        .on( 'init_color_picker', this.init_color_picker )
        .trigger( 'init_color_picker' );
    },

    /**
     * Reload UI
     *
     * @param {Object} event
     * @param {Int} qty
     */
    reload: function() {
      designer_views_ajax.load_designer_views();
    },

    /**
     * Check if variable product is designer product
     */
    variable_is_designer: function() {
      var wrapper = $( '#woocommerce-product-data' );
      if ( $( this ).is( ':checked' ) ) {
        wrapper.find( '.show_if_designer' ).show();

        return;
      }

      wrapper.find( '.show_if_designer' ).hide();
      wrapper.find( 'ul.wc-tabs li:visible' ).eq( 0 ).find( 'a' ).click();
    },

    /**
     * Notice dismiss
     */
    notice_dismiss: function() {
      $( this ).closest( 'div.notice' ).remove();
    },

    /**
     * Run actions when designer views is loaded
     *
     * @param {Object} event
     * @param {Int} needsUpdate
     */
    designer_views_loaded: function( event, needsUpdate ) {
      needsUpdate = needsUpdate || false;

      var wrapper = $( '#woocommerce-product-data' );

      if ( ! needsUpdate ) {
        // Remove designer-view-needs-update classes
        $( '.designer_views .designer-view-needs-update', wrapper ).removeClass( 'designer-view-needs-update' );

        // Disable cancel and save buttons
        $( 'button.cancel-designer-views-changes, button.save-designer-views-changes', wrapper ).attr( 'disabled', 'disabled' );
      }

      // Init TipTip
      $( '#tiptip_holder' ).removeAttr( 'style' );
      $( '#tiptip_arrow' ).removeAttr( 'style' );
      $( '.designer_views .tips, .designer_views .help_tip, .designer_views .woocommerce-help-tip', wrapper ).tipTip({
        'attribute': 'data-tip',
        'fadeIn':    50,
        'fadeOut':   50,
        'delay':     200
      });

      $( document.body ).trigger( 'wc-enhanced-select-init' );
      designer_views_media.init();
    },

    /**
     * Run actions when added a variation
     *
     * @param {Object} event
     * @param {Int} qty
     */
    designer_views_added: function( event, qty ) {
      if ( 1 === qty ) {
        designer_views_actions.designer_views_loaded( null, true );
      }
    },

    /**
     * Init color picker
     */
    init_color_picker: function() {
      $('[data-type="colorpicker"]').each(function() {
        $( this ).wpColorPicker();
      });
    },

    append_edit_color_attribute_modal: function( response ) {
      if ( !$().modal || !response ) {
        return;
      }

      $( response )
        .on( $.modal.OPEN, function( event, modal ) {
          modal.elm.on( 'click', '.save_color_attribute',
            designer_views_actions.save_color_attribute );
        })
        .on( $.modal.CLOSE, function( event, modal ) {
          modal.elm.find('.wp-color-picker').wpColorPicker('close');
          modal.elm.remove();
        })
        .appendTo( 'body' ).modal();

        $( 'body' ).trigger( 'init_color_picker' );
    },

    add_new_color_attribute: function( event ) {
      var $wrapper  = $( this ).closest( '.woocommerce_attribute' );
      var attribute = $wrapper.data( 'taxonomy' );

      $.ajax({
        url: admin_meta_boxes_designer_views.ajax_url,
        data: {
          action:   'udesign_load_edit_color_attribute',
          security: admin_meta_boxes_designer_views.load_edit_color_attribute_nonce,
          taxonomy: attribute
        },
        type: 'POST',
        success: function( response ) {
          designer_views_actions.append_edit_color_attribute_modal( response );
        }
      });
    },

    /**
     * Init color attribute edit
     */
    init_edit_color_attribute: function() {
      $( '.woocommerce_attribute.pa_color .select2-container' ).
        off( 'click' ).on( 'click', '.select2-search-choice',
          designer_views_actions.edit_color_attribute ).
        off( 'click' ).on( 'click', '.select2-selection__choice',
          designer_views_actions.edit_color_attribute );
    },

    edit_color_attribute: function( event ) {
      var target = $(event.target);
      if ( target.is( '.select2-selection__choice__remove' ) ||
        target.is( '.select2-search-choice-close' ) ) {
        return;
      }

      var $wrapper  = $( this ).closest( '.woocommerce_attribute' );
      var attribute = $wrapper.data( 'taxonomy' );
      var $token    = $( event.currentTarget );
      var name      = $token.attr( 'title' ) || $token.find( 'div' ).first().text();

      $.ajax({
        url: admin_meta_boxes_designer_views.ajax_url,
        data: {
          action:   'udesign_load_edit_color_attribute',
          security: admin_meta_boxes_designer_views.load_edit_color_attribute_nonce,
          taxonomy: attribute,
          term: name
        },
        type: 'POST',
        success: function( response ) {
          designer_views_actions.append_edit_color_attribute_modal( response );
        }
      });
    },

    save_color_attribute: function( event ) {
      $( '.product_attributes' ).block({
        message: null,
        overlayCSS: {
          background: '#fff',
          opacity: 0.6
        }
      });

      var $term_wrapper = $( this ).closest( '.woocommerce_attribute' );
      var taxonomy = $term_wrapper.data( 'taxonomy' );
      var term_id = parseInt( $term_wrapper.data( 'term_id' ), 10 );
      var new_term_name = $term_wrapper.find( ':input.term_name' ).val();
      var new_term_color = $term_wrapper.find( ':input.term_color' ).val();
      var $wrapper = $( '#woocommerce-product-data .woocommerce_attribute.' + taxonomy );

      if ( new_term_name && new_term_color ) {
        $.ajax({
          url: admin_meta_boxes_designer_views.ajax_url,
          data: {
            action:   'udesign_save_color_attribute',
            security: admin_meta_boxes_designer_views.save_color_attribute_nonce,
            taxonomy: taxonomy,
            term_id:  term_id,
            term:     new_term_name,
            color:    new_term_color
          },
          type: 'POST',
          error: function( response ) {
            window.alert( response.error );
          },
          success: function( response ) {
            if ( response.slug && response.term_id ) {
              var $select = $wrapper.find( 'select.attribute_values' );
              var option_value = $('.select2-selection--multiple').length ?
                response.term_id : response.slug;
              var options = '';
              if ( term_id ) {
                options = $select.find( 'option[value="' + option_value + '"]' )
                  .text(response.name).end().html();
              } else {
                options = $select.append( '<option value="' + option_value +
                  '" selected="selected">' + response.name + '</option>' ).html();
              }

              if ( options ) {
                $select.html(options).change();
              }
            }
          },
          complete: function() {
            $( '.product_attributes' ).unblock();
            $.modal.close();
          }
        });
      } else {
        $( '.product_attributes' ).unblock();
      }

      return false;
    }
  };

  /**
   * Designer views media actions
   */
  var designer_views_media = {

    /**
     * wp.media post ID
     *
     * @type {Int}
     */
    wp_media_post_id: wp.media.model.settings.post.id,

    /**
     * Initialize media actions
     */
    init: function() {
      $('#designer_product_data .upload_image_button').each(function(index, el) {
        var _custom_media = false;
        var _orig_send_attachment = wp.media.editor.send.attachment;

        var $el = $(el);
        var frame_key = $el.data('frame');
        var frame = wp.media.frames[frame_key];

        $el.click(function(e) {
          e.preventDefault();

          if ($el.is('.remove')) {
            $el.find('.upload_image_id').val('').change();
            $el.find('img').attr('src', admin_meta_boxes_designer_views.woocommerce_placeholder_img_src);
            $el.toggleClass('remove', false);

            return false;
          }

          // If the media frame already exists, reopen it.
          if (frame) {
            frame.open();
            return;
          }

          // Create a new media frame
          frame = wp.media({
            title: 'Select or Upload an Image',
            button: {
              text: 'Use this media'
            },
            multiple: false  // Set to true to allow multiple files to be selected
          });


          // When an image is selected in the media frame...
          frame.on('select', function() {

            // Get media attachment details from the frame state
            var attachment = frame.state().get('selection').first().toJSON();

            if ( attachment.id ) {
              var url = attachment.sizes.thumbnail ?
                attachment.sizes.thumbnail.url : attachment.url;

              $el.find('.upload_image_id').val(attachment.id).change();
              $el.find('img').attr('src', url);
              $el.toggleClass('remove', true);
            }
          });

          // Finally, open the modal on click
          frame.open();
          return false;
        });
      });

      $('#designer_product_data .designer_view_editor').each(function(index, el) {
        designer_views_media.enable_interact(el.querySelector('.interact-holder'));

        var $el = $(el);
        var _custom_media = false;
        var _orig_send_attachment = wp.media.editor.send.attachment;
        var $wrapper = $el.closest('.designer_view');
        var frame_key = $el.data('frame');
        var frame = wp.media.frames[frame_key];
        var $holder = $el.find('.view-image-holder');
        var holderWidth = $holder.width();
        var holderHeight = $holder.height();
        var scalingFactor = 0.6;
        var $imageID = $el.find('.image-id');
        var $imageW = $el.find('.image-w');
        var $imageH = $el.find('.image-h');
        var $viewW = $wrapper.find(':input.view-w');
        var $viewH = $wrapper.find(':input.view-h');
        var $imageViewW = $wrapper.find(':input.image-view-w');
        var $imageViewH = $wrapper.find(':input.image-view-h');
        var $imageViewX = $wrapper.find(':input.image-view-x');
        var $imageViewY = $wrapper.find(':input.image-view-y');
        var $interactHolder = $el.find('.interact-holder');

        var interactResize = function(heightBased, triggered) {
          var width = parseFloat($imageViewW.val());
          if (!width || !isFinite(width)) {
            return;
          }
          var height = parseFloat($imageViewH.val());
          if (!height || !isFinite(height)) {
            return;
          }

          var aspectRatio = parseFloat($viewW.val()) / parseFloat($viewH.val());
          if (!aspectRatio || !isFinite(aspectRatio)) {
            return;
          }

          if (heightBased) {
            width = parseFloat(height * aspectRatio);
          } else {
            height = parseFloat(width / aspectRatio);
          }

          if (triggered) {
            if ((width > holderWidth || height > holderHeight) ||
                (width < 20 || height < 20)) {
              var hWidth = holderWidth * scalingFactor;
              var hHeight = holderHeight * scalingFactor;

              var scale = hHeight / height;
              if (width / height > holderWidth / hHeight) {
                scale = hWidth / width;
              }

              width *= scale;
              height *= scale;
            }
          }

          width = width.toFixed(2);
          height = height.toFixed(2);

          $imageViewW.val(width);
          $imageViewH.val(height);

          $interactHolder.css({
            width: width + 'px',
            height: height + 'px'
          });

          if (triggered) {
            interactDrag(true);
          }
        };

        var interactDrag = function(center) {
          var x = parseFloat($imageViewX.val());
          var y = parseFloat($imageViewY.val());
          if (center) {
            x = (holderWidth - parseFloat($imageViewW.val())) / 2;
            y = (holderHeight - parseFloat($imageViewH.val())) / 2;
          }

          if (!x || !isFinite(x)) {
            return;
          }
          x = x.toFixed(2);

          if (!y || !isFinite(y)) {
            return;
          }
          y = y.toFixed(2);

          $interactHolder.attr({'data-x': x, 'data-y': y}).css({
              transform: 'translate(' + x + 'px, ' + y + 'px)',
              webkitTransform: 'translate(' + x + 'px, ' + y + 'px)'
          });
        };

        $viewW.on('change', function(event) {
          var aspectRatio = parseFloat($viewW.val()) / parseFloat($viewH.val());
          if (!aspectRatio || !isFinite(aspectRatio)) {
            return;
          }

          var val = parseFloat($imageViewW.val());
          if (!val || !isFinite(val)) {
            val = holderWidth * scalingFactor;
            $imageViewW.val(val);
          }

          val *= aspectRatio;
          if (!val || !isFinite(val)) {
            return;
          }

          $imageViewH.val(val.toFixed(2));
          interactResize(true, true);
        });

        $viewH.on('change', function(event) {
          var aspectRatio = parseFloat($viewW.val()) / parseFloat($viewH.val());
          if (!aspectRatio || !isFinite(aspectRatio)) {
            return;
          }

          var val = parseFloat($imageViewH.val());
          if (!val || !isFinite(val)) {
            val = holderHeight * scalingFactor;
            $imageViewH.val(val);
          }

          val *= aspectRatio;
          if (!val || !isFinite(val)) {
            return;
          }

          $imageViewW.val(val.toFixed(2));
          interactResize(false, true);
        });

        $imageViewW.on('change', function(event) {
          interactResize(false, !event.isTrigger);
        });

        $imageViewH.on('change', function(event) {
          interactResize(true, !event.isTrigger);
        });

        $imageViewX.on('change', function(event) {
          interactDrag();
        });

        $imageViewY.on('change', function(event) {
          interactDrag();
        });

        $el.find('.remove_view_image').click(function(e) {
          e.preventDefault();

          $el.toggleClass('added', false);
          $imageID.val('0').change();
          $imageH.val('280').change();
          $imageW.val('250').change();
          $holder.removeAttr('style');
        });

        $el.find('.set_view_image').click(function(e) {
          e.preventDefault();

          // If the media frame already exists, reopen it.
          if (frame) {
            frame.open();
            return;
          }

          // Create a new media frame
          frame = wp.media({
            title: 'Select or Upload an Image',
            button: {
              text: 'Use this media'
            },
            multiple: false  // Set to true to allow multiple files to be selected
          });

          // When an image is selected in the media frame...
          frame.on('select', function() {

            // Get media attachment details from the frame state
            var attachment = frame.state().get('selection').first().toJSON();

            if ( attachment.id ) {
              $.ajax({
                method: 'POST',
                url: ajaxurl,
                dataType: 'json',
                data: {
                  action: 'udesign_get_designer_view_thumbnail',
                  attachment_id: attachment.id
                }
              }).done(function(image) {
                var url = image[0];
                var height = image[2];
                var width = image[1];

                $el.toggleClass('added', true);
                $imageID.val(attachment.id).change();
                $imageH.val(height).change();
                $imageW.val(width).change();
                $holder.css({
                  'backgroundImage': 'url(' + url + ')',
                  'backgroundSize': width + 'px' + height + 'px',
                  'height': height,
                  'width': width
                });

                holderWidth = width;
                holderHeight = height;
              });
            }
          });

          // Finally, open the modal on click
          frame.open();
          return false;
        });
      });

      $( 'a.add_media' ).on( 'click', this.restore_wp_media_post_id );
    },

    /**
     * Enable Interact.js on elements
     */
    enable_interact: function(el) {
      interact(el)
        .draggable({
          restrict: {
            restriction: 'parent',
            endOnly: false,
            elementRect: {
              left: 0,
              right: 1,
              top: 0,
              bottom: 1
            }
          }
        })
        .resizable({
          preserveAspectRatio: true,
          restrict: {
            restriction: 'parent',
            endOnly: false,
            elementRect: {
              left: 0,
              right: 1,
              top: 0,
              bottom: 1
            }
          },
          edges: {
            left: true,
            right: true,
            bottom: true,
            top: true
          }
        })
        .on('dragmove', function(event) {
          var target = $(event.target);
          var holder = target.closest('.designer_view_editor');
          var x = ((parseFloat(target.attr('data-x')) || 0) + parseFloat(event.dx)).toFixed(2);
          var y = ((parseFloat(target.attr('data-y')) || 0) + parseFloat(event.dy)).toFixed(2);

          target.css({
            transform: 'translate(' + x + 'px, ' + y + 'px)',
            webkitTransform: 'translate(' + x + 'px, ' + y + 'px)'
          });

          target.attr('data-x', x);
          target.attr('data-y', y);
          holder.find(':input.image-view-x').val(x).change();
          holder.find(':input.image-view-y').val(y).change();
        })
        .on('resizemove', function(event) {
          var target = $(event.target);
          var holder = target.closest('.designer_view_editor');
          var width = parseFloat(event.rect.width).toFixed(2);
          var height = parseFloat(event.rect.height).toFixed(2);
          var x = ((parseFloat(target.attr('data-x')) || 0) + parseFloat(event.deltaRect.left)).toFixed(2);
          var y = ((parseFloat(target.attr('data-y')) || 0) + parseFloat(event.deltaRect.top)).toFixed(2);

          target.css({
            width: width + 'px',
            height: height + 'px',
            transform: 'translate(' + x + 'px, ' + y + 'px)',
            webkitTransform: 'translate(' + x + 'px, ' + y + 'px)'
          });

          target.attr('data-x', x);
          target.attr('data-y', y);
          holder.find(':input.image-view-w').val(width).change();
          holder.find(':input.image-view-h').val(height).change();
          holder.find(':input.image-view-x').val(x).change();
          holder.find(':input.image-view-y').val(y).change();
        });
    },

    /**
     * Restore wp.media post ID.
     */
    restore_wp_media_post_id: function() {
      wp.media.model.settings.post.id = designer_views_media.wp_media_post_id;
    }
  };

  /**
   * Product designer views metabox ajax methods
   */
  var designer_views_ajax = {

    /**
     * Initialize designer views ajax methods
     */
    init: function() {
      $( 'li.designer_views_tab a' ).on( 'click', this.initial_load );

      $( '#designer_product_data' )
        .on( 'click', 'button.save-designer-views-changes', this.save_designer_views )
        .on( 'click', 'button.cancel-designer-views-changes', this.cancel_designer_views );

      $( document.body )
        .on( 'change', '#designer_product_data .designer_views :input', this.input_changed );

      $( 'form#post' ).on( 'submit', this.save_on_submit );

      $( '.wc-metaboxes-wrapper' ).on( 'click', 'a.do_designer_views_action', this.do_designer_views_action );
    },

    /**
     * Check if have some changes before leave the page
     *
     * @return {Bool}
     */
    check_for_changes: function() {
      var need_update = $( '#designer_product_data' ).find( '.designer_views .designer-view-needs-update' );

      if ( 0 < need_update.length ) {
        if ( window.confirm( admin_meta_boxes_designer_views.i18n_edited_designer_views ) ) {
          designer_views_ajax.save_changes();
        } else {
          need_update.removeClass( 'designer-view-needs-update' );
          return false;
        }
      }

      return true;
    },

    /**
     * Block edit screen
     */
    block: function() {
      $( '#woocommerce-product-data' ).block({
        message: null,
        overlayCSS: {
          background: '#fff',
          opacity: 0.6
        }
      });
    },

    /**
     * Unblock edit screen
     */
    unblock: function() {
      $( '#woocommerce-product-data' ).unblock();
    },

    /**
     * Initial load designer views
     *
     * @return {Bool}
     */
    initial_load: function() {
      if ( 0 === $( '#designer_product_data' ).find( '.designer_views .designer_view' ).length ) {
        designer_views_ajax.load_designer_views();
      }
    },

    /**
     * Load designer views via Ajax
     *
     * @param {Int} page (default: 1)
     * @param {Int} per_page (default: 10)
     */
    load_designer_views: function( page, per_page ) {
      page     = page || 1;
      per_page = per_page || 100;

      var wrapper = $( '#designer_product_data' ).find( '.designer_views' );

      designer_views_ajax.block();
      $.ajax({
        url: admin_meta_boxes_designer_views.ajax_url,
        data: {
          action:     'udesign_load_designer_views',
          security:   admin_meta_boxes_designer_views.load_designer_views_nonce,
          product_id: admin_meta_boxes_designer_views.post_id,
          attributes: wrapper.data( 'attributes' ),
          page:       page,
          per_page:   per_page
        },
        type: 'POST',
        success: function( response ) {
          wrapper.empty().append( response ).attr( 'data-page', page );

          $( '#woocommerce-product-data' ).trigger( 'designer_views_loaded' );

          var views = wrapper.find( '.designer_view' ).length;
          designer_views_ajax.update_designer_views_count( views );
          designer_views_ajax.unblock();
        }
      });
    },

    /**
     * Set variations count
     *
     * @param {Int} qty
     *
     * @return {Int}
     */
    update_designer_views_count: function( qty ) {
      qty  = parseInt( qty, 10 ) || 0;
      var wrapper         = $( '#designer_product_data' );
      var views           = wrapper.find( '.designer_views' );
      var toolbars        = wrapper.find( '.toolbar' );
      var displaying_num  = wrapper.find( '.displaying-num' );

      // Set the new total of variations
      views.attr( 'data-total', qty );
      toolbars.toggle( qty > 0 );

      if ( 1 === qty ) {
        displaying_num.text( admin_meta_boxes_designer_views.i18n_designer_view_count_single.replace( '%qty%', qty ) );
      } else {
        displaying_num.text( admin_meta_boxes_designer_views.i18n_designer_view_count_plural.replace( '%qty%', qty ) );
      }

      return qty;
    },

    /**
     * Ger designer views fields and convert to object
     *
     * @param  {Object} fields
     *
     * @return {Object}
     */
    get_designer_views_fields: function( fields ) {
      var data = $( ':input', fields ).serializeJSON();

      return data;
    },

    /**
     * Save designer views changes
     *
     * @param {Function} callback Called once saving is complete
     */
    save_changes: function( callback ) {
      var wrapper     = $( '#designer_product_data' ).find( '.designer_views' );
      var need_update = $( '.designer-view-needs-update', wrapper );
      var data        = {};

      // Save only with views need update.
      if ( 0 < need_update.length ) {
        designer_views_ajax.block();

        data                 = designer_views_ajax.get_designer_views_fields( wrapper );
        data.action          = 'udesign_save_designer_views';
        data.security        = admin_meta_boxes_designer_views.save_designer_views_nonce;
        data.product_id      = admin_meta_boxes_designer_views.post_id;
        data['product-type'] = $( '#product-type' ).val();

        $.ajax({
          url: admin_meta_boxes_designer_views.ajax_url,
          data: data,
          type: 'POST',
          success: function( response ) {
            // Allow change page, delete and add new designer views
            need_update.removeClass( 'designer-view-needs-update' );
            $( 'button.cancel-designer-views-changes, button.save-designer-views-changes' ).attr( 'disabled', 'disabled' );

            $( '#woocommerce-product-data' ).trigger( 'designer_views_saved' );

            if ( typeof callback === 'function' ) {
              callback( response );
            }

            designer_views_ajax.unblock();
          }
        });
      }
    },

    /**
     * Save designer views
     *
     * @return {Bool}
     */
    save_designer_views: function() {
      $( '#designer_product_data' ).trigger( 'designer_views_save_designer_views_button' );

      designer_views_ajax.save_changes( function( error ) {
        var wrapper = $( '#designer_product_data' ).find( '.designer_views' );

        $( '#designer_product_data' ).find( '#woocommerce_errors' ).remove();

        if ( error ) {
          wrapper.before( error );
        }
      });

      return false;
    },

    /**
     * Save on post form submit
     */
    save_on_submit: function( e ) {
      var need_update = $( '#designer_product_data' ).find( '.designer_views .designer-view-needs-update' );

      if ( 0 < need_update.length ) {
        e.preventDefault();
        $( '#designer_product_data' ).trigger( 'designer_views_save_designer_views_on_submit' );
        designer_views_ajax.save_changes( designer_views_ajax.save_on_submit_done );
      }
    },

    /**
     * After saved, continue with form submission
     */
    save_on_submit_done: function() {
      $( 'form#post' ).submit();
    },

    /**
     * Discard changes.
     *
     * @return {Bool}
     */
    cancel_designer_views: function() {
      $( '#designer_product_data' ).find( '.designer_views .designer-view-needs-update' )
        .removeClass( 'designer-view-needs-update' );

      return false;
    },

    /**
     * Link all designer views (or at least try :p)
     *
     * @return {Bool}
     */
    link_all_designer_variations: function() {
      designer_views_ajax.check_for_changes();

      if ( window.confirm( woocommerce_admin_meta_boxes_variations.i18n_link_all_variations ) ) {
        var regular_price = window.prompt( woocommerce_admin_meta_boxes_variations.i18n_enter_a_value );
        designer_views_ajax.block();

        var data = {
          action: 'woocommerce_link_all_variations',
          post_id: woocommerce_admin_meta_boxes_variations.post_id,
          security: woocommerce_admin_meta_boxes_variations.link_variation_nonce
        };

        $.post( woocommerce_admin_meta_boxes_variations.ajax_url, data, function( response ) {

          $.ajax({
            url: woocommerce_admin_meta_boxes_variations.ajax_url,
            data: {
              action:       'woocommerce_bulk_edit_variations',
              security:     woocommerce_admin_meta_boxes_variations.bulk_edit_variations_nonce,
              product_id:   woocommerce_admin_meta_boxes_variations.post_id,
              product_type: $( '#product-type' ).val(),
              bulk_action:  'variable_designer_view_images',
              data:         {}
            },
            type: 'POST',
            complete: function() {
              if ( regular_price ) {
                $.ajax({
                  url: woocommerce_admin_meta_boxes_variations.ajax_url,
                  data: {
                    action:       'woocommerce_bulk_edit_variations',
                    security:     woocommerce_admin_meta_boxes_variations.bulk_edit_variations_nonce,
                    product_id:   woocommerce_admin_meta_boxes_variations.post_id,
                    product_type: $( '#product-type' ).val(),
                    bulk_action:  'variable_regular_price',
                    data:         { value: regular_price }
                  },
                  type: 'POST',
                  complete: function() {
                    $( '#variable_product_options' ).trigger('reload');
                    $('ul.wc-tabs li.variations_tab a').click();
                  }
                });
              } else {
                $( '#variable_product_options' ).trigger('reload');
                $('ul.wc-tabs li.variations_tab a').click();
              }
            }
          });
        });
      }

      return false;
    },

    /**
     * Add new class when have changes in some input
     */
    input_changed: function(event) {
      $( this )
        .closest( '.designer_view' )
        .addClass( 'designer-view-needs-update' );

      $( 'button.cancel-designer-views-changes, button.save-designer-views-changes' ).removeAttr( 'disabled' );

      $( '#designer_product_data' ).trigger( 'designer_views_input_changed' );
    },

    /**
     * Actions
     */
    do_designer_views_action: function() {
      var do_designer_views_action = $( 'select.designer_views_actions' ).val(),
        data       = {},
        changes    = 0,
        value;

      switch ( do_designer_views_action ) {
        case 'link_all_designer_variations' :
          designer_views_ajax.link_all_designer_variations();
          return;
        default :
          break;
      }
    }
  };

  designer_views_actions.init();
  designer_views_media.init();
  designer_views_ajax.init();

});
