jQuery(function($) {
  'use strict';

  var products_import = {
    container_selector: '.product-items',
    item_selector: '.product-item',
    item_width: 200,

    init: function() {
      $(document).ready(this.ready);
      $(window).on('resize', this.resize);
    },

    ready: function() {
      $(document).on('click', products_import.item_selector, function(e) {
        products_import.select_item($(this));
        products_import.check_selection();
      });

      $(document).on('click', '.product-checkbox', function(e) {
        e.stopPropagation();

        var el = $(this).closest(products_import.item_selector);
        products_import.select_item(el);
        products_import.check_selection();
      });

      $(document).on('click', '#select', function(e) {
        products_import.select_items();
        products_import.check_selection();
      });

      $(document).on('click', '.pagination-link', products_import.change_page);
      $(document).on('click', '#import', products_import.import_products);

      products_import.resize();
    },

    resize: function() {
      var grid = $(products_import.container_selector);
      if (!grid.length) {
        return;
      }

      var grid_items = grid.find(products_import.item_selector);
      if (!grid_items.length) {
        return;
      }

      var grid_width = Math.ceil(grid.width());
      if (grid_width < 500) {
        grid_items.css({
          'width': '98%',
          'margin': '1%'
        });

        return;
      }

      var items = Math.floor(grid_width / products_import.item_width);
      grid_items.css({
        'width': (96.0 / items) + '%',
        'margin': (2 / items) + '%'
      });
    },

    select_item: function(el, value) {
      if (!value) {
        value = !el.hasClass('selected');
      }

      el.toggleClass('selected', value);
      el.find('.product-checkbox').prop('checked', value);
    },

    select_items: function() {
      var items = $(products_import.item_selector);

      var select_all = items.length != items.filter('.selected').length;
      for (var i = 0; i < items.length; i++) {
        products_import.select_item($(items[i]), select_all);
      }
    },

    check_selection: function () {
      var items = $(products_import.item_selector);
      if (!items.length) {
        return;
      }

      var count_selected = items.filter('.selected').length;
      var select_msg = products_import_data.messages.select;
      if (items.length == count_selected) {
        select_msg = products_import_data.messages.deselect;
      }

      $('#select').text(select_msg);
      $('#import').attr('disabled', !count_selected);
    },

    change_page: function() {
      var value = $(this).data('page');
      if (!value) {
        return;
      }

      $('input[name="paged"]').val(value);
      $('#posts-filter').submit();
    },

    import_products: function() {
      var queue = [];
      $(products_import.item_selector + '.selected').each(function(i, val) {
        var id = $(val).data('id');
        if (!id) {
          return;
        }

        queue.push(id);
      });

      products_import.import_product(queue);
    },

    import_product: function(queue, cb) {
      if (!queue || !queue.length) {
        if (cb) {
          cb();
        }
        return;
      }

      var id = queue.shift();
      if (!id) {
        products_import.import_product(queue);
        return;
      }

      var el = $(products_import.item_selector + '[data-id="' + id + '"]');

      udesign.ui.block(el, products_import_data.messages.importing_product);
      jQuery.ajax({
        url: products_import_data.ajax_url,
        type: 'POST',
        data: {
          product_id: id,
          action: 'udesign_import_product',
          security: products_import_data.security.product_import_nonce
        }
      }).always(function(response) {
        if (!response || !response.success || !response.data ||
            !response.data.product_id || !response.data.metadata) {
          products_import.set_product_status(el, 'status-failed');
          products_import.import_product(queue);
          return;
        }

        var product_id = response.data.product_id;
        var metadata = response.data.metadata;
        var variations = response.data.variations ? response.data.variations : [];

        udesign.ui.unblock(el);
        products_import.import_variation(product_id, id, metadata, variations, el, 1, function() {
          products_import.set_product_status(el, 'status-success');
          products_import.import_product(queue);
        });
      });
    },

    import_variation: function(product_id, remote_product_id, metadata, queue, el, counter, cb) {
      if (!queue || !queue.length) {
        if (cb) {
          cb();
        }
        return;
      }

      var id = queue.shift();
      if (!id) {
        products_import.import_variation(product_id, remote_product_id, metadata, queue, el, ++counter, cb);
        return;
      }

      udesign.ui.block(el, products_import_data.messages.importing_variation + ' #' + counter);
      jQuery.ajax({
        url: products_import_data.ajax_url,
        type: 'POST',
        data: {
          product_id: product_id,
          remote_product_id: remote_product_id,
          variation_id: id,
          metadata: metadata,
          action: 'udesign_import_variation',
          security: products_import_data.security.product_import_nonce
        }
      }).always(function(response) {
        udesign.ui.unblock(el);

        if ( !response || !response.success ) {
          console.log( 'Failed to import variation ' + id );
        }

        products_import.import_variation(product_id, remote_product_id, metadata, queue, el, ++counter, cb);
      });
    },

    set_product_status: function( el, statusCls ) {
      if (el.hasClass('selected')) {
        el.click();
      }

      el.toggleClass(statusCls, true);
      el.find('.' + statusCls).toggleClass('hidden', false);
      udesign.ui.unblock(el);
    }
  };

  products_import.init();
});
