/* global single_customize_params */
jQuery( function( $ ) {
  'use strict';

  // single_customize_params is required to continue, ensure the object exists
  if ( typeof single_customize_params === 'undefined' ) {
    return false;
  }

  $( document.body )
    .on( 'init_single_customize_button', function() {
      var single_customize_button = $( '.single_customize_button' );
      if ( !single_customize_button.length ) {
        return false;
      }

      if ( !single_customize_params.designer_page_url ) {
        single_customize_button.hide();
        return false;
      }

      var variations_form = single_customize_button.closest('.variations_form');
      if ( !variations_form.length ) {
        return false;
      }

      variations_form.on( 'show_variation', function( event, variation, purchasable ) {
        single_customize_button.toggleClass( 'disabled', !purchasable );
      });

      variations_form.on( 'hide_variation', function( event ) {
        single_customize_button.toggleClass( 'disabled', true );
      });

      var single_add_to_cart_button = variations_form.find( '.single_add_to_cart_button' );
      single_customize_button.on( 'click', function( event ) {
        event.preventDefault();
        if ( single_add_to_cart_button.is( '.wc-variation-is-unavailable' ) ) {
          return window.alert( single_customize_params.i18n_unavailable_text );
        } else if ( single_add_to_cart_button.is('.wc-variation-selection-needed' ) ) {
          return window.alert( single_customize_params.i18n_make_a_selection_text );
        }

        var variation_input = variations_form.find( 'input[name="variation_id"]' ).first();
        if ( !variation_input.length ) {
          return window.alert( single_customize_params.i18n_make_a_selection_text );
        }

        var variation_id = parseInt( variation_input.val(), 10 );
        if ( !variation_id ) {
          return window.alert( single_customize_params.i18n_make_a_selection_text );
        }

        var design_id = 0;
        var design_input = variations_form.find( 'input[name="design_id"]' ).first();
        if ( design_input.length ) {
          design_id = parseInt( design_input.val(), 10 ) || 0;
        }

        window.location.href = single_customize_params.designer_page_url + '?' +
          $.param({product_id: variation_id, design_id: design_id});
      });
    }).trigger( 'init_single_customize_button' );

});
