/**
 * uDesign
 */
var udesign = {
  ui: {
    /**
     * Show admin notice
     */
    showNotice: function(container, msg, type, dismissible) {
      var cls = 'notice notice-' + type + (dismissible === false ? '' : ' is-dismissible');
      jQuery(container).append('<div class="' + cls + '"><p>' + msg + '</p></div>');
      jQuery(document).trigger('wp-updates-notice-added');
    },

    /**
     * Block ui
     */
    block: function(container, message) {
      jQuery(container).block({
        message: message,
        overlayCSS: {
          background: '#fff',
          opacity: 0.6
        }
      });
    },

    /**
     * Unblock ui
     */
    unblock: function(container) {
      jQuery(container).unblock();
    }
  },

  blob: {
    /**
     * Convert svg image to png blob
     */
    fromSVG: function(svg, minWidth, cb, canvas) {
      var img = new Image();
      img.setAttribute('crossOrigin', 'Anonymous');
      img.addEventListener('load', function() {
        canvas = canvas ? canvas : document.createElement('canvas');

        var scale = (img.width < minWidth) ? minWidth / img.width : 1;
        canvas.width = img.width * scale;
        canvas.height = img.height * scale;

        var ctx = canvas.getContext('2d');
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.drawImage(img, 0, 0);

        canvas.toBlob(function(blob) {
          cb(blob);
          canvas = null;
        }, 'image/png');
      }, false);

      img.src = svg;
    },

    /**
     * Upload png blob attachment
     */
    upload: function(data, cb) {
      var form = new FormData();
      form.append('action', 'udesign_upload_blob_attachment');
      form.append('security', data.security);
      form.append('post_id', data.postID);
      form.append('post_type', data.postType);
      form.append('image', data.image.data, data.image.name + '.png');
      form.append('set_thumbnail', data.setThumbnail || 0);

      jQuery.ajax({
        url: vam_udesign_params.ajax_url,
        data: form,
        type: 'POST',
        processData: false,
        contentType: false,
      }).always(cb);
    }
  },

  designs: {
    /**
     * Upload designer views
     */
    upload: function(data, cb) {
      var form = new FormData();
      form.append('post_id', data.id);
      form.append('action', 'udesign_upload_design_views');
      form.append('security', data.security);

      var views = data.views;
      for(var view in views) {
        if (!views[view]) {
          continue;
        }

        var blob = window.dataURLtoBlob(views[view]);
        form.append('views[]', blob, view);
      }

      jQuery.ajax({
        url: vam_udesign_params.ajax_url,
        data: form,
        type: 'POST',
        processData: false,
        contentType: false,
      }).always(cb);
    },

    /**
     * Download designer views
     */
    download: function(data, cb) {
      var form = new FormData();
      form.append('post_id', data.id);
      form.append('action', 'udesign_download_design_views');
      form.append('security', data.security);
      form.append('designs', data.designs);

      var views = data.views;
      for(var view in views) {
        if (!views[view]) {
          continue;
        }

        var blob = window.dataURLtoBlob(views[view]);
        form.append('views[]', blob, view);
      }

      jQuery.ajax({
        url: vam_udesign_params.ajax_url,
        data: form,
        type: 'POST',
        processData: false,
        contentType: false,
      }).always(cb);
    },

    /**
     * Add design to cart
     */
    add_to_cart: function(data, $button, cb) {
      if (!$button) {
        $button = jQuery();
      }

      // Trigger event
      jQuery('body').trigger('adding_to_cart', [ $button, data ]);

      // Ajax action
      jQuery.ajax({
        url: vam_udesign_params.ajax_url,
        data: data,
        type: 'POST',
      }).always(function(response) {
        cb = cb || function(){};

        if (!response) {
          if ($button && $button.length) {
            $button.removeClass('loading');
          }

          cb(response);
          return;
        }

        var this_page = window.location.toString();
        this_page = this_page.replace('add-to-cart', 'added-to-cart');

        if (response.success === false) {
          jQuery('.vam_udesign_messages').html('<div class="woocommerce-error">' +
            response.data.message + '</div>');

          if ($button && $button.length) {
            $button.removeClass('loading');
          }

          cb(response);
          return;
        }

        if ($button && $button.length) {
          $button.removeClass('loading');
        }

        var fragments = response.fragments;
        var cart_hash = response.cart_hash;

        // Block fragments class
        if (fragments) {
          jQuery.each(fragments, function(key) {
            jQuery(key).addClass('updating');
          });
        }

        // Block widgets and fragments
        jQuery('.shop_table.cart, .updating, .cart_totals').fadeTo('400', '0.6').block({
          message: null,
          overlayCSS: {
            opacity: 0.6
          }
        });

        // Changes button classes
        if ($button && $button.length) {
          $button.addClass('added');
        }

        // Replace fragments
        if (fragments) {
          jQuery.each(fragments, function(key, value) {
            jQuery(key).replaceWith(value);
          });
        }

        // Unblock
        jQuery('.widget_shopping_cart, .updating').stop(true).css('opacity', '1').unblock();

        // Cart page elements
        jQuery('.shop_table.cart').load(this_page + ' .shop_table.cart:eq(0) > *', function() {
          jQuery('.shop_table.cart').stop(true).css('opacity', '1').unblock();
          jQuery(document.body).trigger('cart_page_refreshed');
        });

        jQuery('.cart_totals').load(this_page + ' .cart_totals:eq(0) > *', function() {
          jQuery('.cart_totals').stop(true).css('opacity', '1').unblock();
        });

        // Trigger event so themes can refresh other areas
        jQuery(document.body).trigger('added_to_cart', [ fragments, cart_hash, $button ]);

        cb(response);
      });
    }
  },

  posts: {
    /**
     * Set post thumbnail
     */
    setThumb: function(data, cb) {
      jQuery.ajax({
        url: vam_udesign_params.ajax_url,
        type: 'POST',
        data: {
          action: 'udesign_set_post_thumbnail',
          post_id: data.postID,
          thumbnail_id: data.attachmentID,
          security: data.security
        },
        success: cb
      });
    }
  }
};
