<?php

/**
 * Fired during plugin activation
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

class VAM_UDesign_Activator {

  /**
   * Short Description. (use period)
   *
   * Long Description.
   *
   * @since    1.0.0
   */
  public static function activate() {
    self::add_htaccess_rules();
    self::add_default_pages();
    self::migrate_data();
  }

  public static function add_htaccess_rules() {
    if ( !got_mod_rewrite() ) {
      return;
    }

    $home_path = get_home_path();
    if ( !is_writable( $home_path ) ) {
      return;
    }

    $rules = array (
      "<IfModule mod_headers.c>",
      "<FilesMatch '\.(ico|gif|jpg|jpeg|png|svg|ttf|eot|woff|woff2)$'>",
      "Header set Access-Control-Allow-Origin '*'",
      "</FilesMatch>",
      "</IfModule>"
    );

    insert_with_markers( $home_path . '.htaccess', 'VAM_UDESIGN', $rules );
  }

  public static function add_default_pages() {
    $pages          = get_pages( array(
      'post_status' => 'publish,draft,trash'
    ) );
    $designer_page  = null;
    $designs_page   = null;
    if ( ! empty( $pages ) ) {
      foreach ( $pages as $page ) {
        if ( has_shortcode( $page->post_content, 'udesign_designer' ) ) {
          $designer_page = $page;
        } elseif ( 'designer' === $page->post_name ) {
          $designer_page = $page;
        } elseif ( 'designs' === $page->post_name ) {
          $designs_page = $page;
        }
      }
    }

    $designer_post = array(
      'post_title'    => __( 'Designer', 'vam-udesign' ),
      'post_type'     => 'page',
      'post_status'   => 'publish',
      'post_content'  => '[udesign_designer]',
    );

    $designer_page_id = null;
    if ( ! empty( $designer_page ) ) {
      $designer_post['ID'] = $designer_page->ID;
      $designer_page_id = wp_update_post( $designer_post, true );
    } else {
      $designer_page_id = wp_insert_post( $designer_post, true );
    }

    if ( ! empty( $designer_page_id ) && ! is_wp_error( $designer_page_id ) ) {
      update_option( 'udesign_designer_page_id', $designer_page_id, true );
    }

    $designs_post = array(
      'post_title'    => __( 'Designs', 'vam-udesign' ),
      'post_type'     => 'page',
      'post_status'   => 'publish',
    );

    $designs_page_id = null;
    if ( ! empty( $designs_page ) ) {
      $designs_post['ID'] = $designs_page->ID;
      $designs_page_id = wp_update_post( $designs_post, true );
    } else {
      $designs_page_id = wp_insert_post( $designs_post, true );
    }

    if ( ! empty( $designs_page_id ) && ! is_wp_error( $designs_page_id ) ) {
      update_option( 'udesign_designs_page_id', $designs_page_id, true );
    }
  }

  public static function migrate_data() {
    require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-vam-udesign-migrations.php';
    VAM_UDesign_Migrations::migrate_product_attributes();

    set_transient( 'vam-udesign-execute-migrations', true, 0 );
  }
}
