<?php

class VAM_UDesign_Ajax {

  private static $instance;

  public static function instance() {
    if ( self::$instance == null ) {
      self::$instance = new VAM_UDesign_Ajax;
    }
  }

  private function __construct() {
    self::add_ajax_events();

    add_action( 'woocommerce_delete_product_transients', array( $this, 'delete_product_transients' ), 10, 1 );
    add_action( 'woocommerce_bulk_edit_variations', array( $this, 'variable_designer_view_images'), 10, 4 );
  }

  /**
   * Hook in methods - uses WordPress ajax handlers (admin-ajax).
   */
  public static function add_ajax_events() {
    // EVENT                        => nopriv
    $ajax_events = array(
      'get_product'                 => true,
      'get_product_details'         => true,
      'get_product_variations'      => true,
      'get_product_gallery'         => true,
      'load_designer_product'       => true,
      'get_designer_view_thumbnail' => false,
      'load_designer_views'         => false,
      'save_designer_views'         => false,
      'get_product_categories'      => false,
      'get_clipart_categories'      => false,
      'get_design_categories'       => false,
      'get_products'                => true,
      'get_cliparts'                => true,
      'get_designs'                 => true,
      'get_my_designs'              => true,
      'get_clipart'                 => true,
      'get_design'                  => true,
      'delete_my_design'            => true,
      'import_design'               => false,
      'import_product'              => false,
      'import_variation'            => false,
      'upload_clipart'              => true,
      'create_clipart'              => false,
      'create_design'               => false,
      'create_cart_design'          => true,
      'create_user_design'          => true,
      'upload_blob_attachment'      => true,
      'set_post_thumbnail'          => false,
      'upload_design_views'         => true,
      'download_design_views'       => false,
      'get_design_preview'          => true,
      'get_design_price'            => true,
      'load_edit_color_attribute'   => false,
      'save_color_attribute'        => false,
      'add_to_cart'                 => true,
      'feature_design'              => false,
    );

    foreach ( $ajax_events as $ajax_event => $nopriv ) {
      add_action( 'wp_ajax_udesign_' . $ajax_event, array( __CLASS__, $ajax_event ) );

      if ( $nopriv ) {
        add_action( 'wp_ajax_nopriv_udesign_' . $ajax_event, array( __CLASS__, $ajax_event ) );
      }
    }
  }

  public function delete_product_transients( $product_id ) {
    delete_transient( 'udesign_product_variations_' . $product_id );
  }

  public static function send_json( $response, $status_code = null ) {
    @header( 'Content-Type: application/json; charset=' . get_option( 'blog_charset' ) );
    if ( null !== $status_code ) {
      status_header( $status_code );
    }

    echo $response;
    if ( wp_doing_ajax() ) {
      wp_die( '', '', array( 'response' => null ) );
    } else {
      die;
    }
  }

  public static function send_json_error( $message ) {
    wp_send_json_error( array (
      'message' => __( $message, 'vam-udesign' )
    ) );
  }

  public static function error_response( $message ) {
    return array (
      'success' => false,
      'message' => __( $message, 'vam-udesign' )
    );
  }

  public static function parse_file_upload( $files ) {
      $parsed = array();

      foreach( $files as $key => &$attr ){
          foreach( $attr as $i => &$val ){
              $parsed[$i][$key] = $val;
          }
      }

      return $parsed;
  }

  public static function load_image( $path ) {
    $image_type = false;
    if ( function_exists( 'exif_imagetype' ) ) {
      $image_type = @exif_imagetype( $path );
    }

    if ( empty( $image_type ) ) {
      if ( !function_exists( finfo_file ) ) {
        return false;
      }

      $finfo = finfo_open( FILEINFO_MIME_TYPE );
      $image_type = finfo_file( $finfo, vam_udesign_path_from_url( $path ) );
      finfo_close( $finfo );
    }

    if ( empty( $image_type ) ) {
      return false;
    }

    switch ( $image_type ) {
    case 'image/gif':
    case IMAGETYPE_GIF:
      return imagecreatefromgif( $path );
    case 'image/jpeg':
    case IMAGETYPE_JPEG:
      return imagecreatefromjpeg( $path );
    case 'image/png':
    case IMAGETYPE_PNG:
      return imagecreatefrompng( $path );
    }

    return false;
  }

  public static function get_designer_variation_attributes( $variation_id ) {
    // get variation product
    $variation = wc_get_product( $variation_id );
    if ( empty( $variation ) ) {
      return self::error_response( 'Could not find the selected variation' );
    }
    if ( 'variation' != $variation->get_type() ) {
      return self::error_response( 'Invalid product selected' );
    }

    // get designer product
    $post = get_post( $variation_id );
    if ( empty( $post ) ) {
      return self::error_response( 'Could not find the selected variation post' );
    }
    $product_id = $post->post_parent;

    $parent_post = get_post( $product_id );
    if ( empty( $parent_post ) ) {
      return self::error_response( 'Could not find the selected product post' );
    }

    $product = new VAM_UDesign_Product( $product_id );
    if ( empty( $product ) ) {
      return self::error_response( 'Could not find the selected designer product' );
    }

    // get variation color
    $attributes = $variation->get_variation_attributes();
    if ( empty( $attributes['attribute_pa_color'] ) ) {
      return self::error_response( 'Could not find the selected color' );
    }

    $color_term = get_term_by( 'slug', $attributes['attribute_pa_color'], 'pa_color' );
    if ( empty( $color_term ) ) {
      return self::error_response( 'Could not find the selected color' );
    }

    // get product views
    $views = $product->get_views_meta();
    if ( empty( $views ) ) {
      return self::error_response( 'Could not find any product views' );
    }

    $color_id = $color_term->term_id;
    foreach( $views as $view ) {
      if ( empty( $view['colors'][$color_id] ) ) {
        return self::error_response( 'Product is not properly configured for the selected color' );
      }
    }

    $result = new stdClass();
    $result->product = $variation;
    $result->parent = $product;
    $result->post = $post;
    $result->parent_post = $parent_post;
    $result->attributes = $attributes;
    $result->views = $views;
    $result->color = $color_term;

    return array(
      'success' => true,
      'data' => $result
    );
  }

  public static function get_design_view_info( $design, $view, $color_id, $design_dims = array() ) {
    if ( empty( $design ) || empty( $view ) || empty( $color_id ) ) {
      return false;
    }
    if ( empty( $design['layers'] ) ) {
      return false;
    }

    $export_bounds = false;
    if ( !empty( $design['export']['bounds'] ) ) {
      $export_bounds = $design['export']['bounds'];
    }

    $use_export_data = true;
    if ( !is_array( $export_bounds ) ) {
      $use_export_data = false;
    } else {
      if ( empty( $export_bounds['width'] ) || empty( $export_bounds['height'] ) ||
        empty( $export_bounds['x'] ) || empty( $export_bounds['y'] ) ) {
        $use_export_data = false;
      }
    }

    $product_image_src = wp_get_attachment_image_src( $view['colors'][$color_id], 'full' );
    if ( empty( $product_image_src ) ) {
      return false;
    }
    $dest_width = intval( $product_image_src[1] );
    $dest_height = intval( $product_image_src[2] );

    // calculate view bounds
    $view_image_width = empty( $view['image_width'] ) ? 250 : intval( $view['image_width'] );
    $view_image_height = empty( $view['image_height'] ) ? 280 : intval( $view['image_height'] );

    $view_x = intval( $dest_width * floatval( $view['x'] ) / $view_image_width );
    $view_y = intval( $dest_height * floatval( $view['y'] ) / $view_image_height );
    $view_width = intval( $dest_width * floatval( $view['w'] ) / $view_image_width );
    $view_height = intval( $dest_height * floatval( $view['h'] ) / $view_image_height );

    if ( $use_export_data && !empty( $export_bounds['ratio'] ) ) {
      $old_ratio = round( floatval( $export_bounds['ratio'] ), 3 );
      if ( !empty( $view['view_width'] ) && !empty( $view['view_height'] ) ) {
        $cur_ratio = round( floatval( $view['view_width'] ) / floatval( $view['view_height'] ), 3 );
        $use_export_data = $cur_ratio == $old_ratio;
      }

      if ( !$use_export_data ) {
        if ( !empty( $view['image_id'] ) ) {
          $view_image_meta = wp_get_attachment_metadata( $view['image_id'] );
          if ( !empty( $view_image_meta ) ) {
            $vw = intval( intval( $view_image_meta['width'] ) * floatval( $view['w'] ) / $view_image_width );
            $vh = intval( intval( $view_image_meta['height'] ) * floatval( $view['h'] ) / $view_image_height );
            $use_export_data = round( $vw / $vh, 3 ) == $old_ratio;
          } else {
            $use_export_data = false;
          }
        } else {
          $use_export_data = false;
        }
      }

      if ( !$use_export_data ) {
        $use_export_data = round( $view_width / $view_height, 3 ) == $old_ratio;
      }
    } else {
      $use_export_data = false;
    }

    $bounds = new stdClass();
    $bounds->x = 0;
    $bounds->y = 0;
    $bounds->width = 0;
    $bounds->height = 0;

    if ( $use_export_data ) {
      $center_x = $view_x + $view_width / 2;
      $center_y = $view_y + $view_height / 2;

      $bounds->width = floatval( $export_bounds['width'] ) * $view_width;
      $bounds->height = floatval( $export_bounds['height'] ) * $view_height;
      $bounds->x = $center_x + $view_width * floatval( $export_bounds['x'] ) - $bounds->width / 2;
      $bounds->y = $center_y + $view_height * floatval( $export_bounds['y'] ) - $bounds->height / 2;
    } else {
      $src_width = 0;
      $src_height = 0;
      if ( empty( !$design_dims ) ) {
        $src_width = floatval( $design_dims['width'] );
        $src_height = floatval( $design_dims['height'] );
      } else {
        if ( empty( $export_bounds['original'] ) ) {
          return false;
        }

        $originalBounds = $export_bounds['original'];
        if ( empty( $originalBounds['width'] ) || empty( $originalBounds['height'] ) ) {
          return false;
        }

        $src_width = floatval( $originalBounds['width'] );
        $src_height = floatval( $originalBounds['height'] );
      }

      $scale = $view_height / $src_height;
      if ( $src_width / $src_height > $view_width / $view_height ) {
        $scale = $view_width / $src_width;
      }

      $bounds->width = $src_width * $scale;
      $bounds->height = $src_height * $scale;
      $bounds->x = $view_x + ( $view_width - $bounds->width ) / 2;
      $bounds->y = $view_y + ( $view_height - $bounds->height ) / 2;
    }

    // calculate design price
    $real_area = 0;
    if ( !empty( $view['view_width'] ) && !empty( $view['view_height'] ) ) {
      $real_area = floatval( $view['view_width'] ) * floatval( $view['view_height'] );
    }

    $real_min_area = 0;
    if ( !empty( $real_area ) && !empty( $view['view_min_percent'] ) ) {
      $real_min_area = floatval( $view['view_min_percent'] ) / 100.0 * $real_area;
    }

    $total_area = $view_width * $view_height;
    $area = $bounds->width * $bounds->height;

    $min_area = ( empty( $real_area ) || empty( $real_min_area ) ) ? 0 : $total_area * $real_min_area / $real_area;
    $area = ( $min_area > $area ) ? $min_area : $area;

    $result = new stdClass();
    $result->price = floatval( $view['view_price'] ) * $area / $total_area;
    $result->bounds = $bounds;

    return $result;
  }

  public static function generate_design_preview_data( $design_id, $designs, $design_views, $product_id, $variation, $views, $color_id, $custom_design, $cache_directory = 'designs' ) {
    if ( empty( $design_id ) ) {
      $design_id = intval( microtime( true ) * 1000000 );
    }

    $price = 0;
    $preview_urls = array();

    foreach( $design_views as $view_id => &$design_view ) {
      if ( empty( $views[$view_id] ) ) {
        continue;
      }
      $view = $views[$view_id];

      // get design from current view
      $design = false;
      foreach( $designs as &$d ) {
        if ( empty( $d['view'] ) ) {
          continue;
        }

        if ( $d['view'] == $view_id ) {
          $design = $d;
          break;
        }
      }

      if ( empty( $design ) ) {
        continue;
      }

      // get design image and its size
      if ( empty( $design_view['full'] ) ) {
        return false;
      }
      $design_image_url = $design_view['full'];

      $design_size = getimagesize( $design_image_url );
      if ( empty( $design_size ) ) {
        return false;
      }
      $src_width = intval( $design_size[0] );
      $src_height = intval( $design_size[1] );

      // get product image and its size
      $product_image_src = wp_get_attachment_image_src( $view['colors'][$color_id], 'full' );
      if ( empty( $product_image_src ) ) {
        return false;
      }
      $dest_width = $product_image_src[1];
      $dest_height = $product_image_src[2];

      // get design price and bounds on the current view
      $view_info = self::get_design_view_info( $design, $view,
        $color_id, array( 'width' => $src_width, 'height' => $src_height ) );
      if ( empty( $view_info ) ) {
        continue;
      }

      // generate preview image
      $upload_dir = wp_upload_dir();
      $relative_path = join( DIRECTORY_SEPARATOR, array( 'cache', 'vam-udesign', $cache_directory, strval( $design_id ), 'products', strval( $product_id ), 'views', strval( $view_id ) ) );

      $cache_path = join( DIRECTORY_SEPARATOR, array( $upload_dir['basedir'],  $relative_path ) );
      if ( !file_exists( $cache_path ) ) {
        mkdir( $cache_path, 0777, true );
      }

      $preview_basename = sprintf( 'product-%d', VAM_UDesign_WC_Compat::get_product_id( $variation ) );
      $preview_name = sprintf( '%s.png', $preview_basename );
      $preview_path = join( DIRECTORY_SEPARATOR, array( $cache_path, $preview_name ) );

      // check if the file exists or is in need of an update
      $generate_preview_image = false;
      if ( !$custom_design ) {
        if ( file_exists( $preview_path ) ) {
          $product_modified_time = get_post_modified_time( 'U', false, $product_id );
          $design_modified_time = get_post_modified_time( 'U', false, $design_id );

          $modified_time = filemtime( $preview_path );
          if ( $product_modified_time > $modified_time || $design_modified_time > $modified_time ) {
              $generate_preview_image = true;
          }
        } else {
          $generate_preview_image = true;
        }
      } else {
        $generate_preview_image = true;
      }

      // generate preview image
      if ( $generate_preview_image ) {
        $src = imagecreatefrompng( $design_image_url );
        if ( empty( $src ) ) {
          return false;
        }

        $dest = self::load_image( $product_image_src[0] );
        if ( empty( $dest ) ) {
          imagedestroy($src);
          return false;
        }
        if ( empty( imagealphablending( $dest, true ) ) ) {
          imagedestroy($src);
          imagedestroy($dest);
          return false;
        }
        if ( empty( imagesavealpha( $dest, true ) ) ) {
          imagedestroy($src);
          imagedestroy($dest);
          return false;
        }

        if ( empty( imagecopyresampled( $dest, $src, $view_info->bounds->x, $view_info->bounds->y, 0, 0,
          $view_info->bounds->width, $view_info->bounds->height, $src_width, $src_height ) ) ) {
          imagedestroy($src);
          imagedestroy($dest);
          return false;
        }

        if ( empty( imagepng( $dest, $preview_path ) ) ) {
          imagedestroy($src);
          imagedestroy($dest);
          return false;
        }

        imagedestroy($src);
        imagedestroy($dest);
      }

      // generate single preview image
      $single_size = vam_udesign_get_image_size( 'udesign_design_single' );
      $single_preview_name = sprintf( '%s-%dx%d.png', $preview_basename, intval( $single_size['width'] ), intval( $single_size['height'] ) );
      $single_preview_path = join( DIRECTORY_SEPARATOR, array( $cache_path, $single_preview_name ) );

      if ( $generate_preview_image || !file_exists( $single_preview_path ) ) {
        if ( empty( vam_udesign_generate_thumbnail( $preview_path, $single_preview_name, intval( $single_size['width'] ), intval( $single_size['height'] ), $single_preview_path ) ) ) {
          return false;
        }
      }

      // generate thumbnail preview image
      $thumbnail_size = vam_udesign_get_image_size( 'udesign_design_thumbnail' );
      $thumbnail_preview_name = sprintf( '%s-%dx%d.png', $preview_basename, intval( $thumbnail_size['width'] ), intval( $thumbnail_size['height'] ) );
      $thumbnail_preview_path = join( DIRECTORY_SEPARATOR, array( $cache_path, $thumbnail_preview_name ) );

      if ( $generate_preview_image || !file_exists( $thumbnail_preview_path ) ) {
        if ( empty( vam_udesign_generate_thumbnail( $preview_path, $thumbnail_preview_name, intval( $thumbnail_size['width'] ), intval( $thumbnail_size['height'] ), $thumbnail_preview_path ) ) ) {
          return false;
        }
      }

      $price += $view_info->price;

      $preview_urls[$view_id] = array(
        'full' => str_replace( '\\', '/', join( '/', array( $upload_dir['baseurl'], $relative_path, $preview_name ) ) ),
        'single' => str_replace( '\\', '/', join( '/', array( $upload_dir['baseurl'], $relative_path, $single_preview_name ) ) ),
        'thumbnail' => str_replace( '\\', '/', join( '/', array( $upload_dir['baseurl'], $relative_path, $thumbnail_preview_name ) ) )
      );
    }

    $price += floatval( $variation->get_price() );

    return array(
      'display_price' => $price,
      'price_html'    => wc_price( $price ),
      'preview_urls'  => $preview_urls
    );
  }

  public static function generate_design_download_image( $design, $design_view_path ) {
    if ( empty( $design ) || empty( $design_view_path ) ) {
      return false;
    }
    if ( empty( $design['export']['bounds'] ) ) {
      return false;
    }

    $export_bounds = $design['export']['bounds'];
    if ( !is_array( $export_bounds ) ) {
      return false;
    }
    if ( empty( $export_bounds['width'] ) || empty( $export_bounds['height'] ) ||
      empty( $export_bounds['x'] ) || empty( $export_bounds['y'] ) ) {
      return false;
    }

    // load source image
    $src = imagecreatefrompng( $design_view_path );
    if ( empty( $src ) ) {
      return false;
    }

    // calculate design position and dimensions
    $src_width = intval( imagesx( $src ) );
    $src_height = intval( imagesy( $src ) );

    $dest_width = intval( $src_width / floatval( $export_bounds['width'] ) );
    $dest_height = intval( $src_height / floatval( $export_bounds['height'] ) );

    $src_x = intval( $dest_width / 2 + $dest_width * floatval( $export_bounds['x'] ) - $src_width / 2 );
    $src_y = intval( $dest_height / 2 + $dest_height * floatval( $export_bounds['y'] ) - $src_height / 2 );

    if ( $dest_width < $src_width || $src_x < 0 ) {
      $dest_width = $src_width;
      $src_x = 0;
    }
    if ( $dest_height < $src_height || $src_y < 0 ) {
      $dest_height = $src_height;
      $src_y = 0;
    }

    // create destination image
    $dest = imagecreatetruecolor( $dest_width, $dest_height );
    if ( empty( $dest ) ) {
      imagedestroy( $src );
      return false;
    }
    if ( empty( imagealphablending( $dest, true ) ) ) {
      imagedestroy( $src );
      imagedestroy( $dest );
      return false;
    }
    if ( empty( imagesavealpha( $dest, true ) ) ) {
      imagedestroy( $src );
      imagedestroy( $dest );
      return false;
    }

    // give destination image a transparent background
    $background_color = imagecolorallocatealpha( $dest, 255, 255, 255, 127 );
    if ( empty( $background_color ) ) {
      imagedestroy( $src );
      imagedestroy( $dest );
      return false;
    }
    if ( empty( imagefill( $dest, 0, 0, $background_color ) ) ) {
      imagedestroy( $src );
      imagedestroy( $dest );
      return false;
    }

    // copy design on the destination image
    if ( empty( imagecopyresampled( $dest, $src, $src_x, $src_y, 0, 0, $src_width, $src_height, $src_width, $src_height ) ) ) {
      imagedestroy( $src );
      imagedestroy( $dest );
      return false;
    }
    imagedestroy( $src );

    ob_start();
    imagepng( $dest, null );
    $dest_data = ob_get_contents();
    ob_end_clean();

    imagedestroy( $dest );
    return $dest_data;
  }

  /**
   * Set default variation images by designer view color via AJAX.
   */
  public static function variable_designer_view_images( $bulk_action, $data, $product_id, $variations ) {
    if ( ! 'variable_designer_view_images' == $bulk_action ) {
      return;
    }

    $designer_views = maybe_unserialize( get_post_meta( $product_id, '_vam_udesign_views', true ) );
    if ( empty( $designer_views ) ) {
      return;
    }

    $color_images = array();
    foreach ( $designer_views as $view_id => $designer_view ) {
      if ( empty( $designer_view['colors'] ) ) {
        continue;
      }

      foreach ( $designer_view['colors'] as $color_id => $image_id ) {
        $color_images[$color_id][$view_id] = $image_id;
      }
    }

    foreach ( $variations as $variation_id ) {
      $color_term_slug  = get_post_meta( $variation_id, 'attribute_pa_color', true );
      $color_term       = get_term_by( 'slug', $color_term_slug, 'pa_color');
      if ( empty( $color_term ) ) {
        continue;
      }

      $color_term_id = $color_term->term_id;
      if ( empty( $color_images[$color_term_id] ) ) {
        continue;
      }

      $image_id = absint( current( $color_images[$color_term_id] ) );
      if ( $image_id ) {
        update_post_meta( $variation_id, '_thumbnail_id', $image_id );
      }
    }
  }

  public static function get_product_categories() {
    ob_start();
    check_ajax_referer( 'designer', 'security' );

    wp_send_json( self::designer_product_categories() );
  }

  public static function get_clipart_categories() {
    ob_start();
    check_ajax_referer( 'designer', 'security' );

    wp_send_json( self::clipart_categories() );
  }

  public static function get_design_categories() {
    ob_start();
    check_ajax_referer( 'designer', 'security' );

    wp_send_json( self::design_categories() );
  }

  public static function cliparts( $args ) {
    // Base query arguments
    $query_args = array(
      'fields'          => 'ids',
      'udesign_query'   => 'clipart_query',
      'orderby'         => 'title',
      'order'           => 'ASC',
      'post_type'       => 'clipart',
      'post_status'     => 'publish',
      'posts_per_page'  => 15,
      'paged'           => 1,
      'tax_query'       => array(),
      'meta_query'      => array()
    );

    // Posts per page
    if ( !empty( $args['posts_per_page'] ) ) {
      $posts_per_page = absint( $args['posts_per_page'] );
      if ( !empty( $posts_per_page ) ) {
        $query_args['posts_per_page'] = $posts_per_page;
      }
    }

    // Page
    if ( !empty( $args['paged'] ) ) {
      $paged = absint( $args['paged'] );
      if ( !empty( $paged ) ) {
        $query_args['paged'] = $paged;
      }
    }

    // From category
    if ( ! empty( $args['category_id'] ) ) {
      $query_args['tax_query'][] = array(
        'taxonomy' => 'clipart_cat',
        'terms' => $args['category_id'],
      );
    }

    // Search query
    if ( ! empty( $args['query'] ) ) {
      $query_args['meta_query'][] = array(
        'key'     => '_vam_udesign_search_field',
        'value'   => strtolower( sanitize_text_field( $args['query'] ) ),
        'compare' => 'LIKE'
      );
    }

    // Get cliparts
    $cliparts = new WP_Query( $query_args );
    wp_reset_postdata();

    return $cliparts;
  }

  public static function designs( $args ) {
    // Base query arguments
    $query_args = array(
      'fields'          => 'ids',
      'udesign_query'   => 'design_query',
      'orderby'         => 'post_title',
      'order'           => 'ASC',
      'post_type'       => 'design',
      'post_status'     => 'publish',
      'posts_per_page'  => 15,
      'paged'           => 1,
      'tax_query'       => array(),
      'meta_query'      => array()
    );

    // Order
    if ( !empty( $args['order'] ) ) {
      $query_args['order'] = $args['order'];
    }
    if ( !empty( $args['orderby'] ) ) {
      $query_args['orderby'] = $args['orderby'];
    }

    // Post status
    if ( !empty( $args['post_status'] ) ) {
      $query_args['post_status'] = $args['post_status'];
    }

    // Post author
    if ( !empty( $args['post_author'] ) ) {
      $query_args['author'] = $args['post_author'];
    }

    // Posts per page
    if ( !empty( $args['posts_per_page'] ) ) {
      $posts_per_page = absint( $args['posts_per_page'] );
      if ( !empty( $posts_per_page ) ) {
        $query_args['posts_per_page'] = $posts_per_page;
      }
    }

    // Page
    if ( !empty( $args['paged'] ) ) {
      $paged = absint( $args['paged'] );
      if ( !empty( $paged ) ) {
        $query_args['paged'] = $paged;
      }
    }

    $query_args['meta_query'][] = array(
      'relation' => 'OR',
      array(
        'key'     => '_design_disable_customize',
        'value'   => '1',
        'compare' => 'NOT EXISTS'
      ),
      array(
        'key'     => '_design_disable_customize',
        'value'   => '0',
        'compare' => '='
      )
    );

    // From category
    if ( ! empty( $args['category_id'] ) ) {
      $query_args['tax_query'][] = array(
        'taxonomy' => 'design_cat',
        'terms' => $args['category_id'],
      );
    }

    // Search query
    if ( ! empty( $args['query'] ) ) {
      $query_args['meta_query'][] = array(
        'key'     => '_vam_udesign_search_field',
        'value'   => strtolower( sanitize_text_field( $args['query'] ) ),
        'compare' => 'LIKE'
      );
    }

    // Get designs
    $designs = new WP_Query( $query_args );
    wp_reset_postdata();

    return $designs;
  }

  public static function designer_product_categories() {
    $query = vam_udesign_get_products( array(
      'designer_products' => true
    ) );

    // Get product categories
    $product_categories = array();
    foreach ( wp_get_object_terms( $query->posts, 'product_cat' ) as $product_category ) {
      $product_categories[] = array(
        'id'    => $product_category->term_id,
        'name'  => $product_category->name
      );
    }

    return $product_categories;
  }

  public static function clipart_categories() {
    // Get clipart categories
    $clipart_categories = array();
    foreach ( get_terms( 'clipart_cat' ) as $clipart_category ) {
      $clipart_categories[] = array(
        'id'    => $clipart_category->term_id,
        'name'  => $clipart_category->name
      );
    }

    return $clipart_categories;
  }

  public static function design_categories() {
    // Get design categories
    $design_categories = array();

    if ( is_user_logged_in() && !is_admin() &&
      get_option( 'udesign_customer_save_design_for_later', 'enabled' ) == 'enabled' ) {
      $design_categories[] = array(
        'id' => -1,
        'name' => __( 'My designs', 'vam-udesign' )
      );
    }

    foreach ( get_terms( 'design_cat' ) as $design_category ) {
      $design_categories[] = array(
        'id'    => $design_category->term_id,
        'name'  => $design_category->name
      );
    }

    return $design_categories;
  }

  public static function get_product_variations() {
    if ( !isset( $_POST['product_id'] ) || empty( $_POST['product_id'] ) ) {
      wp_send_json_error();
    }

    $product = wc_get_product( $_POST['product_id'] );
    $variations = self::get_available_variations( $product );

    wp_send_json_success( $variations );
  }

  /**
   * Get an array of available variations for the a product.
   * Use this function as it's faster than the core implementation.
   * @param WC_Product $product
   *
   * @return array|mixed
   */
  private static function get_available_variations( $product ) {
    global $wpdb;

    $product_id = VAM_UDesign_WC_Compat::get_product_id( $product );
    $transient_name = 'udesign_product_variations_' . $product_id;

    $transient_data = get_transient( $transient_name );
    if ( !empty( $transient_data ) ){
      return $transient_data;
    }

    $available_variations = array();

    //Get the children all in one call.
    //This will prime the WP_Post cache so calls to get_child are much faster.

    $args = array(
      'post_parent' => $product_id,
      'post_type' => 'product_variation',
      'orderby' => 'menu_order',
      'order' => 'ASC',
      'post_status' => 'publish',
      'numberposts' => -1,
      'no_found_rows' => true
    );
    $children = get_posts( $args );

    foreach ( $children as $child ) {
      $variation = wc_get_product( $child );
      $variation_id = VAM_UDesign_WC_Compat::get_product_id( $variation );

      // Hide out of stock variations if 'Hide out of stock items from the catalog' is checked
      if ( empty( $variation_id ) || ( 'yes' === get_option( 'woocommerce_hide_out_of_stock_items' ) && !$variation->is_in_stock() ) ) {
        continue;
      }

      // Filter 'woocommerce_hide_invisible_variations' to optionally hide invisible variations (disabled variations and variations with empty price)
      if ( apply_filters( 'woocommerce_hide_invisible_variations', false, $product_id, $variation ) && !$variation->variation_is_visible() ) {
        continue;
      }

      $available_variations[] = array(
        'variation_id'        => $variation_id,
        'variation_is_active' => $variation->variation_is_active(),
        'attributes'          => $variation->get_variation_attributes(),
        'image_id'            => $variation->get_image_id(),
        'price'               => $variation->get_price(),
      );
    }

    set_transient( $transient_name, $available_variations, DAY_IN_SECONDS * 30 );

    return $available_variations;
  }

  public static function load_designer_product() {
    global $product, $design;
    ob_start();

    check_ajax_referer( 'load-designer-product', 'security' );

    if ( empty( $_POST['product_id'] ) ) {
      die( -1 );
    }

    $design_id = 0;
    if ( !empty( $_POST['design_id'] ) ) {
      $design_id = absint( $_POST['design_id'] );
      $design = vam_udesign_get_design( $design_id );
    }

    $variation_id = 0;
    $product_id = absint( $_POST['product_id'] );
    if ( 'product_variation' == get_post_type( $product_id ) ) {
      $variation_id = $product_id;
      $product_id = absint( wp_get_post_parent_id( $product_id ) );
    }

    $product = wc_get_product( $product_id );
    $selected_attributes = array();
    if ( !empty( $variation_id ) ) {
      $variation_attributes = wc_get_product_variation_attributes( $variation_id );
      foreach ( $variation_attributes as $key => $value ) {
        $taxonomy = esc_attr( str_replace( 'attribute_', '', $key ) );
        $selected_attributes[$taxonomy] = $value;
      }
    } else {
      $selected_attributes = VAM_UDesign_WC_Compat::get_default_attributes( $product );
    }

    $product_props = vam_udesign_get_design_product_props( $product, $design_id, $selected_attributes );

    // Load the template
    vam_udesign_get_template( 'single-design/add-to-cart/design.php', array(
      'available_variations' => $product_props->variations,
      'attributes'           => $product_props->attributes,
      'selected_attributes'  => $product_props->selected_attributes
    ) );

    die();
  }

  public static function get_designer_view_thumbnail() {
    $attachmentID = $_POST['attachment_id'];
    $meta = wp_get_attachment_metadata( $attachmentID );
    $orientation = ( floatval( $meta['width'] ) > floatval( $meta['height'] ) ) ?
      'designer_view_landscape' : 'designer_view_portrait';

    wp_send_json( wp_get_attachment_image_src( $attachmentID, $orientation ) );
  }

  /**
   * Load designer views via AJAX.
   */
  public static function load_designer_views() {
    ob_start();

    check_ajax_referer( 'load-designer-views', 'security' );

    // Check permissions again and make sure we have what we need
    if ( ! current_user_can( 'edit_products' ) || empty( $_POST['product_id'] ) ) {
      die( -1 );
    }

    global $post;
    $product_id = absint( $_POST['product_id'] );
    $post       = get_post( $product_id ); // Set $post global so its available like within the admin screens

    // Product attributes - taxonomies and custom, ordered, with visibility and variation attributes set
    $attributes = maybe_unserialize( get_post_meta( $product_id, '_product_attributes', true ) );
    if ( empty( $attributes ) ) {
      ?>
      <div class="notice notice-alt woocommerce-message">
        <p><?php _e( 'Add at least one View, one Color and one Size on the <strong>Attributes</strong> tab.', 'vam-udesign' ); ?></p>
      </div>
      <?php
      die();
    } elseif ( empty( $attributes['pa_color'] ) ) {
      ?>
      <div class="notice notice-alt woocommerce-message">
        <p><?php _e( 'Add at least one Color on the <strong>Attributes</strong> tab.', 'vam-udesign' ); ?></p>
      </div>
      <?php
      die();
    } elseif ( empty( $attributes['pa_size'] ) ) {
      ?>
      <div class="notice notice-alt woocommerce-message">
        <p><?php _e( 'Add at least one Size on the <strong>Attributes</strong> tab.', 'vam-udesign' ); ?></p>
      </div>
      <?php
      die();
    } elseif ( empty( $attributes['pa_view'] ) ) {
      ?>
      <div class="notice notice-alt woocommerce-message">
        <p><?php _e( 'Add at least one View on the <strong>Attributes</strong> tab.', 'vam-udesign' ); ?></p>
      </div>
      <?php
      die();
    }

    $color_terms     = array();
    $color_taxonomy  = 'pa_color';
    $all_color_terms = get_terms( $color_taxonomy, apply_filters( 'woocommerce_product_attribute_terms', array(
      'orderby'    => 'id',
      'hide_empty' => 0
    ) ) );
    foreach ( $all_color_terms as $term ) {
      if ( has_term( absint( $term->term_id ), $color_taxonomy, $product_id ) ) {
        $color_terms[] = $term;
      }
    }

    $view_terms     = array();
    $view_taxonomy  = 'pa_view';
    $all_view_terms = get_terms( $view_taxonomy, apply_filters( 'woocommerce_product_attribute_terms', array(
      'orderby'    => 'id',
      'hide_empty' => 0
    ) ) );
    foreach ( $all_view_terms as $term ) {
      if ( has_term( absint( $term->term_id ), $view_taxonomy, $product_id ) ) {
        $view_terms[] = $term;
      }
    }
    $view_terms_count = count( $view_terms );
    $loop = 0;

    if ( $view_terms_count ) {
      $designer_views = maybe_unserialize( get_post_meta( $product_id, '_vam_udesign_views', true ) );

      foreach ( $view_terms as $view_term ) {
        $view_id = $view_term->term_id;
        $view_data = array();
        $view_fields = array(
          'view_id' => $view_id,
          'view_width' => '',
          'view_height' => '',
          'view_min_percent' => '',
          'view_price' => '',
          'image_width' => '250',
          'image_height' => '280',
          'image_id' => '',
          'x' => '',
          'y' => '',
          'w' => '',
          'h' => '',
          'colors' => null
        );

        foreach ( $view_fields as $field => $value ) {
          if ( 'colors' == $field ) {
            foreach ( $color_terms as $color_term ) {
              $color_id = $color_term->term_id;

              $view_data[ $field ][ $color_id ] = isset( $designer_views[$view_id][ $field ][ $color_id ] ) ? maybe_unserialize( $designer_views[$view_id][ $field ][ $color_id ] ) : 0;
            }
          } else {
            $view_data[ $field ] = isset( $designer_views[$view_id][ $field ] ) ? maybe_unserialize( $designer_views[$view_id][ $field ] ) : $value;
          }
        }

        $view_data['view_width']       = wc_format_localized_decimal( $view_data['view_width'] );
        $view_data['view_height']      = wc_format_localized_decimal( $view_data['view_height'] );
        $view_data['view_min_percent'] = wc_format_localized_decimal( $view_data['view_min_percent'] );
        $view_data['view_price']       = wc_format_localized_price( $view_data['view_price'] );
        $view_data['image_width']      = wc_format_localized_decimal( $view_data['image_width'] );
        $view_data['image_height']     = wc_format_localized_decimal( $view_data['image_height'] );
        $view_data['image_id']         = absint( $view_data['image_id'] );
        $view_data['_image']           = '';

        if ( $view_data['image_id'] ) {
          $image_meta                 = wp_get_attachment_metadata( $view_data['image_id'] );
          $orientation                = floatval( $image_meta['width'] ) > floatval( $image_meta['height'] ) ? 'designer_view_landscape' : 'designer_view_portrait';
          $view_data['_image']        = wp_get_attachment_image_src( $view_data['image_id'], $orientation )[0];
        }

        include ( plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/html-designer-view.php' );

        $loop++;
      }
    }

    die();
  }

  /**
   * Save designer views via AJAX.
   */
  public static function save_designer_views() {
    ob_start();

    check_ajax_referer( 'save-designer-views', 'security' );

    // Check permissions again and make sure we have what we need
    if ( ! current_user_can( 'edit_products' ) || empty( $_POST['product_id'] ) || empty( $_POST['_vam_udesign_views'] ) ) {
      die( -1 );
    }

    $product_id = absint( $_POST['product_id'] );
    $views = $_POST['_vam_udesign_views'];

    update_post_meta( $product_id, '_vam_udesign_views', $views );

    die();
  }

  public static function get_product() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_POST['product_id'] ) ) {
      wp_send_json_error();
    }

    $product_id = absint( $_POST['product_id'] );
    $wc_product = wc_get_product( $product_id );
    if ( 'variation' != $wc_product->get_type() ) {
      wp_send_json_error();
    }

    $post = get_post( $product_id );
    $designer_product = new VAM_UDesign_Product( $post->post_parent );
    $thumb = wp_get_attachment_image_src( $designer_product->product->get_image_id(), 'shop_catalog' );
    $colors = array();
    $color_terms = $designer_product->get_color_terms();
    foreach ( $color_terms as $color_term ) {
      $colors[] = array(
        'name' => $color_term['term']->name,
        'slug' => $color_term['term']->slug,
        'color' => $color_term['color']
      );
    }

    $sizes = array();
    $size_terms = $designer_product->get_size_terms();
    foreach ( $size_terms as $size_term ) {
      $sizes[] = array(
        'name' => $size_term->name,
        'slug' => $size_term->slug,
        'size' => $size_term->slug,
      );
    }

    $color_slug = get_post_meta( $product_id, 'attribute_pa_color', true );
    $color_term = get_term_by( 'slug', $color_slug, 'pa_color' );
    $swatch_term = new VAM_UDesign_WC_Swatch_Term( $color_term->term_id );
    $color = $swatch_term->get_color();
    $size = get_post_meta( $product_id, 'attribute_pa_size', true );

    $result = array(
      'id'          => $product_id,
      'product_id'  => $post->post_parent,
      'name'        => $designer_product->product->get_title(),
      'thumb'       => $thumb[0],
      'price'       => html_entity_decode( strip_tags( wc_price( $wc_product->get_price() ) ) ),
      'color'       => $color,
      'size'        => $size,
      'colors'      => $colors,
      'sizes'       => $sizes,
      'variations'  => array(),
      'views'       => array(),
    );

    $available_variations = self::get_available_variations( $designer_product->product );
    foreach ( $available_variations as $key => $variation ) {
      $color_slug   = $variation['attributes']['attribute_pa_color'];
      $size_slug    = $variation['attributes']['attribute_pa_size'];
      $color_term   = get_term_by( 'slug', $color_slug, 'pa_color' );
      $swatch_term  = new VAM_UDesign_WC_Swatch_Term( $color_term->term_id );
      $thumb        = wp_get_attachment_image_src( $variation['image_id'], 'designer-view-portrait' );

      $result['variations'][] = array(
        'id'    => $variation['variation_id'],
        'size'  => $size_slug,
        'size_slug' => $size_slug,
        'color' => $swatch_term->get_color(),
        'color_slug' => $color_slug,
        'price' => html_entity_decode( strip_tags( wc_price( $variation['price'] ) ) ),
        'thumb' => $thumb[0],
      );
    }

    $selected = true;
    $views_meta =  $designer_product->get_views_meta();
    foreach ( $views_meta as $view_id => $view_meta ) {
      $view_image_meta = wp_get_attachment_metadata( $view_meta['image_id'] );
      if ( empty( $view_image_meta ) ) {
        continue;
      }

      $image_size = array(
        'width' => floatval( $view_image_meta['width'] ),
        'height' => floatval( $view_image_meta['height'] )
      );

      $v = array(
        'name' => $view_id,
        'selected' => $selected,
        'background' => array(
          'images' => array(),
          'width' => $image_size['width'],
          'height' => $image_size['height']
        )
      );

      $selected = false;

      $view_width = empty( $view_meta['image_width'] ) ? 250 : intval( $view_meta['image_width'] );
      $view_height = empty( $view_meta['image_height'] ) ? 280 : intval( $view_meta['image_height'] );
      $view_colors_meta = $view_meta['colors'];

      foreach ( $view_colors_meta as $color_id => $image_id ) {
        $full = wp_get_attachment_image_src( $image_id, 'full' );
        $thumb = wp_get_attachment_image_src( $image_id, 'designer-view-portrait' );

        $swatch_term = new VAM_UDesign_WC_Swatch_Term( $color_id );
        $v['background']['images'][$swatch_term->get_color()] = array(
          'thumb' => $thumb[0],
          'src' => $full[0],
        );
      }

      $v['bounds'] = array(
        'width'  => intval( intval( $image_size['width'] ) * floatval( $view_meta['w'] ) / $view_width ),
        'height' => intval( intval( $image_size['height'] ) * floatval( $view_meta['h'] ) / $view_height),
        'x'      => intval( $image_size['width'] * floatval( $view_meta['x'] ) / $view_width ),
        'y'      => intval( $image_size['height'] * floatval( $view_meta['y'] ) / $view_height ),
      );

      $real_width = 0;
      $real_height = 0;
      if ( !empty( $view_meta['view_width'] ) && !empty( $view_meta['view_height'] ) ) {
        $real_width = floatval( $view_meta['view_width'] );
        $real_height = floatval( $view_meta['view_height'] );
      }

      $v['measurements'] = array(
        'width' => $real_width,
        'height' => $real_height,
      );

      $result['views'][] = $v;
    }

    wp_send_json( $result );
  }

  public static function get_product_details() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_POST['product_id'] ) ) {
      wp_send_json_error();
    }

    $product_id       = absint( $_POST['product_id'] );
    $post             = get_post( $product_id );
    $designer_product = new VAM_UDesign_Product( $product_id );
    $wc_product       = $designer_product->product;
    if ( 'variable' != $wc_product->get_type() ) {
      wp_send_json_error();
    }

    $result   = array(
      'name'        => $wc_product->get_title(),
      'description' => $post->post_excerpt,
      'variations'  => array(),
      'colors'      => array(),
      'sizes'       => array(),
    );

    $available_variations = self::get_available_variations( $wc_product );
    foreach ( $available_variations as $key => $variation ) {
      $color_slug   = $variation['attributes']['attribute_pa_color'];
      $color_term   = get_term_by( 'slug', $color_slug, 'pa_color' );
      $swatch_term  = new VAM_UDesign_WC_Swatch_Term( $color_term->term_id );
      $thumb        = wp_get_attachment_image_src( $variation['image_id'], 'designer-view-portrait' );

      $result['variations'][] = array(
        'id'    => $variation['variation_id'],
        'size'  => $variation['attributes']['attribute_pa_size'],
        'color' => $swatch_term->get_color(),
        'price' => html_entity_decode( strip_tags( wc_price( $variation['price'] ) ) ),
        'thumb' => $thumb[0],
      );
    }

    $default = array_filter( (array) get_post_meta( $product_id, '_default_attributes', true ), 'strlen' );
    foreach ( $designer_product->get_color_terms() as $key => $color_term ) {
      $selected = false;
      if ( empty( $default['pa_color'] ) ) {
        $selected = $key === 0;
      } else {
        $selected = $color_term['term']->slug === $default['pa_color'];
      }

      $result['colors'][] = array(
        'name'      => $color_term['term']->name,
        'color'     => $color_term['color'],
        'selected'  => $selected,
      );
    }

    foreach ( $designer_product->get_size_terms() as $key => $size_term ) {
      $selected = false;
      if ( empty( $default['pa_size'] ) ) {
        $selected = $key === 0;
      } else {
        $selected = $size_term->slug === $default['pa_size'];
      }

      $result['sizes'][] = array(
        'name'      => $size_term->name,
        'size'      => $size_term->slug,
        'selected'  => $selected,
      );
    }

    wp_send_json( $result );
  }

  public static function get_products() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    $category_id = empty( $_POST['category_id'] ) ? null : absint( $_POST['category_id'] );
    $search_query = empty( $_POST['query'] ) ? null : $_POST['query'];
    $posts_per_page = empty( $_POST['limit'] ) ? 15 : absint( $_POST['limit'] );
    $page = empty( $_POST['page'] ) ? 1 : absint( $_POST['page'] );

    $query = vam_udesign_get_products( array(
      'designer_products' => true,
      'category_id' => $category_id,
      'search_query' => $search_query,
      'posts_per_page' => $posts_per_page,
      'paged' => $page
    ) );
    $products = $query->posts;

    $product_list = array();
    foreach ( $products as $product_id ) {
      $wc_product = wc_get_product( $product_id );
      $thumb_id = get_post_thumbnail_id( $product_id );
      $thumb = wp_get_attachment_image_src( $thumb_id, 'designer-view-portrait' );

      if ( !empty( $thumb ) ) {
        $product_list[] = array(
          'id'    => $product_id,
          'name'  => $wc_product->get_title(),
          'thumb' => $thumb[0],
          'price' => str_replace( '–', ' - ', html_entity_decode( strip_tags( $wc_product->get_price_html() ) ) )
        );
      }
    }

    wp_send_json( array(
      'list' => $product_list,
      'meta' => array(
        'page' => $page,
        'total' => absint( $query->found_posts ),
        'pages' => absint( $query->max_num_pages ),
        'category' => $category_id,
        'query' => $search_query
      )
    ) );
  }

  public static function get_cliparts() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    $category_id = empty( $_POST['category_id'] ) ? null : absint( $_POST['category_id'] );
    $search_query = empty( $_POST['query'] ) ? null : $_POST['query'];
    $posts_per_page = empty( $_POST['limit'] ) ? null : absint( $_POST['limit'] );
    $page = empty( $_POST['page'] ) ? 1 : absint( $_POST['page'] );

    $query = self::cliparts( array(
        'category_id' => $category_id,
        'query' => $search_query,
        'posts_per_page' => $posts_per_page,
        'paged' => $page
    ) );
    $cliparts = $query->posts;

    $clipart_list = array();
    foreach ( $cliparts as $clipart_id ) {
      $thumb_id = get_post_thumbnail_id( $clipart_id );
      $thumb = wp_get_attachment_image_src( $thumb_id, 'designer-view-portrait' );

      if ( !empty( $thumb ) ) {
        $clipart_list[] = array(
          'id' => $clipart_id,
          'name' => get_the_title( $clipart_id ),
          'thumb' => $thumb[0]
        );
      }
    }

    wp_send_json( array(
      'list' => $clipart_list,
      'meta' => array(
        'page' => $page,
        'total' => absint( $query->found_posts ),
        'pages' => absint( $query->max_num_pages ),
        'category' => $category_id,
        'query' => $search_query
      )
    ) );
  }

  public static function get_designs() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    $category_id = empty( $_POST['category_id'] ) ? null : intval( $_POST['category_id'] );
    $search_query = empty( $_POST['query'] ) ? null : $_POST['query'];
    $posts_per_page = empty( $_POST['limit'] ) ? null : absint( $_POST['limit'] );
    $page = empty( $_POST['page'] ) ? 1 : absint( $_POST['page'] );

    $post_status = null;
    $post_author = null;
    $post_category_id = $category_id;
    $post_orderby = 'post_title';
    $post_order = 'ASC';
    if ( $category_id  === -1 ) {
      $user = wp_get_current_user();
      if ( empty( $user ) || empty( $user->ID ) ) {
        self::send_json_error( 'Must be logged in to retrieve saved designs' );
      }

      $post_status = 'user';
      $post_author = $user->ID;
      $post_orderby = 'post_date';
      $post_order = 'DESC';
      $post_category_id = null;
    }

    $query = self::designs( array(
        'category_id' => $post_category_id,
        'query' => $search_query,
        'posts_per_page' => $posts_per_page,
        'paged' => $page,
        'post_status' => $post_status,
        'post_author' => $post_author,
        'orderby' => $post_orderby,
        'order' => $post_order
    ) );
    $designs = $query->posts;

    $design_list = array();
    foreach ( $designs as $design_id ) {
      $thumb_id = get_post_thumbnail_id( $design_id );
      $thumb = wp_get_attachment_image_src( $thumb_id, 'designer-view-portrait' );

      if ( ! empty( $thumb ) ) {
        $design_list[] = array(
          'id' => $design_id,
          'name' => get_the_title( $design_id ),
          'thumb' => $thumb[0],
          'mine' => $category_id === -1
        );
      }
    }

    wp_send_json( array(
      'list' => $design_list,
      'meta' => array(
        'page' => $page,
        'total' => absint( $query->found_posts ),
        'pages' => absint( $query->max_num_pages ),
        'category' => $category_id,
        'query' => $search_query
      )
    ) );
  }

  public static function get_clipart() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_POST['clipart_id'] ) ) {
      wp_send_json_error();
    }
    $clipart = get_post( $_POST['clipart_id'] );

    $attachment_id = get_post_meta( $clipart->ID, '_clipart_image_id', true );
    if ( empty( $attachment_id ) ) {
      wp_send_json_error();
    }

    $image_src = wp_get_attachment_image_src( $attachment_id, 'full' );
    if ( empty( $image_src ) ) {
      wp_send_json_error();
    }

    $src = $image_src[0];
    $width = empty( $image_src[1] ) ? 300 : $image_src[1];
    $height = empty( $image_src[2] ) ? 300 : $image_src[2];

    // try to get svg size from the full size thumbnail
    $mime = get_post_mime_type( $attachment_id );

    $is_svg = $mime == 'image/svg+xml';
    if ( $is_svg ) {
      $thumb_src = wp_get_attachment_image_src( get_post_thumbnail_id( $clipart->ID ), 'full' );
      if ( !empty( $thumb_src ) ) {
        $width = empty( $thumb_src[1] ) ? 300 : $thumb_src[1];
        $height = empty( $thumb_src[2] ) ? 300 : $thumb_src[2];
      }
    }

    // scale up/down image to fit into the active designer view
    if ( !empty( $_POST['view_width'] ) && !empty( $_POST['view_height'] ) ) {
        $view_width = floatval( $_POST['view_width'] ) * 0.8;
        $view_height = floatval( $_POST['view_height'] ) * 0.8;

        if ( $is_svg || $width > $view_width || $height > $view_height ) {
          $scale = $view_height / $height;
          if ( $width / $height > $view_width / $view_height ) {
              $scale = $view_width / $width;
          }

          $width *= $scale;
          $height *= $scale;
        }
    }

    $thumb_src = wp_get_attachment_image_src( get_post_thumbnail_id( $clipart->ID ), 'designer_thumb' );

    $result = array(
      'id' => $clipart->ID,
      'name' => $clipart->post_title,
      'thumb' => empty( $thumb_src ) ? $src[0] : $thumb_src[0],
      'src' => $src,
      'width' => $width,
      'height' => $height,
      'mime' => $mime
    );

    wp_send_json( $result );
  }

  public static function get_design() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_POST['design_id'] ) ) {
      wp_send_json_error();
    }

    $design = get_post_meta( $_POST['design_id'], '_design_meta', true );
    if ( empty( $design ) ) {
      wp_send_json_error();
    }

    self::send_json($design);
  }

  public static function delete_my_design() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_POST['design_id'] ) ) {
      wp_send_json_error();
    }
    $design_id = $_POST['design_id'];

    $user = wp_get_current_user();
    if ( empty( $user ) || empty( $user->ID ) ) {
      self::send_json_error( 'Must be logged in to save design' );
    }

    $post = get_post( $design_id );
    if ( empty( $post ) || empty( $post->post_author ) ) {
      self::send_json_error( 'Invalid design provided' );
    }

    if ( $post->post_author != $user->ID ) {
      self::send_json_error( 'You can only delete your designs' );
    }

    if ( empty( wp_delete_post( $design_id, true ) ) ) {
      self::send_json_error( 'Could not delete design' );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'message' => __( 'Design successfully deleted' )
      )
    ) );
  }

  public static function upload_blob_attachment() {
    ob_start();

    check_ajax_referer( 'blob_attachment', 'security' );

    if ( empty( $_POST['post_id'] ) ) {
      self::send_json_error( 'Must provide post id' );
    }
    $post_id = $_POST['post_id'];

    if ( empty( $_POST['post_type'] ) ) {
      self::send_json_error( 'Must provide post type' );
    }
    $post_type = $_POST['post_type'];

    if ( empty( $_FILES['image'] ) ) {
      self::send_json_error( 'No image provided' );
    }
    $file = $_FILES['image'];

    $featured_image = false;
    switch( $post_type ) {
    case 'clipart':
      $featured_image = wp_handle_upload( $file, array( 'test_form' => false ) );
      if ( empty( $featured_image ) || !empty( $featured_image['error'] ) ) {
        self::send_json_error( 'Could not generate clipart thumbnails' );
      }

      break;

    default:
      self::send_json_error( 'Unknown post type' );
    }

    $attach_id = vam_udesign_create_attachment( $featured_image, $post_id );
    if ( empty( $attach_id ) ) {
      self::send_json_error( 'Could not create attachment' );
    }

    if ( !empty( $_POST['set_thumbnail'] ) ) {
      $old_attach_id = get_post_thumbnail_id( $post_id );
      set_post_thumbnail( $post_id, $attach_id );

      if ( !empty( $old_attach_id ) ) {
        wp_delete_attachment( $old_attach_id, true );
      }
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'attachment_id' => $attach_id,
        'message' => __( 'Attachment successfully uploaded' )
      )
    ) );
  }

  public static function upload_clipart() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_FILES['image'] ) ) {
      self::send_json_error( 'No image provided' );
    }

    if ( empty( $_POST['image_width'] ) || empty( $_POST['image_height'] ) ) {
      self::send_json_error( 'No dimensions provided' );
    }

    $uploaded_file = vam_udesign_upload_image( $_FILES['image'] );
    if ( empty( $uploaded_file ) ) {
      self::send_json_error( 'Could not upload clipart' );
    }

    $url = $uploaded_file['url'];
    $filepath = $uploaded_file['file'];
    $width = $_POST['image_width'];
    $height = $_POST['image_height'];
    $mime = $uploaded_file['type'];

    $is_svg = $mime == 'image/svg+xml';
    if ( !$is_svg ) {
      $size = getimagesize( $filepath );
      if ( !empty( $size ) ) {
        $width = $size[0];
        $height = $size[1];
      }
    }

    // scale up/down image to fit into the active designer view
    if ( !empty( $_POST['view_width'] ) && !empty( $_POST['view_height'] ) ) {
      $view_width = floatval( $_POST['view_width'] ) * 0.8;
      $view_height = floatval( $_POST['view_height'] ) * 0.8;

      if ( $is_svg || $width > $view_width || $height > $view_height ) {
        $scale = $view_height / $height;
        if ( $width / $height > $view_width / $view_height ) {
          $scale = $view_width / $width;
        }

        $width *= $scale;
        $height *= $scale;
      }
    }

    $result = array(
      'id' => intval( microtime( true ) * 1000000 ),
      'name' => pathinfo( $filepath )['filename'],
      'thumb' => $url,
      'src' => $url,
      'width' => $width,
      'height' => $height,
      'mime' => $mime
    );

    wp_send_json( $result );
  }

  public static function create_clipart() {
    ob_start();

    check_ajax_referer( 'clipart', 'security' );

    if ( empty( $_FILES['image'] ) ) {
      self::send_json_error( 'No image provided' );
    }
    $file = $_FILES['image'];

    if ( empty( $_POST['category_ids'] ) ) {
      self::send_json_error( 'Must select at least one category' );
    }

    $category_ids = json_decode( $_POST['category_ids'] );
    if ( empty( $category_ids ) ) {
      self::send_json_error( 'Could not read selected categories' );
    }

    // upload clipart image
    $uploaded_file = vam_udesign_upload_image( $file );
    if ( empty( $uploaded_file ) ) {
      self::send_json_error( 'Could not upload image' );
    }

    // create clipart post
    $clipart_title = sanitize_text_field( ucwords( pathinfo( $uploaded_file['file'] )['filename'] ) );

    $clipart_id = vam_udesign_create_clipart( array(
      'name' => $clipart_title,
      'image' => $uploaded_file,
      'auto_thumb' => true,
      'category_ids' => $category_ids,
      'search_field' => strtolower( $clipart_title )
    ) );

    if ( empty( $clipart_id ) ) {
      self::send_json_error( 'Could not create clipart post' );
    }

    // get clipart image
    $src = vam_udesign_get_clipart_image( $clipart_id );
    if ( empty( $src ) ) {
      self::send_json_error( 'Could not get clipart image url' );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'clipart_id' => $clipart_id,
        'attachment_url' => $src[0],
        'message' => __( 'clipart successfully uploaded' )
      )
    ) );
  }

  public static function create_design() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    $name = 'Untitled design';
    if ( !empty( $_POST['design_name'] ) ) {
      $name = $_POST['design_name'];
    }

    if ( empty( $_POST['design_meta'] ) ) {
      self::send_json_error( 'Must provide a design' );
    }
    $design_meta = $_POST['design_meta'];

    $background_color = false;
    if ( !empty( $_POST['background'] ) ) {
      $background_color = $_POST['background'];
    }

    $default_product_id = false;
    if ( !empty( $_POST['default_product_id'] ) ) {
      $default_product_id = $_POST['default_product_id'];
    }

    $design_id = vam_udesign_create_design( array(
      'name' => $name,
      'metadata' => $design_meta,
      'background_color' => $background_color,
      'default_product_id' => absint( $default_product_id )
    ) );

    if ( empty( $design_id ) ) {
      self::send_json_error( 'Could not create design post' );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'id' => $design_id,
        'message' => __( 'design successfully uploaded' )
      )
    ) );
  }

  public static function create_cart_design() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    $name = 'Cart design';
    if ( !empty( $_POST['design_name'] ) ) {
      $name = $_POST['design_name'];
    }

    if ( empty( $_POST['design_meta'] ) ) {
      self::send_json_error( 'Must provide a design' );
    }
    $design_meta = $_POST['design_meta'];

    $background_color = false;
    if ( !empty( $_POST['background'] ) ) {
      $background_color = $_POST['background'];
    }

    $default_product_id = 0;
    if ( !empty( $_POST['default_product_id'] ) ) {
      $default_product_id = $_POST['default_product_id'];
    }

    $design_id = vam_udesign_create_design( array(
      'name' => $name,
      'post_status' => 'cart',
      'metadata' => $design_meta,
      'background_color' => $background_color,
      'default_product_id' => absint( $default_product_id )
    ) );

    if ( empty( $design_id ) ) {
      self::send_json_error( 'Could not create design post' );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'id' => $design_id,
        'message' => __( 'Design successfully uploaded' )
      )
    ) );
  }

  public static function create_user_design() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    $user = wp_get_current_user();
    if ( empty( $user ) || empty( $user->ID ) ) {
      self::send_json_error( 'Must be logged in to save design' );
    }

    if ( empty( $_POST['design_name'] ) ) {
      self::send_json_error( 'Must provide a name for the design' );
    }
    $name = $_POST['design_name'];

    if ( empty( $_POST['design_meta'] ) ) {
      self::send_json_error( 'Must provide a design' );
    }
    $design_meta = $_POST['design_meta'];

    $background_color = false;
    if ( !empty( $_POST['background'] ) ) {
      $background_color = $_POST['background'];
    }

    $default_product_id = 0;
    if ( !empty( $_POST['default_product_id'] ) ) {
      $default_product_id = $_POST['default_product_id'];
    }

    $design_id = vam_udesign_create_design( array(
      'name' => $name,
      'post_status' => 'user',
      'post_author' => $user->ID,
      'metadata' => $design_meta,
      'background_color' => $background_color,
      'default_product_id' => absint( $default_product_id )
    ) );

    if ( empty( $design_id ) ) {
      self::send_json_error( 'Could not create design post' );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'id' => $design_id,
        'message' => __( 'Design successfully uploaded' )
      )
    ) );
  }

  public static function set_post_thumbnail() {
    ob_start();

    check_ajax_referer( 'post-thumbnail', 'security' );

    if ( empty( $_POST['post_id'] ) ) {
      die();
    }
    $post_id = $_POST['post_id'];

    if ( empty( $_POST['thumbnail_id'] ) ) {
      die();
    }
    $thumbnail_id = $_POST['thumbnail_id'];

    $old_attach_id = get_post_thumbnail_id( $post_id );
    set_post_thumbnail( $post_id, $thumbnail_id );

    if ( !empty( $old_attach_id ) ) {
      wp_delete_attachment( $old_attach_id, true );
    }

    echo _wp_post_thumbnail_html( $thumbnail_id, $post_id );
    die();
  }

  public static function upload_design_views() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    if ( empty( $_POST['post_id'] ) ) {
      self::send_json_error( 'Must provide post id' );
    }
    $post_id = $_POST['post_id'];

    if ( empty( $_FILES['views'] ) ) {
      self::send_json_error( 'No views provided' );
    }

    $views = self::parse_file_upload( $_FILES['views'] );
    if ( empty( $views ) ) {
      self::send_json_error( 'Invalid file upload' );
    }

    $thumbnails = vam_udesign_generate_design_thumbnails( $post_id, $views );
    if ( empty( $thumbnails ) ) {
      self::send_json_error( 'Could not generate design thumbnails' );
    }

    $attach_id = vam_udesign_create_attachment( $thumbnails['featured_image'], $post_id );
    if ( empty( $attach_id ) ) {
      self::send_json_error( 'Could not create attachment' );
    }

    $old_attach_id = get_post_thumbnail_id( $post_id );
    if ( !empty( $old_attach_id ) ) {
      wp_delete_attachment( $old_attach_id, true );
    }

    set_post_thumbnail( $post_id, $attach_id );

    wp_send_json( array(
      'success' => true,
      'thumbs' => $thumbnails,
      'data' => array(
        'attachment_id' => $attach_id,
        'message' => __( 'design views successfully uploaded' )
      )
    ) );
  }

  public static function download_design_views() {
    ob_start();

    check_ajax_referer( 'designer_download', 'security' );

    // get product
    if ( empty( $_POST['post_id'] ) ) {
      self::send_json_error( 'Must provide product id' );
    }
    $post_id = $_POST['post_id'];

    $result = self::get_designer_variation_attributes( $post_id );
    if ( empty( $result['success'] ) || empty( $result['success'] ) ) {
      self::send_json_error( empty( $result['message'] ) ? 'Product is not properly configured' : $result['message'] );
    }
    $variation = $result['data'];

    // get designs
    if ( empty( $_POST['designs'] ) ) {
      self::send_json_error( 'Must provide designs to download' );
    }

    $designs = vam_udesign_json_decode( $_POST['designs'], true );
    if ( empty( $designs ) ) {
      self::send_json_error( 'Invalid designs provided' );
    }

    // get design views
    if ( empty( $_FILES['views'] ) ) {
      self::send_json_error( 'No views provided' );
    }

    $views = self::parse_file_upload( $_FILES['views'] );
    if ( empty( $views ) ) {
      self::send_json_error( 'Invalid file upload' );
    }

    $download_files = array();
    foreach( $views as &$view ) {
      $view_id = intval( $view['name'] );

      // get design for the current view
      $design = false;
      foreach( $designs as &$d ) {
        if ( empty( $d['view'] ) ) {
          continue;
        }

        if ( intval( $d['view'] ) == $view_id ) {
          $design = $d;
          break;
        }
      }

      if ( empty( $design ) ) {
        continue;
      }

      // get view name
      $view_name = strval( $view_id );

      $view_term = get_term( $view_id, 'pa_view' );
      if ( !empty( $view_term ) ) {
        $view_name = $view_term->slug;
      }

      // generate download file
      $download_file = self::generate_design_download_image( $design, $view['tmp_name'] );
      if ( empty( $download_file ) ) {
        self::send_json_error( 'Could not generate design download file' );
      }

      $download_name = sprintf( '%s-%s.png', $variation->parent_post->post_name, $view_name );
      $download_files[$download_name] = $download_file;
    }

    $file = false;
    if ( count( $download_files ) == 1 ) {
      $download_file = base64_encode( current( $download_files ) );
      if ( empty( $download_file ) ) {
        self::send_json_error( 'Could not encode design download file' );
      }

      $file = array(
        'name' => key( $download_files ),
        'blob' => 'data:image/png;base64,' . $download_file,
        'mime' => 'image/png'
      );
    } else {
      $zip = new VAM_UDesign_Zip_File();
      foreach( $download_files as $download_name => &$download_file ) {
        $zip->addFile( $download_file, $download_name );
      }

      $download_file = base64_encode( $zip->file() );
      if ( empty( $download_file ) ) {
        self::send_json_error( 'Could not encode design download file' );
      }

      $file = array(
        'name' => sprintf( '%s.zip', $variation->parent_post->post_name ),
        'blob' => 'data:application/zip;base64,' . $download_file,
        'mime' => 'application/zip'
      );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'file' => $file,
        'message' => __( 'design views successfully downloaded' )
      )
    ) );
  }

  public static function get_design_preview() {
    ob_start();

    check_ajax_referer( 'get-design-preview', 'security' );

    if ( empty( $_POST['design_id'] ) ) {
      self::send_json_error( 'Must select a design' );
    }
    $design_id = absint( $_POST['design_id'] );

    $design_views = get_post_meta( $design_id, '_design_thumbnails', true );
    if ( empty( $design_views ) ) {
      self::send_json_error( 'Could not find design thumbnails' );
    }

    $data = array();
    $template_data = array(
      'post_id' => $design_id
    );

    if ( !empty( $_POST['variation_id'] ) ) {
      $variation_id = absint( $_POST['variation_id'] );

      $result = self::get_designer_variation_attributes( $variation_id );
      if ( empty( $result['success'] ) || empty( $result['success'] ) ) {
        self::send_json_error( empty( $result['message'] ) ? 'Product is not properly configured' : $result['message'] );
      }
      $variation = $result['data'];

      $design_meta = get_post_meta( $_POST['design_id'], '_design_meta', true );
      if ( empty( $design_meta ) ) {
        self::send_json_error( 'Could not retrieve design' );
      }
      $design_data = vam_udesign_json_decode( $design_meta );
      if ( empty( $design_data ) ) {
        self::send_json_error( 'Invalid design provided' );
      }

      $data = self::generate_design_preview_data( $design_id, $design_data, $design_views,
        $variation->post->post_parent, $variation->product, $variation->views, $variation->color->term_id, false );
      if ( empty( $data ) ) {
        self::send_json_error( 'Could not generate design preview data' );
      }

      $swatch_term = new VAM_UDesign_WC_Swatch_Term( $variation->color->term_id );
      $template_data['product'] = array(
        'thumbnails' => $data['preview_urls'],
        'color' => $swatch_term->get_color()
      );
    }

    $data['preview_html'] = vam_udesign_get_template_html( 'single-design/design-image.php', $template_data );

    wp_send_json( array(
      'success' => true,
      'data' => $data
    ) );
  }

  public static function get_design_price() {
    ob_start();

    check_ajax_referer( 'designer', 'security' );

    // read data
    if ( empty( $_POST['variations'] ) ) {
      self::send_json_error( 'Must select at least one product' );
    }
    $variations = $_POST['variations'];

    if ( empty( $_POST['design_meta'] ) ) {
      self::send_json_error( 'Must create a design first' );
    }
    $design_meta = $_POST['design_meta'];

    $designs = vam_udesign_json_decode( $design_meta, true );
    if ( empty( $designs ) ) {
      self::send_json_error( 'Invalid design provided' );
    }

    // calculate variation prices
    $total_price = 0;
    $total_quantity = 0;

    $views = false;
    $color_id = false;
    $attributes_loaded = false;

    foreach( $variations as &$variation ) {
      if ( empty( $variation['id'] ) ) {
        self::send_json_error( 'Invalid variation provided' );
      }
      $variation_id = $variation['id'];

      $quantity = 1;
      if ( !empty( $variation['quantity'] ) ) {
        $quantity = absint( $variation['quantity'] );
      }

      if ( $attributes_loaded ) {
        $variation = wc_get_product( $variation_id );
        if ( empty( $variation ) ) {
          return self::error_response( 'Could not find the selected variation' );
        }
        if ( 'variation' != $variation->get_type() ) {
          return self::error_response( 'Invalid product selected' );
        }
      } else {
        $result = self::get_designer_variation_attributes( $variation_id );
        if ( empty( $result['success'] ) || empty( $result['success'] ) ) {
          self::send_json_error( empty( $result['message'] ) ? 'Product is not properly configured' : $result['message'] );
        }
        $variation_data = $result['data'];

        $views = $variation_data->views;
        $color_id = $variation_data->color->term_id;
        $variation = $variation_data->product;
      }

      $total_quantity += $quantity;
      $total_price += floatval( $variation->get_price() ) * $quantity;
    }

    // calculate design price
    $design_price = 0;
    foreach( $designs as &$design ) {
      if ( empty( $design['view'] ) ) {
        continue;
      }
      $view_id = absint( $design['view'] );

      if ( empty( $views[$view_id] ) ) {
        continue;
      }
      $view = $views[$view_id];

      $view_info = self::get_design_view_info( $design, $view, $color_id );
      if ( empty( $view_info ) ) {
        continue;
      }

      $design_price += $view_info->price;
    }

    $total_price += $design_price * $total_quantity;

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'display_price' => $total_price,
        'price_html'    => wc_price( $total_price ),
      )
    ) );
  }

  public static function import_clipart( $layer, $api_key ) {
    $layer_id = $layer['id'];
    $layer_src = $layer['src'];
    $layer_thumb = $layer['thumb'];

    // Check if clipart already exists
    $query = vam_udesign_get_cliparts( array( 'remote_id' => $layer_id ) );
    if ( !empty( $query->posts ) ) {
      $clipart_id = $query->posts[0];
      $clipart_image_src = false;
      $clipart_thumb_src = false;

      $clipart_image = vam_udesign_get_clipart_image( $clipart_id );
      if ( empty( $clipart_image ) ) {
        $uploaded_image = vam_udesign_upload_remote_image( $layer_src );
        if ( empty( $uploaded_image ) ) {
          return false;
        }

        $attach_id = vam_udesign_create_attachment( $uploaded_file, $clipart_id );
        if ( empty( $attach_id ) ) {
          return false;
        }

        $clipart_image_src = $uploaded_image['url'];
        update_post_meta( $clipart_id, '_clipart_image_id', $attach_id );
      } else {
        $clipart_image_src = $clipart_image[0];
      }

      $clipart_thumb = vam_udesign_get_clipart_thumb( $clipart_id );
      if ( empty( $clipart_thumb ) ) {
        $uploaded_image = vam_udesign_upload_remote_image( $layer_thumb );
        if ( empty( $uploaded_image ) ) {
          return false;
        }

        $attach_id = vam_udesign_create_attachment( $uploaded_image, $clipart_id );
        if ( empty( $attach_id ) ) {
          return false;
        }

        $clipart_image_src = $uploaded_image['url'];
        set_post_thumbnail( $clipart_id, $attach_id );
      } else {
        $clipart_thumb_src = $clipart_thumb[0];
      }

      return array(
        'id' => $clipart_id,
        'remote_id' => $layer_id,
        'src' => $clipart_image_src,
        'thumb' => $clipart_thumb_src,
        'uploaded' => false
      );
    }

    // Check if the clipart was user uploaded
    $api_url = sprintf( '%s/%s/%s?%s', VAM_UDESIGN_API_URL, 'cliparts',
      $layer_id, http_build_query( array( 'api_key' => $api_key ) ) );

    $clipart = vam_udesign_remote_request( $api_url, array(), true );
    if ( empty( $clipart ) ) {
      $clipart_image = vam_udesign_upload_remote_image( $layer_src );
      if ( empty( $clipart_image ) ) {
        return false;
      }

      $clipart_thumb = vam_udesign_upload_remote_image( $layer_thumb );
      if ( empty( $clipart_thumb ) ) {
        return false;
      }

      return array(
        'id' => $layer_id,
        'remote_id' => $layer_id,
        'src' => $clipart_image['url'],
        'thumb' => $clipart_thumb['url'],
        'uploaded' => true
      );
    }

    // Create a new clipart
    $clipart_image = vam_udesign_upload_remote_image( $clipart['src'] );
    if ( empty( $clipart_image ) ) {
      return false;
    }

    $clipart_thumb = vam_udesign_upload_remote_image( $clipart['thumb'] );
    if ( empty( $clipart_thumb ) ) {
      return false;
    }

    $category_ids = array();
    foreach( $clipart['categories'] as $category ) {
      $category_id = vam_udesign_create_term( $category['slug'], 'clipart_cat' );
      if ( !empty( $category_id ) ) {
        $category_ids[] = $category_id;
      }
    }

    $tag_ids = array();
    foreach( $clipart['tags'] as $tag ) {
      $tag_id = vam_udesign_create_term( $tag['slug'], 'clipart_tag' );
      if ( !empty( $tag_id ) ) {
        $tag_ids[] = $tag_id;
      }
    }

    $clipart_id = vam_udesign_create_clipart( array(
      'name' => $clipart['name'],
      'image' => $clipart_image,
      'thumb' => $clipart_thumb,
      'category_ids' => $category_ids,
      'tag_ids' => $tag_ids,
      'search_field' => $clipart['search_field'],
      'remote_id' => $clipart['id']
    ) );

    if ( empty( $clipart_id ) ) {
      return false;
    }

    return array(
      'id' => $clipart_id,
      'remote_id' => $layer_id,
      'src' => $clipart_image['url'],
      'thumb' => $clipart_thumb['url'],
      'uploaded' => false
    );
  }

  public static function import_design() {
    ob_start();

    check_ajax_referer( 'design_import', 'security' );

    $api_key = get_option( 'udesign_api_key', '' );
    if ( empty( $api_key ) ) {
      self::send_json_error( 'Invalid API key' );
    }

    if ( empty( $_POST['design_id'] ) ) {
      self::send_json_error( 'Must provide a design' );
    }
    $remote_design_id = $_POST['design_id'];

    $query = vam_udesign_get_designs( array( 'remote_id' => $remote_design_id ) );
    if ( !empty( $query->posts ) ) {
      wp_send_json( array(
        'success' => true,
        'data' => array(
          'design_id' => $query->posts,
        )
      ) );
    }

    $api_url = sprintf( '%s/%s/%s?%s', VAM_UDESIGN_API_URL, 'designs',
      $remote_design_id, http_build_query( array( 'api_key' => $api_key ) ) );

    $remote_design = vam_udesign_remote_request( $api_url, array(), true );
    if ( empty( $remote_design ) ) {
      self::send_json_error( 'Could not retrieve design from server' );
    }

    if ( empty( $remote_design['name'] ) ) {
      self::send_json_error( 'Could not get design title' );
    }

    if ( empty( $remote_design['metadata'] ) ) {
      self::send_json_error( 'Could not get design metadata' );
    }

    $design_meta = vam_udesign_json_decode( $remote_design['metadata'] );
    if ( empty( $design_meta ) ) {
      self::send_json_error( 'Could not process design metadata' );
    }

    if ( empty( $remote_design['thumb'] ) ) {
      self::send_json_error( 'Could not get design thumbnail' );
    }

    if ( empty( $remote_design['thumbs'] ) ) {
      self::send_json_error( 'Could not get design thumbnails' );
    }
    $remote_thumbs = $remote_design['thumbs'];

    if ( empty( $remote_design['views'] ) ) {
      self::send_json_error( 'Could not get design views' );
    }
    $remote_views = $remote_design['views'];

    // retrieve or create product views
    $views = array();
    foreach( $remote_views as &$remote_view ) {
      if ( !vam_udesign_has_props( $remote_view, array( 'id', 'slug' ) ) ) {
        continue;
      }

      $view_id = vam_udesign_create_term( $remote_view['slug'], 'pa_view' );
      if ( empty( $view_id ) ) {
        self::send_json_error( 'Could not create design views' );
      }

      $views[absint( $remote_view['id'] )] = absint( $view_id );
    }

    // rewrite design layers
    foreach( $design_meta as &$design_view ) {
      if ( empty( $design_view['view'] ) ) {
        self::send_json_error( 'Could not read design views' );
      }
      $view = absint( $design_view['view'] );

      if ( empty( $views[$view] ) ) {
        self::send_json_error( 'Could not process design views' );
      }
      $design_view['view'] = $views[$view];

      if ( empty ( $design_view['export'] ) ) {
        self::send_json_error( 'Could not read design export metadata' );
      }

      if ( !empty( $design_view['export']['data'] ) ) {
        $design_export_data = vam_udesign_json_decode( $design_view['export']['data'] );
        if ( empty( $design_export_data ) ) {
          self::send_json_error( 'Could not process design export metadata' );
        }

        $design_view['export']['data'] = $design_export_data;
      }

      if ( empty( $design_view['layers'] ) ) {
        continue;
      }

      $layers = &$design_view['layers'];
      foreach( $layers as &$layer ) {
        if ( !vam_udesign_has_props( $layer, array( 'id', 'type', 'src', 'thumb' ) ) ) {
          continue;
        }
        if ( !in_array( $layer['type'], array( 'clipart', 'vector' ) ) ) {
          continue;
        }

        $clipart = self::import_clipart( $layer, $api_key );
        if ( empty( $clipart ) ) {
          self::send_json_error( 'Could not import cliparts' );
        }

        $layer['id'] = $clipart['id'];
        $layer['src'] = $clipart['src'];

        if ( $clipart['uploaded'] ) {
          $layer['thumb'] = $clipart['thumb'];
        } else {
          $thumb = vam_udesign_get_clipart_thumb( $clipart['id'], 'designer_thumb' );
          $layer['thumb'] = empty( $thumb ) ? $clipart['thumb'] : $thumb[0];
        }
      }
    }

    // Create a new design
    $design_meta = json_encode( $design_meta );
    if ( empty( $design_meta ) ) {
        self::send_json_error( 'Could not encode design metadata' );
    }

    $thumb = vam_udesign_upload_remote_image( $remote_design['thumb'] );
    if ( empty( $thumb ) ) {
      self::send_json_error( 'Could not create design thumbnail' );
    }

    $category_ids = array();
    foreach( $remote_design['categories'] as &$category ) {
      $category_id = vam_udesign_create_term( $category['slug'], 'design_cat' );
      if ( !empty( $category_id ) ) {
        $category_ids[] = $category_id;
      }
    }

    $tag_ids = array();
    foreach( $remote_design['tags'] as &$tag ) {
      $tag_id = vam_udesign_create_term( $tag['slug'], 'design_tag' );
      if ( !empty( $tag_id ) ) {
        $tag_ids[] = $tag_id;
      }
    }

    $design_id = vam_udesign_create_design( array(
      'name' => $remote_design['name'],
      'metadata' => $design_meta,
      'thumb' => $thumb,
      'category_ids' => $category_ids,
      'tag_ids' => $tag_ids,
      'search_field' => $remote_design['search_field'],
      'background_color' => $remote_design['background'],
      'remote_id' => $remote_design_id
    ) );

    if ( empty( $design_id ) ) {
      self::send_json_error( 'Could not create design post' );
    }

    // upload designer thumbnails
    $thumbs = array();
    foreach ( $remote_thumbs as $remote_view_id => &$remote_thumb ) {
      $view_id = $views[absint( $remote_view_id )];
      if ( empty( $view_id ) ) {
        self::send_json_error( 'Could not process design thumbnails' );
      }

      if ( empty( $remote_thumb['full'] ) ) {
        self::send_json_error( 'Could not read design thumbnails' );
      }

      $thumb = vam_udesign_upload_remote_image( $remote_thumb['full'] );
      if ( empty( $thumb ) ) {
        self::send_json_error( 'Could not upload design thumbnails' );
      }

      $thumbs[] = array(
        'name' => $view_id,
        'url' => $thumb['url'],
        'tmp_name' => $thumb['file'],
        'type' => $thumb['type'],
      );
    }

    $thumbnails = vam_udesign_generate_design_thumbnails( $design_id, $thumbs, $thumb );
    if ( empty( $thumbnails ) ) {
      self::send_json_error( 'Could not generate design thumbnails' );
    }

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'design_id' => $design_id,
      )
    ) );
  }

  public static function import_product() {
    ob_start();

    check_ajax_referer( 'product_import', 'security' );

    $api_key = get_option( 'udesign_api_key', '' );
    if ( empty( $api_key ) ) {
      self::send_json_error( 'Invalid API key' );
    }

    if ( empty( $_POST['product_id'] ) ) {
      self::send_json_error( 'Must provide a product' );
    }
    $remote_product_id = $_POST['product_id'];

    $api_url = sprintf( '%s/%s/%s?%s', VAM_UDESIGN_API_URL, 'products',
      $remote_product_id, http_build_query( array( 'api_key' => $api_key ) ) );

    $remote_product_meta = vam_udesign_remote_request( $api_url, array(), true );
    if ( empty( $remote_product_meta ) ) {
      self::send_json_error( 'Could not retrieve product meta information' );
    }

    // Get remote API client
    $remote_client = vam_udesign_new_wordpress_api_client(
      VAM_UDESIGN_API_HOST,
      get_option( 'udesign_remote_products_api_consumer_key' ),
      get_option( 'udesign_remote_products_api_consumer_secret' )
    );
    if ( empty( $remote_client ) ) {
      self::send_json_error( 'Invalid remote products API credentials' );
    }

    // Get local API client
    $local_client = vam_udesign_new_wordpress_api_client(
      get_site_url(),
      get_option( 'udesign_local_products_api_consumer_key' ),
      get_option( 'udesign_local_products_api_consumer_secret' )
    );
    if ( empty( $local_client ) ) {
      self::send_json_error( 'Invalid local products API credentials' );
    }

    // Import product
    $product_importer = vam_udesign_new_product_importer( $remote_client, $local_client );
    if ( empty( $product_importer ) ) {
      self::send_json_error( 'Could not create product importer' );
    }

    $response = $product_importer->import_product( $remote_product_id, $remote_product_meta );
    if ( empty( $response['success'] ) ) {
      self::send_json_error( $response['message'] );
    }
    $data = $response['data'];

    wp_send_json( array(
      'success' => true,
      'data' => $response['data']
    ) );
  }

  public static function import_variation() {
    ob_start();

    check_ajax_referer( 'product_import', 'security' );

    $api_key = get_option( 'udesign_api_key', '' );
    if ( empty( $api_key ) ) {
      self::send_json_error( 'Invalid API key' );
    }

    if ( empty( $_POST['product_id'] ) ) {
      self::send_json_error( 'Must provide a product' );
    }
    $product_id = $_POST['product_id'];

    if ( empty( $_POST['remote_product_id'] ) ) {
      self::send_json_error( 'Must provide a remote product' );
    }
    $remote_product_id = $_POST['remote_product_id'];

    if ( empty( $_POST['variation_id'] ) ) {
      self::send_json_error( 'Must provide a remote product variation' );
    }
    $remote_variation_id = $_POST['variation_id'];

    if ( empty( $_POST['metadata'] ) ) {
      self::send_json_error( 'Must provide a remote product metadata' );
    }

    $remote_product_meta = vam_udesign_json_decode( $_POST['metadata'] );
    if ( empty( $remote_product_meta ) ) {
      self::send_json_error( 'Could not read remote product metadata' );
    }

    // Get remote API client
    $remote_client = vam_udesign_new_wordpress_api_client(
      VAM_UDESIGN_API_HOST,
      get_option( 'udesign_remote_products_api_consumer_key' ),
      get_option( 'udesign_remote_products_api_consumer_secret' )
    );
    if ( empty( $remote_client ) ) {
      self::send_json_error( 'Invalid remote products API credentials' );
    }

    // Get local API client
    $local_client = vam_udesign_new_wordpress_api_client(
      get_site_url(),
      get_option( 'udesign_local_products_api_consumer_key' ),
      get_option( 'udesign_local_products_api_consumer_secret' )
    );
    if ( empty( $local_client ) ) {
      self::send_json_error( 'Invalid local products API credentials' );
    }

    // Import variation
    $product_importer = vam_udesign_new_product_importer( $remote_client, $local_client );
    if ( empty( $product_importer ) ) {
      self::send_json_error( 'Could not create product importer' );
    }

    $response = $product_importer->import_variation( $product_id, $remote_product_id, $remote_variation_id, $remote_product_meta );
    if ( empty( $response['success'] ) ) {
      self::send_json_error( $response['message'] );
    }
    $variation_id = $response['data'];

    wp_send_json( array(
      'success' => true,
      'data' => array(
        'variation_id' => $variation_id,
      )
    ) );
  }

  /**
   * Load product color attribute editor via AJAX.
   */
  public static function load_edit_color_attribute() {
    ob_start();

    check_ajax_referer( 'load-edit-color-attribute', 'security' );

    if ( ! current_user_can( 'manage_product_terms' ) ) {
      die(-1);
    }

    if ( empty( $_POST['taxonomy'] ) ) {
      die(-1);
    }

    $taxonomy = esc_attr( $_POST['taxonomy'] );
    $name = empty( $_POST['term'] ) ? '' : wc_clean( $_POST['term'] );
    if ( ! empty( $name ) ) {
      $term         = get_term_by( 'name', $name, $taxonomy );
      $swatch_term  = new VAM_UDesign_WC_Swatch_Term( $term->term_id );
      $color        = $swatch_term->get_color();
    }

    if ( empty( $term ) ) {
      $term     = ( object ) array(
        'term_id' => 0,
        'name'    => '',
        'slug'    => '',
      );
    }

    if ( empty( $color ) ) {
      $color = '#ffffff';
    }

    include ( plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/html-edit-color-attribute.php' );
    die();
  }

  /**
   * Create or update a color attribute via ajax.
   */
  public static function save_color_attribute() {
    ob_start();

    check_ajax_referer( 'save-color-attribute', 'security' );

    if ( ! current_user_can( 'manage_product_terms' ) ) {
      die(-1);
    }

    $taxonomy = esc_attr( $_POST['taxonomy'] );
    $term_id  = wc_clean( $_POST['term_id'] );
    $name     = wc_clean( $_POST['term'] );
    $color    = wc_clean( $_POST['color'] );

    if ( taxonomy_exists( $taxonomy ) ) {

      if ( empty( $term_id ) ) {
        $result = wp_insert_term( $name, $taxonomy );
      } else {
        $result = wp_update_term( $term_id, $taxonomy, array(
          'name' =>  $name
        ) );
      }

      if ( is_wp_error( $result ) ) {
        wp_send_json( array(
          'error' => $result->get_error_message()
        ) );
      }

      $term = get_term_by( 'id', $result['term_id'], $taxonomy );
      update_woocommerce_term_meta( $term->term_id, $taxonomy . '_swatches_id_type', 'color' );
      update_woocommerce_term_meta( $term->term_id, $taxonomy . '_swatches_id_color', $color );

      wp_send_json( array(
        'term_id' => $term->term_id,
        'name'    => $term->name,
        'slug'    => $term->slug
      ) );
    }

    die();
  }

  /**
   * AJAX add to cart.
   */
  public static function add_to_cart() {
    ob_start();

    // get design id
    if ( empty( $_POST['design_id'] ) ) {
      self::send_json_error( 'Must provide a design' );
    }
    $design_id = absint( $_POST['design_id'] );

    // get variation id
    if ( empty( $_POST['variation_id'] ) ) {
      self::send_json_error( 'Must select a product first' );
    }
    $variation_id = absint( $_POST['variation_id'] );

    // get quantity
    $quantity = empty( $_POST['quantity'] ) ? 1 : wc_stock_amount( $_POST['quantity'] );

    // retrieve design meta
    $design_meta = get_post_meta( $design_id, '_design_meta', true );
    if ( empty( $design_meta ) ) {
      self::send_json_error( 'Could not retrieve design' );
    }
    $design_data = vam_udesign_json_decode( $design_meta, true );
    if ( empty( $design_data ) ) {
      self::send_json_error( 'Invalid design provided' );
    }

    // retrieve designer views
    $design_views = get_post_meta( $design_id, '_design_thumbnails', true );
    if ( empty( $design_views ) ) {
      self::send_json_error( 'The provided design is not properly configured' );
    }

    // retrieve variation attributes
    $result = self::get_designer_variation_attributes( $variation_id );
    if ( empty( $result['success'] ) || empty( $result['success'] ) ) {
      self::send_json_error( empty( $result['message'] ) ? 'Product is not properly configured' : $result['message'] );
    }
    $variation = $result['data'];

    // get product id
    $product_id = empty( $_POST['product_id'] ) ? $variation->post->post_parent : absint( $_POST['product_id'] );
    $product_id = apply_filters( 'vam_udesign_add_to_cart_product_id', $product_id );
    if ( empty( $product_id ) ) {
      $product_id = $variation->post->post_parent;
    }

    // generate cart data
    $data = self::generate_design_preview_data( 0, $design_data, $design_views, $product_id,
      $variation->product, $variation->views, $variation->color->term_id, false, 'cart' );
    if ( empty( $data ) ) {
      self::send_json_error( 'Could not generate design cart data' );
    }

    $cart_item_data = array_merge( $data, array(
      'design_id'           => $design_id,
      'is_designer_product' => true
    ) );

    // add design to cart
    $is_valid = apply_filters( 'vam_udesign_add_to_cart_validation', true, $product_id, $quantity, $variation_id, $variation->attributes, $cart_item_data );
    if ( $is_valid && false !== WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variation->attributes, $cart_item_data ) && 'publish' === get_post_status( $product_id ) ) {
      do_action( 'vam_udesign_ajax_added_to_cart', $product_id, $quantity, $variation_id, $variation->attributes, $cart_item_data );

      WC_AJAX::get_refreshed_fragments();
    }

    self::send_json_error( 'Could not add design to cart' );
  }

  public static function get_product_gallery() {
    ob_start();

    check_ajax_referer( 'load-designer-product', 'security' );

    $category_id = null;
    if ( !empty( $_POST['category_id'] ) ) {
      $category_id = absint( $_POST['category_id'] );
    }

    vam_udesign_template_single_products_gallery( $category_id );
    die();
  }

  /**
   * Toggle Featured status of a design from admin.
   */
  public static function feature_design() {
    if ( current_user_can( 'edit_products' ) && check_admin_referer( 'udesign-feature-design' ) ) {
      $design_id = absint( $_GET['design_id'] );

      if ( 'design' === get_post_type( $design_id ) ) {
        update_post_meta( $design_id, '_design_featured', get_post_meta( $design_id, '_design_featured', true ) === 'yes' ? 'no' : 'yes' );
      }
    }

    wp_safe_redirect( wp_get_referer() ? remove_query_arg( array( 'trashed', 'untrashed', 'deleted', 'ids' ), wp_get_referer() ) : admin_url( 'edit.php?post_type=design' ) );
    die();
  }
}

VAM_UDesign_Ajax::instance();
