<?php
/**
 * Cart
 *
 * @class     VAM_UDesign_Cart
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

class VAM_UDesign_Cart {

  /**
   * Hook in methods.
   */
  public static function init() {
    add_filter( 'woocommerce_add_cart_item_data', array( __CLASS__, 'add_cart_item_data' ), 9, 3 );
    add_filter( 'woocommerce_get_cart_item_from_session', array( __CLASS__, 'get_cart_item_from_session' ), 10, 3 );
    add_filter( 'woocommerce_before_calculate_totals', array( __CLASS__, 'before_calculate_totals' ), 10, 1 );
    add_filter( 'woocommerce_cart_item_thumbnail', array( __CLASS__, 'cart_item_thumbnail' ), 10, 3 );
    add_filter( 'woocommerce_cart_item_permalink', array( __CLASS__, 'cart_item_permalink' ), 10, 3 );
  }

  public static function add_cart_item_data( $cart_item_data, $product_id, $variation_id ) {
    if ( !empty( $_POST['preview_urls'] ) ) {
      $cart_item_data['preview_urls'] = $_POST['preview_urls'];
    }

    if ( !empty( $_POST['design_id'] ) ) {
      $cart_item_data['design_id'] = $_POST['design_id'];
    }

    if ( !empty( $_POST['is_designer_product'] ) ) {
      $cart_item_data['is_designer_product'] = $_POST['is_designer_product'];
    }

    return $cart_item_data;
  }

  public static function get_cart_item_from_session( $session_data, $values, $key ) {
    if ( !empty( $values['preview_urls'] ) ) {
      $session_data['preview_urls'] = $values['preview_urls'];
    }

    if ( !empty( $values['design_id'] ) ) {
      $session_data['design_id'] = $values['design_id'];
    }

    if ( !empty( $values['is_designer_product'] ) ) {
      $session_data['is_designer_product'] = $values['is_designer_product'];
    }

    return $session_data;
  }

  public static function before_calculate_totals( $cart_object ) {
    foreach ( $cart_object->cart_contents as $key => $value ) {
      if ( empty( $value['is_designer_product'] ) ) {
        continue;
      }

      VAM_UDesign_WC_Compat::set_prop( $value['data'], 'price', $value['display_price'] );

      $name = VAM_UDesign_WC_Compat::get_prop( $value['data'], 'name' );
      $prefix = __( 'Customized', 'vam-udesign' );

      if ( strpos( $name, $prefix ) !== 0 ) {
        VAM_UDesign_WC_Compat::set_prop( $value['data'], 'name', sprintf( '%s %s', $prefix, $name ) );
      }
    }
  }

  public static function cart_item_thumbnail( $product_image, $cart_item, $cart_item_key ) {
    if ( empty( $cart_item['is_designer_product'] ) ) {
      return $product_image;
    }

    if ( empty( $cart_item['preview_urls'] ) ) {
      return $product_image;
    }

    $image = '';
    foreach ( $cart_item['preview_urls'] as $view_id => $preview ) {
      $image .= sprintf( '<img src="%s" />', $preview['thumbnail'] );
    }

    if ( empty( $image) ) {
      return $product_image;
    }

    return $image;
  }

  public static function cart_item_permalink( $permalink, $cart_item, $cart_item_key ) {
    if ( !empty( $cart_item['is_designer_product'] ) ) {
      return '#';
    }

    return $permalink;
  }
}

VAM_UDesign_Cart::init();
