<?php

/**
 * Fired during plugin deactivation
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 */

/**
 * Fired during plugin deactivation.
 *
 * This class defines all code necessary to run during the plugin's deactivation.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
class VAM_UDesign_Deactivator {

  /**
   * Short Description. (use period)
   *
   * Long Description.
   *
   * @since    1.0.0
   */
  public static function deactivate() {
    global $wpdb;
    $attribute_taxonomies = wc_get_attribute_taxonomies();
    $accepted_types       = array('select', 'text');
    $default_type         = 'select';

    foreach ($attribute_taxonomies as $attribute_taxonomy) {
      $attribute_taxonomy = (array) maybe_unserialize( $attribute_taxonomy );

      if ( ! in_array($attribute_taxonomy['attribute_type'], $accepted_types ) ) {
        $update = $wpdb->update(
          $wpdb->prefix . 'woocommerce_attribute_taxonomies',
          array( 'attribute_type' => $default_type ),
          array( 'attribute_id' => (int) $attribute_taxonomy['attribute_id'] ),
          array( '%s' ),
          array( '%d' )
        );

        if ( false === $update ) {
          return new WP_Error( 'udesign_cannot_edit', __( 'Could not edit the attribute.', 'vam-udesign' ), array( 'status' => 400 ) );
        }
      }
    }

    delete_transient( 'wc_attribute_taxonomies' );
  }
}
