<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

/**
 * VAM_UDesign_Media class
 *
 * @class       VAM_UDesign_Media
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
class VAM_UDesign_Media {

  /**
   * Init shortcodes.
   */
  public static function init() {
    add_filter( 'wp_check_filetype_and_ext', array( __CLASS__, 'check_filetype_and_ext' ), 10, 4 );
    add_filter( 'upload_mimes', array( __CLASS__, 'allow_additional_mime_types' ), 10, 1 );
    add_filter( 'post_mime_types', array( __CLASS__, 'allow_additional_mime_type_filters' ), 10, 1 );
  }

  public static function check_filetype_and_ext( $data, $file, $filename, $mimes ) {
    global $wp_version;
    if ( version_compare( $wp_version, '4.7.1' ) < 0 ) {
      return $data;
    }

    $filetype = wp_check_filetype( $filename, $mimes );

    return array(
      'ext'             => $filetype['ext'],
      'type'            => $filetype['type'],
      'proper_filename' => $data['proper_filename']
    );
  }

  public static function allow_additional_mime_types( $mimes ) {
    $mimes['svg'] = 'image/svg+xml';
    return $mimes;
  }

  public static function allow_additional_mime_type_filters( $mimes ) {
    $mimes['image/svg+xml'] = array(
      __( 'SVG Images' ),
      __( 'Manage SVG' ),
      _n_noop( 'SVG <span class="count">(%s)</span>', 'SVG <span class="count">(%s)</span>' )
    );

    $mimes['image/png'] = array(
      __( 'PNG Images' ),
      __( 'Manage PNG' ),
      _n_noop( 'PNG <span class="count">(%s)</span>', 'PNG <span class="count">(%s)</span>' )
    );

    $mimes['image/jpeg'] = array(
      __( 'JPEG Images' ),
      __( 'Manage JPEG' ),
      _n_noop( 'JPEG <span class="count">(%s)</span>', 'JPEG <span class="count">(%s)</span>' )
    );

    return $mimes;
  }
}
