<?php

/**
 * Runs plugin data migrations after plugin activation
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 */

/**
 * Fired after plugin activation.
 *
 * This class migrates all default data after plugin activation.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

class VAM_UDesign_Migrations {
  public static function run() {
    if ( !get_transient( 'vam-udesign-execute-migrations' ) ) {
      return;
    }
    set_transient( 'vam-udesign-execute-migrations', false, 0 );

    self::migrate_product_attribute_terms();
  }

  public static function migrate_product_attributes() {
    self::insert_product_attribute_taxonomy( 'pa_color', array(
      'attribute_label'   => __( 'Color', 'vam-udesign' ),
			'attribute_name'    => 'color',
			'attribute_type'    => 'custom',
    ) );

    self::insert_product_attribute_taxonomy( 'pa_size', array(
      'attribute_label'   => __( 'Size', 'vam-udesign' ),
			'attribute_name'    => 'size',
			'attribute_type'    => 'custom',
    ) );

    self::insert_product_attribute_taxonomy( 'pa_view', array(
      'attribute_label'   => __( 'View', 'vam-udesign' ),
			'attribute_name'    => 'view',
			'attribute_type'    => 'select',
    ) );
  }

  public static function migrate_product_attribute_terms() {
    // Add size terms
    self::insert_product_attribute_term( 'XS', 'pa_size', 'label' );
    self::insert_product_attribute_term( 'S', 'pa_size', 'label' );
    self::insert_product_attribute_term( 'M', 'pa_size', 'label' );
    self::insert_product_attribute_term( 'L', 'pa_size', 'label' );
    self::insert_product_attribute_term( 'XL', 'pa_size', 'label' );
    self::insert_product_attribute_term( 'XXL', 'pa_size', 'label' );

    // Add view terms
    self::insert_product_attribute_term( 'Front', 'pa_view' );
    self::insert_product_attribute_term( 'Back', 'pa_view' );
    self::insert_product_attribute_term( 'Side', 'pa_view' );
  }

  private static function insert_product_attribute_taxonomy( $name, $attribute_data ) {
    if ( empty( $name ) || empty( $attribute_data ) ) {
      return;
    }
    if ( taxonomy_exists( $name ) ) {
      return;
    }

    $attribute_id = VAM_UDesign_WC_Compat::wc_attribute_taxonomy_id_by_name( $name );
    if ( $attribute_id ) {
      $_POST['save_attribute'] = true;
      $_GET['edit'] = $attribute_id;
      $_REQUEST['_wpnonce'] = wp_create_nonce( "woocommerce-save-attribute_$attribute_id" );
    } else {
      $_POST['add_new_attribute'] = true;
      $_REQUEST['_wpnonce'] = wp_create_nonce( 'woocommerce-add-new_attribute' );
    }

    $_POST = array_merge( $_POST, $attribute_data );

    ob_start();
    WC_Admin_Attributes::output();
    ob_end_clean();
  }

  private static function insert_product_attribute_term( $name, $taxonomy, $type = '' ) {
    if ( empty( $name ) || empty( $taxonomy ) ) {
      return;
    }

    $term = get_term_by( 'name', $name, $taxonomy, 'ARRAY_A' );
    if ( empty( $term ) ) {
      $term = wp_insert_term( $name, $taxonomy );
      if ( empty ( $term ) || is_wp_error( $term ) ) {
        return;
      }
    }

    if ( !empty( $type ) ) {
      update_woocommerce_term_meta( $term['term_id'], $taxonomy . '_swatches_id_type', $type );
    }
  }
}
