<?php
/**
 * Order
 *
 * @class     VAM_UDesign_Order
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

class VAM_UDesign_Order {

  private static $hidden_order_item_meta = array(
    'preview_urls',
    'design_id',
    'design_meta',
    'is_designer_product'
  );

  /**
   * Hook in methods.
   */
  public static function init() {
    add_filter( 'woocommerce_admin_order_item_thumbnail', array( __CLASS__, 'admin_order_item_thumbnail' ), 10, 3 );
    add_action( 'woocommerce_order_item_meta_end', array( __CLASS__, 'order_item_meta_end_images' ), 10, 3 );
    add_action( 'woocommerce_order_item_meta_end', array( __CLASS__, 'order_item_meta_end_designer_link' ), 20, 3 );
    add_filter( 'woocommerce_order_item_get_formatted_meta_data', array( __CLASS__, 'order_item_get_formatted_meta_data' ), 10, 2 );
    add_filter( 'woocommerce_order_items_meta_get_formatted', array( __CLASS__, 'order_item_get_formatted_meta_data' ), 10, 2 );
    add_filter( 'woocommerce_hidden_order_itemmeta', array( __CLASS__, 'hidden_order_itemmeta' ), 10, 1 );
    add_action( 'woocommerce_order_item_line_item_html', array( __CLASS__, 'order_item_line_item_html' ), 10, 3 );
    add_action( 'woocommerce_admin_order_items_after_line_items', array( __CLASS__, 'admin_order_items_after_line_items' ), 10, 1 );
    add_action( 'woocommerce_recorded_sales', array( __CLASS__, 'recorded_sales' ), 10, 1 );

    if ( VAM_UDesign_WC_Compat::is_wc_version_gte( '3.0' ) ) {
      add_action( 'woocommerce_checkout_create_order_line_item', array( __CLASS__, 'checkout_create_order_line_item' ), 10, 4 );
    } else {
      add_action( 'woocommerce_add_order_item_meta', array( __CLASS__, 'add_order_item_meta' ), 10, 3 );
    }
  }

  public static function admin_order_item_thumbnail( $item_image, $item_id, $item ) {
    if ( empty( wc_get_order_item_meta( $item_id, 'is_designer_product', true ) ) ) {
      return $item_image;
    }

    $image = '';
    $preview_urls = vam_udesign_json_decode( wc_get_order_item_meta( $item_id, 'preview_urls', true ), true );
    foreach ( $preview_urls as $view_id => $preview ) {
      $image .= sprintf( '<img src="%s" />', $preview['thumbnail'] );
    }

    if ( empty( $image) ) {
      return $item_image;
    }

    return $image;
  }

  public static function order_item_meta_end_images( $item_id, $item, $order ) {
    if ( empty( wc_get_order_item_meta( $item_id, 'is_designer_product', true ) ) ) {
      return;
    }

    $images = '';
    $preview_urls = vam_udesign_json_decode( wc_get_order_item_meta( $item_id, 'preview_urls', true ), true );
    foreach ( $preview_urls as $view_id => $preview ) {
      $images .= sprintf( '<img src="%s" />', $preview['thumbnail'] );
    }

    echo '<div class="order-item-thumbnails">' . $images . '</div>';
  }

  public static function order_item_meta_end_designer_link( $item_id, $item, $order ) {
    if ( empty( wc_get_order_item_meta( $item_id, 'is_designer_product', true ) ) ) {
      return;
    }

    $designer_page_id = get_option( 'udesign_designer_page_id' );
    if ( empty( $designer_page_id ) ) {
      return;
    }

    $designer_page_url = get_permalink( $designer_page_id );
    if ( empty( $designer_page_url ) ) {
      return;
    }

    $designer_args = array(
      'product_id'    => VAM_UDesign_WC_Compat::get_prop( $item, 'variation_id' ),
      'order_id'      => VAM_UDesign_WC_Compat::get_id( $order ),
      'order_item_id' => $item_id,
    );

    echo '<div class="order-item-designer-link"><a href="' . add_query_arg( $designer_args, $designer_page_url ) . '" target="_blank">' . __( 'Open in designer', 'vam-udesign' ) . '</a></div>';
  }

  public static function order_item_get_formatted_meta_data( $formatted_meta, $order_item_meta ) {
    foreach ( $formatted_meta as $meta_id => $meta ) {
      $meta = (object) $meta;
      if ( in_array( $meta->key, self::$hidden_order_item_meta ) ) {
        unset( $formatted_meta[$meta_id] );
      }
    }

    return $formatted_meta;
  }

  public static function hidden_order_itemmeta( $hidden_order_itemmeta ) {
    return array_merge( $hidden_order_itemmeta, self::$hidden_order_item_meta );
  }

  public static function checkout_create_order_line_item( $item, $cart_item_key, $values, $order ) {
    if ( empty( $values['is_designer_product'] ) ) {
      return;
    }

    $item->add_meta_data( 'is_designer_product', true, true );
    $item->add_meta_data( 'preview_urls', json_encode( $values['preview_urls'] ), true );

    // add item design id
    $design_id = $values['design_id'];
    $item->add_meta_data( 'design_id', $design_id, true );

    // add item design meta
    $design_meta = get_post_meta( $design_id, '_design_meta', true );
    if ( !empty( $design_meta ) ) {
      $item->add_meta_data( 'design_meta', $design_meta, true );
    }
  }

  public static function add_order_item_meta( $item_id, $values, $cart_item_key ) {
    if ( empty( $values['is_designer_product'] ) ) {
      return;
    }

    wc_add_order_item_meta( $item_id, 'is_designer_product', true, true );
    wc_add_order_item_meta( $item_id, 'preview_urls', json_encode( $values['preview_urls'] ), true );

    // add item design id
    $design_id = $values['design_id'];
    wc_add_order_item_meta( $item_id, 'design_id', $design_id, true );

    // add item design meta
    $design_meta = get_post_meta( $design_id, '_design_meta', true );
    if ( !empty( $design_meta ) ) {
      wc_add_order_item_meta( $item_id, 'design_meta', $design_meta, true );
    }
  }

  public static function order_item_line_item_html( $item_id, $item, $order ) {
    if ( empty( wc_get_order_item_meta( $item_id, 'is_designer_product', true ) ) ) {
      return;
    }

    include ( plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/html-order-item.php' );
  }

  public static function admin_order_items_after_line_items( $order_id ) {
    include ( plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/html-order-item-designer.php' );
  }

  public static function recorded_sales( $order_id ) {
    $order = wc_get_order( $order_id );
    if ( empty( $order ) ) {
      return;
    }

    $order_items = $order->get_items();
    if ( empty( $order_items ) ) {
      return;
    }

    $quantity_key = 'qty';
    if ( VAM_UDesign_WC_Compat::is_wc_version_gte( '3.0.0' ) ) {
      $quantity_key = 'quantity';
    }

    foreach ( $order_items as $order_item_id => &$order_item ) {
      if ( empty( wc_get_order_item_meta( $order_item_id, 'is_designer_product', true ) ) ) {
        continue;
      }

      $design_id = wc_get_order_item_meta( $order_item_id, 'design_id', true );
      if ( empty ( $design_id ) ) {
        continue;
      }

      $total_sales = absint( get_post_meta( $design_id, 'total_sales', true ) );
      if ( empty( $total_sales ) ) {
        $total_sales = 0;
      }

      $total_sales += absint( VAM_UDesign_WC_Compat::get_prop( $order_item, $quantity_key ) );
      update_post_meta( $design_id, 'total_sales', $total_sales );
    }
  }
}
