<?php
/**
 * Post Types
 *
 * Registers post types and taxonomies.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * VAM_UDesign_Post_types Class.
 */
class VAM_UDesign_Post_types {

  /**
   * Hook in methods.
   */
  public static function init() {
    add_action( 'init', array( __CLASS__, 'register_taxonomies' ), 5 );
    add_action( 'init', array( __CLASS__, 'register_post_types' ), 5 );
    add_action( 'init', array( __CLASS__, 'register_post_statuses' ), 5 );
    add_action( 'init', array( __CLASS__, 'support_jetpack_omnisearch' ) );
    add_filter( 'rest_api_allowed_post_types', array( __CLASS__, 'rest_api_allowed_post_types' ) );
  }

  /**
   * Register core taxonomies.
   */
  public static function register_taxonomies() {
    if ( taxonomy_exists( 'clipart_cat' ) ) {
      return;
    }

    register_taxonomy( 'clipart_cat', array( 'clipart' ), array(
      'hierarchical'          => true,
      'public'                => true,
      'show_ui'               => true,
      'show_admin_column'     => true,
      'show_in_nav_menus'     => true,
      'show_tagcloud'         => true,
      'query_var'             => true,
      'update_count_callback' => '',
      'label'                 => __( 'Clipart categories', 'vam-udesign' ),
      'labels'                        => array(
        'name'                        => _x( 'Categories', 'Taxonomy general name', 'vam-udesign' ),
        'singular_name'               => _x( 'Category', 'Taxonomy singular name', 'vam-udesign' ),
        'menu_name'                   => _x( 'Categories', 'Admin menu name', 'vam-udesign' ),
        'all_items'                   => __( 'All Clipart Categories', 'vam-udesign' ),
        'parent_item'                 => __( 'Parent Clipart Category', 'vam-udesign' ),
        'parent_item_colon'           => __( 'Parent Clipart Category:', 'vam-udesign' ),
        'new_item_name'               => __( 'New Clipart Category Name', 'vam-udesign' ),
        'add_new_item'                => __( 'Add New Clipart Category', 'vam-udesign' ),
        'edit_item'                   => __( 'Edit Clipart Category', 'vam-udesign' ),
        'update_item'                 => __( 'Update Clipart Category', 'vam-udesign' ),
        'view_item'                   => __( 'View Clipart Category', 'vam-udesign' ),
        'separate_items_with_commas'  => __( 'Separate clipart categories with commas', 'vam-udesign' ),
        'add_or_remove_items'         => __( 'Add or remove clipart categories', 'vam-udesign' ),
        'choose_from_most_used'       => __( 'Choose from the most used', 'vam-udesign' ),
        'popular_items'               => __( 'Popular Clipart Categories', 'vam-udesign' ),
        'search_items'                => __( 'Search Clipart Categories', 'vam-udesign' ),
        'not_found'                   => __( 'Not Found', 'vam-udesign' ),
      ),
      'capabilities'          => array(
        'manage_terms' => 'manage_product_terms',
        'edit_terms'   => 'edit_product_terms',
        'delete_terms' => 'delete_product_terms',
        'assign_terms' => 'assign_product_terms',
      ),
      'rewrite'               => array(
        'slug'         => 'clipart-category',
        'with_front'   => true,
        'hierarchical' => true,
      )
    ) );

    register_taxonomy( 'clipart_tag', array( 'clipart' ), array(
      'hierarchical'          => false,
      'public'                => true,
      'show_ui'               => true,
      'show_admin_column'     => true,
      'show_in_nav_menus'     => true,
      'show_tagcloud'         => true,
      'update_count_callback' => '',
      'label'                 => __( 'Clipart tags', 'vam-udesign' ),
      'labels'                        => array(
        'name'                        => _x( 'Tags', 'Taxonomy general name', 'vam-udesign' ),
        'singular_name'               => _x( 'Tag', 'Taxonomy singular name', 'vam-udesign' ),
        'menu_name'                   => _x( 'Tags', 'Admin menu name', 'vam-udesign' ),
        'all_items'                   => __( 'All Clipart Tags', 'vam-udesign' ),
        'parent_item'                 => __( 'Parent Clipart Tag', 'vam-udesign' ),
        'parent_item_colon'           => __( 'Parent Clipart Tag:', 'vam-udesign' ),
        'new_item_name'               => __( 'New Clipart Tag Name', 'vam-udesign' ),
        'add_new_item'                => __( 'Add New Clipart Tag', 'vam-udesign' ),
        'edit_item'                   => __( 'Edit Clipart Tag', 'vam-udesign' ),
        'update_item'                 => __( 'Update Clipart Tag', 'vam-udesign' ),
        'view_item'                   => __( 'View Clipart Tag', 'vam-udesign' ),
        'separate_items_with_commas'  => __( 'Separate clipart tags with commas', 'vam-udesign' ),
        'add_or_remove_items'         => __( 'Add or remove clipart tags', 'vam-udesign' ),
        'choose_from_most_used'       => __( 'Choose from the most used', 'vam-udesign' ),
        'popular_items'               => __( 'Popular Clipart Tags', 'vam-udesign' ),
        'search_items'                => __( 'Search Clipart Tags', 'vam-udesign' ),
        'not_found'                   => __( 'Not Found', 'vam-udesign' ),
      ),
      'capabilities'          => array(
        'manage_terms' => 'manage_product_terms',
        'edit_terms'   => 'edit_product_terms',
        'delete_terms' => 'delete_product_terms',
        'assign_terms' => 'assign_product_terms',
      ),
      'rewrite'               => array(
        'slug'         => 'clipart-tag',
        'with_front'   => true,
        'hierarchical' => true,
      )
    ) );

    register_taxonomy( 'design_cat', array( 'design' ), array(
      'hierarchical'          => true,
      'public'                => true,
      'show_ui'               => true,
      'show_admin_column'     => true,
      'show_in_nav_menus'     => true,
      'show_tagcloud'         => true,
      'query_var'             => true,
      'update_count_callback' => '',
      'label'                 => __( 'Design categories', 'vam-udesign' ),
      'labels'                        => array(
        'name'                        => _x( 'Categories', 'Taxonomy general name', 'vam-udesign' ),
        'singular_name'               => _x( 'Category', 'Taxonomy singular name', 'vam-udesign' ),
        'menu_name'                   => _x( 'Categories', 'Admin menu name', 'vam-udesign' ),
        'all_items'                   => __( 'All Design Categories', 'vam-udesign' ),
        'parent_item'                 => __( 'Parent Design Category', 'vam-udesign' ),
        'parent_item_colon'           => __( 'Parent Design Category:', 'vam-udesign' ),
        'new_item_name'               => __( 'New Design Category Name', 'vam-udesign' ),
        'add_new_item'                => __( 'Add New Design Category', 'vam-udesign' ),
        'edit_item'                   => __( 'Edit Design Category', 'vam-udesign' ),
        'update_item'                 => __( 'Update Design Category', 'vam-udesign' ),
        'view_item'                   => __( 'View Design Category', 'vam-udesign' ),
        'separate_items_with_commas'  => __( 'Separate design categories with commas', 'vam-udesign' ),
        'add_or_remove_items'         => __( 'Add or remove design categories', 'vam-udesign' ),
        'choose_from_most_used'       => __( 'Choose from the most used', 'vam-udesign' ),
        'popular_items'               => __( 'Popular Design Categories', 'vam-udesign' ),
        'search_items'                => __( 'Search Design Categories', 'vam-udesign' ),
        'not_found'                   => __( 'Not Found', 'vam-udesign' ),
      ),
      'capabilities'          => array(
        'manage_terms' => 'manage_product_terms',
        'edit_terms'   => 'edit_product_terms',
        'delete_terms' => 'delete_product_terms',
        'assign_terms' => 'assign_product_terms',
      ),
      'rewrite'               => array(
        'slug'         => 'design-category',
        'with_front'   => true,
        'hierarchical' => true,
      )
    ) );

    register_taxonomy( 'design_tag', array( 'design' ), array(
      'hierarchical'          => false,
      'public'                => true,
      'show_ui'               => true,
      'show_admin_column'     => true,
      'show_in_nav_menus'     => true,
      'show_tagcloud'         => true,
      'update_count_callback' => '',
      'label'                 => __( 'Design tags', 'vam-udesign' ),
      'labels'                        => array(
        'name'                        => _x( 'Tags', 'Taxonomy general name', 'vam-udesign' ),
        'singular_name'               => _x( 'Tag', 'Taxonomy singular name', 'vam-udesign' ),
        'menu_name'                   => _x( 'Tags', 'Admin menu name', 'vam-udesign' ),
        'all_items'                   => __( 'All Design Tags', 'vam-udesign' ),
        'parent_item'                 => __( 'Parent Design Tag', 'vam-udesign' ),
        'parent_item_colon'           => __( 'Parent Design Tag:', 'vam-udesign' ),
        'new_item_name'               => __( 'New Design Tag Name', 'vam-udesign' ),
        'add_new_item'                => __( 'Add New Design Tag', 'vam-udesign' ),
        'edit_item'                   => __( 'Edit Design Tag', 'vam-udesign' ),
        'update_item'                 => __( 'Update Design Tag', 'vam-udesign' ),
        'view_item'                   => __( 'View Design Tag', 'vam-udesign' ),
        'separate_items_with_commas'  => __( 'Separate design tags with commas', 'vam-udesign' ),
        'add_or_remove_items'         => __( 'Add or remove design tags', 'vam-udesign' ),
        'choose_from_most_used'       => __( 'Choose from the most used', 'vam-udesign' ),
        'popular_items'               => __( 'Popular Design Tags', 'vam-udesign' ),
        'search_items'                => __( 'Search Design Tags', 'vam-udesign' ),
        'not_found'                   => __( 'Not Found', 'vam-udesign' ),
      ),
      'capabilities'          => array(
        'manage_terms' => 'manage_product_terms',
        'edit_terms'   => 'edit_product_terms',
        'delete_terms' => 'delete_product_terms',
        'assign_terms' => 'assign_product_terms',
      ),
      'rewrite'               => array(
        'slug'         => 'design-tag',
        'with_front'   => true,
        'hierarchical' => true,
      )
    ) );

  }

  /**
   * Register core post types.
   */
  public static function register_post_types() {
    if ( post_type_exists( 'clipart' ) ) {
      return;
    }

    register_post_type( 'clipart', array(
      'label'               => __( 'clipart', 'vam-udesign' ),
      'description'         => __( 'Clipart for designer products', 'vam-udesign' ),
      'supports'            => array( 'title', 'editor', 'thumbnail', 'custom-fields', 'publicize', 'wpcom-markdown' ),
      'taxonomies'          => array( 'clipart-category', 'clipart-tag' ),
      'hierarchical'        => false,
      'public'              => false,
      'show_ui'             => true,
      'show_in_menu'        => true,
      'menu_position'       => 51,
      'menu_icon'           => 'dashicons-art',
      'show_in_admin_bar'   => true,
      'show_in_nav_menus'   => true,
      'can_export'          => true,
      'exclude_from_search' => true,
      'publicly_queryable'  => false,
      'capability_type'     => 'page',
      'map_meta_cap'        => true,
      'query_var'           => true,
      'has_archive'         => false,
      'rewrite'             => false,
      'labels'              => array(
        'name'                  => _x( 'Cliparts', 'Post type general name', 'vam-udesign' ),
        'singular_name'         => _x( 'Clipart', 'Post type singular name', 'vam-udesign' ),
        'menu_name'             => _x( 'Cliparts', 'Admin menu name', 'vam-udesign' ),
        'name_admin_bar'        => _x( 'Clipart', 'Admin bar name', 'vam-udesign' ),
        'parent_item_colon'     => __( 'Parent Clipart:', 'vam-udesign' ),
        'all_items'             => __( 'All Cliparts', 'vam-udesign' ),
        'add_new_item'          => __( 'Add New Clipart', 'vam-udesign' ),
        'add_new'               => __( 'Add New', 'vam-udesign' ),
        'new_item'              => __( 'New Clipart', 'vam-udesign' ),
        'edit_item'             => __( 'Edit Clipart', 'vam-udesign' ),
        'update_item'           => __( 'Update Clipart', 'vam-udesign' ),
        'view_item'             => __( 'View Clipart', 'vam-udesign' ),
        'search_items'          => __( 'Search Clipart', 'vam-udesign' ),
        'not_found'             => __( 'Not found', 'vam-udesign' ),
        'not_found_in_trash'    => __( 'Not found in Trash', 'vam-udesign' ),
      ),
    ) );

    if ( post_type_exists( 'design' ) ) {
      return;
    }

    $design_permalink = _x( 'design', 'slug', 'vam-udesign' );
    register_post_type( 'design', array(
      'label'               => __( 'design', 'vam-udesign' ),
      'description'         => __( 'Design for designer products', 'vam-udesign' ),
      'supports'            => array( 'title', 'editor', 'thumbnail', 'author', 'excerpt', 'comments', 'custom-fields', 'publicize', 'wpcom-markdown' ),
      'taxonomies'          => array( 'design-category', 'design-tag' ),
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => true,
      'menu_position'       => 52,
      'menu_icon'           => 'dashicons-admin-appearance',
      'show_in_admin_bar'   => true,
      'show_in_nav_menus'   => true,
      'can_export'          => true,
      'exclude_from_search' => false,
      'publicly_queryable'  => true,
      'capability_type'     => 'page',
      'map_meta_cap'        => true,
      'query_var'           => true,
      'has_archive'         => ( $designs_page_id = get_option( 'udesign_designs_page_id' ) ) && get_post( $designs_page_id ) ? get_page_uri( $designs_page_id ) : 'designs',
      'rewrite'             => $design_permalink ? array( 'slug' => untrailingslashit( $design_permalink ), 'with_front' => false, 'feeds' => true ) : false,
      'labels'              => array(
        'name'                  => _x( 'Designs', 'Post type general name', 'vam-udesign' ),
        'singular_name'         => _x( 'Design', 'Post type singular name', 'vam-udesign' ),
        'menu_name'             => _x( 'Designs', 'Admin menu name', 'vam-udesign' ),
        'name_admin_bar'        => _x( 'Design', 'Admin bar name', 'vam-udesign' ),
        'parent_item_colon'     => __( 'Parent Design:', 'vam-udesign' ),
        'all_items'             => __( 'All Designs', 'vam-udesign' ),
        'add_new_item'          => __( 'Add New Design', 'vam-udesign' ),
        'add_new'               => __( 'Add New', 'vam-udesign' ),
        'new_item'              => __( 'New Design', 'vam-udesign' ),
        'edit_item'             => __( 'Edit Design', 'vam-udesign' ),
        'update_item'           => __( 'Update Design', 'vam-udesign' ),
        'view_item'             => __( 'View Design', 'vam-udesign' ),
        'search_items'          => __( 'Search Design', 'vam-udesign' ),
        'not_found'             => __( 'Not found', 'vam-udesign' ),
        'not_found_in_trash'    => __( 'Not found in Trash', 'vam-udesign' ),
      ),
    ) );

  }

  /**
   * Register core taxonomies.
   */
  public static function register_post_statuses() {
    register_post_status( 'cart', array(
      'label'                     => __( 'Cart', 'vam-udesign' ),
      'public'                    => false,
      'exclude_from_search'       => false,
      'show_in_admin_all_list'    => false,
      'show_in_admin_status_list' => true,
      'label_count'               => _n_noop( 'Cart <span class="count">(%s)</span>', 'Cart <span class="count">(%s)</span>' ),
    ) );

    register_post_status( 'user', array(
      'label'                     => __( 'User', 'vam-udesign' ),
      'public'                    => false,
      'exclude_from_search'       => false,
      'show_in_admin_all_list'    => false,
      'show_in_admin_status_list' => true,
      'label_count'               => _n_noop( 'User <span class="count">(%s)</span>', 'User <span class="count">(%s)</span>' ),
    ) );
  }

  /**
   * Add Product Support to Jetpack Omnisearch.
   */
  public static function support_jetpack_omnisearch() {
    if ( class_exists( 'Jetpack_Omnisearch_Posts' ) ) {
      new Jetpack_Omnisearch_Posts( 'design' );
    }
  }

  /**
   * Added product for Jetpack related posts.
   *
   * @param  array $post_types
   * @return array
   */
  public static function rest_api_allowed_post_types( $post_types ) {
    $post_types[] = 'design';

    return $post_types;
  }
}

VAM_UDesign_Post_types::init();
