<?php

class VAM_UDesign_Product_Importer {

  private $remote_client;
  private $local_client;
  private $attribute_assoc = array(
    'Color' => 'pa_color',
    'Size' => 'pa_size',
    'View' => 'pa_view'
  );

  private $allowed_product_meta = array(
    '_vam_udesign_views',
    '_designer',
    '_vam_udesign_search_field',
    '_vam_udesign_remote_id'
  );

  public function __construct( $remote_client, $local_client ) {
    $this->remote_client = $remote_client;
    $this->local_client = $local_client;
  }

  private function process_categories( &$product ) {
    if ( empty( $product['categories'] ) ) {
      return;
    }

    $categories = &$product['categories'];
    foreach( $categories as &$category ) {
      if ( empty( $category['slug'] ) ) {
        continue;
      }

      $category_id = vam_udesign_create_term( $category['slug'], 'product_cat' );
      if ( empty( $category_id ) ) {
        continue;
      }

      $category['id'] = $category_id;
      unset( $category['name'] );
      unset( $category['slug'] );
    }
  }

  private function process_tags( &$product ) {
    if ( empty( $product['tags'] ) ) {
      return;
    }

    $tags = &$product['tags'];
    foreach( $tags as &$tag ) {
      if ( empty( $tag['slug'] ) ) {
        continue;
      }

      $tag_id = vam_udesign_create_term( $tag['slug'], 'product_tag' );
      if ( empty( $tag_id ) ) {
        continue;
      }

      $tag['id'] = $tag_id;
      unset( $tag['name'] );
      unset( $tag['slug'] );
    }
  }

  private function process_image( &$image ) {
    if ( empty( $image['src'] ) ) {
      return;
    }

    unset( $image['id'] );
    unset( $image['date_created'] );
    unset( $image['date_created_gmt'] );
    unset( $image['date_modified'] );
    unset( $image['date_modified_gmt'] );
  }

  private function process_images( &$product ) {
    if ( empty( $product['images'] ) ) {
      return;
    }

    $images = &$product['images'];
    foreach( $images as &$image ) {
      $this->process_image( $image );
    }
  }

  private function process_variation_meta( $product_id, &$variation, &$meta ) {
    if ( empty( $variation['image'] ) ) {
      return;
    }
    if ( empty( $meta['view_metas'] ) ) {
      return;
    }
    $remote_views = $meta['view_metas'];

    $variation_image = &$variation['image'];
    $this->process_image( $variation_image );
    unset( $variation_image['src'] );

    if ( empty( $variation['attributes'] ) ) {
      return;
    }

    // Get variation color attribute ID
    $color_id = false;
    $remote_color_id = false;

    $attributes = &$variation['attributes'];
    foreach ( $attributes as &$attribute ) {
      if ( empty( $attribute['name'] ) || empty( $attribute['option'] ) ) {
        continue;
      }

      $attribute_name = $attribute['name'];
      if ( empty( $this->attribute_assoc[$attribute_name] ) ) {
        continue;
      }

      $taxonomy = $this->attribute_assoc[$attribute_name];
      if ( $taxonomy != 'pa_color' ) {
        continue;
      }

      $term_name = $attribute['option'];
      $color_id = vam_udesign_create_term( $term_name, $taxonomy, 'name' );

      $remote_color_term = $this->get_remote_term_from_meta( 'name', $term_name, 'colors', $meta );
      if ( !empty( $remote_color_term['id'] ) ) {
        $remote_color_id = $remote_color_term['id'];
      }
    }

    if ( empty( $color_id ) || empty( $remote_color_id ) ) {
      return;
    }

    $designer_views = maybe_unserialize( get_post_meta( $product_id, '_vam_udesign_views', true ) );
    if ( empty( $designer_views ) ) {
      return;
    }

    $variation_image_id = 0;
    foreach ( $designer_views as $view_id => &$designer_view ) {
      if ( empty( $designer_view['colors'] ) ) {
        continue;
      }

      $view_colors = &$designer_view['colors'];
      if ( !empty( $view_colors[$color_id] ) ) {
        if ( empty( $variation_image_id ) ) {
          $variation_image_id = $view_colors[$color_id];
        }

        continue;
      }

      $view_term = get_term( $view_id, 'pa_view' );
      if ( empty( $view_term ) ) {
        continue;
      }

      $remote_view_term = $this->get_remote_term_from_meta( 'name', $view_term->name, 'views', $meta );
      if ( empty( $remote_view_term['id'] ) ) {
        continue;
      }

      if ( empty( $remote_views[$remote_view_term['id']] ) ) {
        continue;
      }
      $remote_view = $remote_views[$remote_view_term['id']];

      if ( empty( $remote_view['colors'][$remote_color_id]['image'] ) ) {
        continue;
      }
      $remote_image_src = $remote_view['colors'][$remote_color_id]['image'];

      $uploaded_image = vam_udesign_upload_remote_image( $remote_image_src );
      if ( empty( $uploaded_image ) ) {
        continue;
      }

      $color_image_id = vam_udesign_create_attachment( $uploaded_image, 0 );
      if ( empty( $color_image_id ) ) {
        continue;
      }

      $view_colors[$color_id] = $color_image_id;
      if ( empty( $variation_image_id ) ) {
        $variation_image_id = $color_image_id;
      }
    }

    if ( !empty( $variation_image_id ) ) {
      $variation_image['id'] = $variation_image_id;
    }

    update_post_meta( $product_id, '_vam_udesign_views', $designer_views );
  }

  private function process_attribute( &$attribute, $meta = array() ) {
    if ( empty( $attribute['name'] ) ) {
      return;
    }
    $attribute_name = $attribute['name'];

    if ( empty( $this->attribute_assoc[$attribute_name] ) ) {
      return;
    }
    $taxonomy = $this->attribute_assoc[$attribute_name];

    $taxonomy_id = VAM_UDesign_WC_Compat::wc_attribute_taxonomy_id_by_name( $taxonomy );
    if ( empty( $taxonomy_id ) ) {
      return;
    }
    $attribute['id'] = $taxonomy_id;

    if ( empty( $attribute['options'] ) ) {
      return;
    }

    $options = $attribute['options'];
    foreach( $options as $option ) {
      if ( empty( $option ) ) {
        continue;
      }

      $term_id = vam_udesign_create_term( $option, $taxonomy, 'name' );
      if ( empty( $term_id ) ) {
        continue;
      }

      $term_type = false;
      switch ( $taxonomy ) {
      case 'pa_color':
        $term_type = 'color';

        $remote_term = $this->get_remote_term_from_meta( 'name', $option, 'colors', $meta );
        if ( !empty( $remote_term['color'] ) ) {
          update_woocommerce_term_meta( $term_id, $taxonomy . '_swatches_id_color', $remote_term['color'] );
        }

        break;

      case 'pa_size':
        $term_type = 'label';
        break;
      }

      if ( !empty( $term_type ) ) {
        update_woocommerce_term_meta( $term_id, $taxonomy . '_swatches_id_type', $term_type );
      }
    }
  }

  private function process_attributes( &$product, $meta = array() ) {
    if ( empty( $product['attributes'] ) ) {
      return;
    }

    $attributes = &$product['attributes'];
    foreach( $attributes as &$attribute ) {
      $this->process_attribute( $attribute, $meta );
    }
  }

  private function process_default_attributes( &$product ) {
    if ( empty( $product['default_attributes'] ) ) {
      return;
    }

    $attributes = &$product['default_attributes'];
    foreach( $attributes as &$attribute ) {
      $this->process_attribute( $attribute );
    }
  }

  private function process_view_metadata( $id, &$remote_view, &$meta ) {
    if ( empty( $meta['view_metas'][$id] ) ) {
      return false;
    }

    $meta_view = $meta['view_metas'][$id];
    if ( empty( $meta_view['image'] ) || empty( $meta_view['image_id'] ) ) {
      return false;
    }
    if ( empty( $meta_view['colors'] ) ) {
      return false;
    }

    // Upload view image
    $meta_view_image = $meta_view['image'];
    $meta_view_image_id = $meta_view['image_id'];

    $uploaded_image = vam_udesign_upload_remote_image( $meta_view_image );
    if ( empty( $uploaded_image ) ) {
      return false;
    }

    $image_id = vam_udesign_create_attachment( $uploaded_image, 0 );
    if ( empty( $image_id ) ) {
      return false;
    }

    // Upload view color images
    $meta_colors = &$meta_view['colors'];

    $view_colors = array();
    foreach( $meta_colors as $meta_color_id => $meta_color ) {
      if ( empty( $meta_color['image'] ) || empty( $meta_color['image_id'] ) ) {
        continue;
      }

      $meta_color_image = $meta_color['image'];
      $meta_color_image_id = $meta_color['image_id'];

      $color_id = $this->get_local_term_from_meta( $meta_color_id, 'colors', 'pa_color', $meta );
      if ( empty( $color_id ) ) {
        continue;
      }

      $color_image_id = false;
      if ( $meta_color_image_id == $meta_view_image_id ) {
        $color_image_id = $image_id;
      } else {
        $color_image_id = 0;
      }

      $colors[$color_id] = $color_image_id;
    }

    $view = $remote_view;
    $view['colors'] = $colors;
    $view['image_id'] = $image_id;

    return $view;
  }

  private function process_metadata( &$product, &$meta ) {
    if ( empty( $product['meta_data'] ) ) {
      return;
    }
    if ( empty( $meta ) ) {
      return;
    }

    $product_metas = &$product['meta_data'];
    foreach( $product_metas as &$product_meta ) {
      unset( $product_meta['id'] );

      if ( empty( $product_meta['key'] ) ) {
        continue;
      }
      if ( $product_meta['key'] != '_vam_udesign_views' ) {
        continue;
      }
      if ( empty( $product_meta['value'] ) ) {
        continue;
      }

      // Process product view metas
      $meta_views = &$product_meta['value'];

      $views = array();
      foreach( $meta_views as $meta_view_id => &$meta_view ) {
        $view_id = $this->get_local_term_from_meta( $meta_view_id, 'views', 'pa_view', $meta );
        if ( empty( $view_id ) ) {
          continue;
        }

        $view = $this->process_view_metadata( $meta_view_id, $meta_view, $meta, $views );
        if ( empty( $view ) ) {
          continue;
        }

        $views[$view_id] = $view;
      }

      $product_meta['value'] = &$views;
    }

    if ( !empty( $product['id'] ) ) {
      $product_metas[] = array(
        'key' => '_vam_udesign_remote_id',
        'value' => $product['id']
      );
    }

    unset ( $product['meta_data'] );
    return $product_metas;
  }

  private function save_metadata( $product_id, $product_metas ) {
    if ( empty( $product_id ) ) {
      return;
    }
    if ( empty( $product_metas ) ) {
      return;
    }

    foreach( $product_metas as &$product_meta ) {
      if ( empty( $product_meta['key'] ) ) {
        continue;
      }

      $key = $product_meta['key'];
      if ( !in_array( $key, $this->allowed_product_meta ) ) {
        continue;
      }

      if ( empty( $product_meta['value'] ) ) {
        continue;
      }
      $value = $product_meta['value'];

      update_post_meta( $product_id, $key, $value );
    }
  }

  private function get_local_term_from_meta( $id, $key, $taxonomy, &$meta ) {
    if ( empty( $meta[$key] ) ) {
      return false;
    }

    $terms = &$meta[$key];
    foreach( $terms as &$term ) {
      if ( empty( $term['id'] ) || $term['id'] != $id ) {
        continue;
      }
      if ( empty( $term['name'] ) ) {
        return false;
      }

      return vam_udesign_create_term( $term['name'], $taxonomy );
    }

    return false;
  }

  private function get_remote_term_from_meta( $field, $value, $key, &$meta ) {
    if ( empty( $meta[$key] ) ) {
      return false;
    }

    $terms = &$meta[$key];
    foreach( $terms as &$term ) {
      if ( empty( $term[$field] ) || $term[$field] != $value ) {
        continue;
      }

      return $term;
    }

    return false;
  }

  public function import_product( $remote_product_id, $remote_product_meta ) {
    $remote_product_id = absint( $remote_product_id );
    if ( empty( $remote_product_id ) ) {
      return vam_udesign_error_response( 'Invalid product provided' );
    }

    // Get remote product
    $response = $this->remote_client->get( sprintf( 'products/%d', $remote_product_id ) );
    if ( empty( $response['success'] ) ) {
      return $response;
    }

    $product = $response['data'];
    if ( empty( $product ) ) {
      return vam_udesign_error_response( 'Could not retrieve product to import' );
    }

    // Process product categories
    $this->process_categories( $product );

    // Process product tags
    $this->process_tags( $product );

    // Process images
    $this->process_images( $product );

    // Process attributes
    $this->process_attributes( $product, $remote_product_meta );

    // Process default attributes
    $this->process_default_attributes( $product );

    // Process metadata
    $product_metas = $this->process_metadata( $product, $remote_product_meta );

    $remote_variations = array();
    if ( !empty( $product['variations'] ) ) {
      $remote_variations = $product['variations'];
    }

    // Unset unused properties
    unset( $product['id'] );
    unset( $product['slug'] );
    unset( $product['permalink'] );
    unset( $product['sku'] );
    unset( $product['related_ids'] );
    unset( $product['date_created'] );
    unset( $product['date_created_gmt'] );
    unset( $product['date_modified'] );
    unset( $product['date_modified_gmt'] );
    unset( $product['variations'] );

    // Create or update product
    $query = vam_udesign_get_products( array(
      'designer_products' => true,
      'remote_id' => $remote_product_id
    ) );

    $product_id = false;
    if ( !empty( $query ) && !empty( $query->posts ) ) {
      $product_id = absint( $query->posts[0] );
      $product['id'] = $product_id;
    }

    $response = false;
    if ( empty( $product_id ) ) {
      $response = $this->local_client->post( 'products', $product );
    } else {
      $response = $this->local_client->put( sprintf( 'products/%d', $product_id ), $product );
    }

    if ( empty( $response['success'] ) ) {
      return $response;
    }

    $product = $response['data'];
    if ( empty( $product['id'] ) ) {
      return vam_udesign_error_response( 'Could not create product' );
    }

    $product_id = $product['id'];
    $this->save_metadata( $product_id, $product_metas );

    return vam_udesign_success_response( array(
      'product_id' => $product_id,
      'metadata' => $remote_product_meta,
      'variations' => $remote_variations
    ) );
  }

  public function import_variation( $product_id, $remote_product_id, $remote_variation_id, $remote_product_meta ) {
    $product_id = absint( $product_id );
    if ( empty( $product_id ) ) {
      return vam_udesign_error_response( 'Invalid product provided' );
    }

    $remote_product_id = absint( $remote_product_id );
    if ( empty( $remote_product_id ) ) {
      return vam_udesign_error_response( 'Invalid remote product provided' );
    }

    $remote_variation_id = absint( $remote_variation_id );
    if ( empty( $remote_variation_id ) ) {
      return vam_udesign_error_response( 'Invalid variation provided' );
    }

    // Get remote product variation
    $response = $this->remote_client->get( sprintf( 'products/%d/variations/%d', $remote_product_id, $remote_variation_id ) );
    if ( empty( $response['success'] ) ) {
      return $response;
    }

    $variation = $response['data'];
    if ( empty( $variation ) ) {
      return vam_udesign_error_response( 'Could not retrieve product variation to import' );
    }

    // Process attributes
    $this->process_attributes( $variation );

    // Process image
    $this->process_variation_meta( $product_id, $variation, $remote_product_meta );

    // Unset unused properties
    unset( $variation['id'] );
    unset( $variation['permalink'] );
    unset( $variation['sku'] );
    unset( $variation['manage_stock'] );
    unset( $variation['date_created'] );
    unset( $variation['date_created_gmt'] );
    unset( $variation['date_modified'] );
    unset( $variation['date_modified_gmt'] );

    // Create or update variation
    $query = vam_udesign_get_products( array(
      'tax_query' => array(),
      'product_type' => false,
      'post_type' => 'product_variation',
      'remote_id' => $remote_variation_id
    ) );

    $variation_id = false;
    if ( !empty( $query ) && !empty( $query->posts ) ) {
      $variation_id = absint( $query->posts[0] );
      $variation['id'] = $variation_id;
    }

    if ( empty( $variation_id ) ) {
      $response = $this->local_client->post( sprintf( 'products/%d/variations', $product_id ), $variation );
    } else {
      $response = $this->local_client->put( sprintf( 'products/%d/variations/%d', $product_id, $variation_id ), $variation );
    }

    if ( empty( $response['success'] ) ) {
      return $response;
    }

    $variation = $response['data'];
    if ( empty( $variation['id'] ) ) {
      return vam_udesign_error_response( 'Could not create variation' );
    }

    $variation_id = $variation['id'];
    update_post_meta( $variation_id, '_vam_udesign_remote_id', $remote_variation_id );

    return vam_udesign_success_response( $variation_id );
  }
}

function vam_udesign_new_product_importer( $remote_client, $local_client ) {
  if ( empty( $remote_client ) || empty( $local_client ) ) {
    return false;
  }

  return new VAM_UDesign_Product_Importer( $remote_client, $local_client );
}
