<?php

class VAM_UDesign_Product {

  public    $product;
  protected $product_id;
  protected $views_key;
  protected $colors_key;
  protected $size_taxonomy;
  protected $view_taxonomy;
  protected $color_taxonomy;
  protected $views;

  public function __construct( $product ) {
    $this->product        = wc_get_product( $product );
    $this->product_id     = VAM_UDesign_WC_Compat::get_product_id( $this->product );
    $this->views_key      = '_vam_udesign_views';
    $this->colors_key     = '_vam_udesign_colors';
    $this->size_taxonomy  = 'pa_size';
    $this->view_taxonomy  = 'pa_view';
    $this->color_taxonomy = 'pa_color';
    $this->views          = $this->get_views_meta();
  }

  public function get_views_meta() {
    $views_meta = (array) maybe_unserialize( get_post_meta( $this->product_id, $this->views_key, true ) );

    return $views_meta;
  }

  public function update_views_meta( $views_meta ) {
    update_post_meta( $this->product_id, $this->views_key, $views_meta );
  }

  public function has_term( $term ) {
    return has_term( absint( $term->term_id ), $term->taxonomy, $this->product_id );
  }

  public function has_size( $term ) {
    return has_term( absint( $term->term_id ), $this->size_taxonomy, $this->product_id );
  }

  public function has_color( $term ) {
    return has_term( absint( $term->term_id ), $this->color_taxonomy, $this->product_id );
  }

  public function has_view( $term ) {
    return has_term( absint( $term->term_id ), $this->view_taxonomy, $this->product_id );
  }

  public function get_color_terms() {
    $color_taxonomy  = 'pa_color';
    $all_color_terms = get_terms( $color_taxonomy, apply_filters( 'woocommerce_product_attribute_terms', array(
      'orderby'    => 'name',
      'menu_order' => 'ASC',
      'hide_empty' => 0
    ) ) );

    $color_terms = array();
    foreach ( $all_color_terms as $term ) {
      if ( has_term( absint( $term->term_id ), $color_taxonomy, $this->product_id ) ) {
        $term_swatch = new VAM_UDesign_WC_Swatch_Term( $term->term_id );

        $color_terms[] = [
          'term'  => $term,
          'color' => $term_swatch->get_color()
        ];
      }
    }

    return $color_terms;
  }

  public function get_size_terms() {
    $size_taxonomy  = 'pa_size';
    $all_size_terms = get_terms( $size_taxonomy, apply_filters( 'woocommerce_product_attribute_terms', array(
      'orderby'    => 'name',
      'menu_order' => 'ASC',
      'hide_empty' => 0
    ) ) );

    $size_terms = array();
    foreach ( $all_size_terms as $term ) {
      if ( has_term( absint( $term->term_id ), $size_taxonomy, $this->product_id ) ) {
        $size_terms[] = $term;
      }
    }

    return $size_terms;
  }
}
