<?php
/**
 * Designer Shortcode
 *
 * Used on the designer page and design post type admin page.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
class VAM_UDesign_Shortcode_Designer {

  /**
   * Constructor.
   */
  public function __construct() {
    add_action( 'wp_enqueue_scripts', array( $this, 'check_public' ) );
    add_action( 'admin_enqueue_scripts', array( $this, 'check_admin' ) );
  }

  public function check_public() {
    global $post;

    if( is_a( $post, 'WP_Post' ) && has_shortcode( $post->post_content, 'udesign_designer') ) {
      $plugin_admin = new VAM_UDesign_Admin( 'vam-udesign', '1.0.0' );
      $plugin_admin->enqueue_designer_styles();
      $plugin_admin->enqueue_designer_scripts();
    }
  }

  public function check_admin() {
    global $pagenow, $post_type;

    if( in_array( $pagenow, array( 'post.php', 'post-new.php' ) ) && ( 'design' == $post_type || 'shop_order' == $post_type ) ) {
      $plugin_admin = new VAM_UDesign_Admin( 'vam-udesign', '1.0.0' );
      $plugin_admin->enqueue_designer_styles();
      $plugin_admin->enqueue_designer_scripts();
    }
  }

  /**
   * Get the shortcode content.
   *
   * @param array $atts
   * @return string
   */
  public static function get( $atts ) {
    return VAM_UDesign_Shortcodes::shortcode_wrapper( array( __CLASS__, 'output' ), $atts );
  }

  /**
   * Output the shortcode.
   *
   * @param array $atts
   */
  public static function output( $atts ) {
    $is_admin = is_admin();
    $user_logged_in = is_user_logged_in();
    $show_add_to_cart = !isset( $atts['in_admin'] );
    $show_download = isset( $atts['in_shop_order'] );
    $allow_clipart_upload = $is_admin ? true :
      get_option( 'udesign_customer_clipart_upload', 'enabled' ) == 'enabled';
    $show_branding = $is_admin ? false :
      get_option( 'udesign_branding_message', 'enabled' ) == 'enabled';
    $allow_save_design = $user_logged_in && !$is_admin &&
      get_option( 'udesign_customer_save_design_for_later', 'enabled' ) == 'enabled';

    do_action( 'vam_udesign_before_designer' );

    ?>
    <div class="designer-wrapper">
      <div class="designer-header">
        <div class="designer-header-row">
          <div class="designer-toolbar">
            <a href="javascript:void(0);" class="button designer-button button-outline-primary designer-product-item"><i class="icon-t-shirt"></i>&nbsp;<span><?php echo __( 'Products', 'vam-udesign' ); ?></span></a>
            <a href="javascript:void(0);" class="button designer-button button-outline-primary designer-text-item"><i class="icon-text-height"></i>&nbsp;<span><?php echo __( 'Add Text', 'vam-udesign' ); ?></span></a>
            <a href="javascript:void(0);" class="button designer-button button-outline-primary designer-clipart-item"><i class="icon-puzzle"></i>&nbsp;<span><?php echo __( 'Cliparts', 'vam-udesign' ); ?></span></a>
            <a href="javascript:void(0);" class="button designer-button button-outline-primary designer-design-item"><i class="icon-brush"></i>&nbsp;<span><?php echo __( 'Designs', 'vam-udesign' ); ?></span></a>

            <?php if ( $allow_clipart_upload ) { ?>
                <a href="javascript:void(0);" class="button designer-button button-outline-primary designer-image-item"><i class="icon-upload"></i>&nbsp;<span><?php echo __( 'Upload Image', 'vam-udesign' ); ?></span></a>
            <?php } ?>
          </div>
        </div>

        <div class="designer-header-row">
          <div class="designer-toolbar">
            <div class="designer-option designer-option-color">
              <p class="designer-option-label"><?php echo __( 'Color', 'vam-udesign' ); ?></p>
              <div class="swatch-dropdown">
                <div class="dropdown-value colorValue">
                  <div class="selected-option color">&nbsp;</div>
                  <i class="icon-angle-down"></i>
                </div>
                <div class="dropdown-list dropdown-list-left text-left designer-option-list-colors" style="display:none;"></div>
              </div>
            </div>
            <div class="designer-option designer-option-size">
              <p class="designer-option-label"><?php echo __( 'Size', 'vam-udesign' ); ?></p>
              <div class="swatch-dropdown">
                <div class="dropdown-value sizeValue">
                  <div class="selected-option size">&nbsp;</div>
                  <i class="icon-angle-down"></i>
                </div>
                <div class="dropdown-list dropdown-list-left dropdown-list-block text-left designer-option-list-sizes" style="display:none;"></div>
              </div>
            </div>
          </div>

          <div class="designer-toolbar designer-toolbar-right">
          <?php if ( $allow_save_design ) { ?>
            <div class="designer-option">
              <a href="javascript:void(0);" class="button alt designer-button designer-save-design"><?php echo esc_html( __( 'Save Design', 'vam-udesign' ) ); ?></a>
            </div>
          <?php } ?>
          <?php if ( $show_add_to_cart ) { ?>
            <div class="designer-option">
              <a href="javascript:void(0);" class="button alt designer-button designer-add-to-cart"><?php echo esc_html( apply_filters( 'woocommerce_product_single_add_to_cart_text', __( 'Add to cart', 'vam-udesign' ) ) ); ?></a>
            </div>
          <?php } ?>
          </div>

          <?php if ( $show_download ) { ?>
          <div class="designer-toolbar designer-toolbar-right">
            <div class="designer-option">
              <label for="designer-download-width">
                <?php echo __( 'Width (px)', 'vam-udesign' ); ?>
                <?php echo VAM_UDesign_WC_Compat::wc_help_tip( __( 'Desired width for print image.', 'vam-udesign' ) ); ?>
              </label>
              <input type="number" min="1" value="1920" class="designer-download-width" id="designer-download-width">
              <a href="javascript:void(0);" class="button alt designer-button designer-download-view"><i class="icon-download"></i><?php echo __( 'Download View', 'vam-udesign' ); ?></a>
              <a href="javascript:void(0);" class="button alt designer-button designer-download-all"><i class="icon-download"></i><?php echo __( 'Download Design', 'vam-udesign' ); ?></a>
            </div>
          </div>
          <?php } ?>
        </div>
      </div>
      <div id="designer-main-region" class="designer-main"></div>

      <?php if ( $allow_clipart_upload ) { ?>
      <div class="designer-popup upload-popup">
        <div class="designer-popup-header">
          <div class="designer-popup-heading"><?php echo __( 'Upload an image', 'vam-udesign' ); ?></div>
          <a href="#" class="designer-popup-close"><i class="icon-cancel-circled2"></i></a>
        </div>
        <div class="designer-popup-content">
          <div id="designer-upload-section">
            <div class="dropzone needsclick dz-clickable" id="designer-upload-dropzone">
              <div class="dz-message needsclick">
                <h2><?php echo __( 'Drop files here or click to upload.', 'vam-udesign' ); ?></h2>
                <span class="needsclick"><?php echo __( 'We accept the following file types: PNG, JPG, GIF and SVG.', 'vam-udesign' ); ?></span>
              </div>
            </div>
          </div>

          <div class="designer-popup-footer">
            <button id="designer-upload-image" class="button designer-button"><i class="icon-upload"></i><span><?php echo __( 'Upload', 'vam-udesign' ); ?></span></button>
          </div>
        </div>
      </div>
      <?php } ?>

      <?php if ( $allow_save_design ) { ?>
      <div class="designer-popup save-design-popup">
        <div class="designer-popup-header">
          <div class="designer-popup-heading"><?php echo __( 'Save Design', 'vam-udesign' ); ?></div>
          <a href="#" class="designer-popup-close"><i class="icon-cancel-circled2"></i></a>
        </div>
        <div class="designer-popup-content">
          <div>
            <p class="msg msg-save-design msg-success msg-save-design-success" style="display:none">
              <i class="icon-ok"></i>
              <span class="msg"><?php _e( 'Design successfully saved', 'vam-udesign' ); ?></span>
            </p>
            <p class="msg msg-save-design msg-fail msg-save-design-error" style="display:none"
              data-default-error="<?php _e( 'The design could not be saved', 'vam-udesign' ); ?>">
              <i class="icon-cancel"></i>
              <span class="msg"></span>
            </p>
            <p class="msg msg-save-design msg-invalid msg-save-design-invalid-design" style="display:none">
              <i class="icon-cancel"></i>
              <span class="msg"><?php _e( 'Please add some elements in the designer first', 'vam-udesign' ); ?></span>
            </p>
            <p class="msg msg-save-design msg-invalid msg-save-design-invalid-name" style="display:none">
              <i class="icon-cancel"></i>
              <span class="msg"><?php _e( 'Please give the design a name', 'vam-udesign' ); ?></span>
            </p>

            <label for="designer-save-design-name"><?php echo __( 'Design Name', 'vam-udesign' ); ?></label>
            <div class="clear clearfix">
              <input type="text" class="designer-save-design-name input-text" id="designer-save-design-name" maxlength="100">
              <button id="designer-save-design-button" class="button designer-button"></i><span><?php echo __( 'Save', 'vam-udesign' ); ?></span></button>
            </div>
          </div>
        </div>
      </div>
      <?php } ?>

      <?php if ( $show_add_to_cart ) { ?>
      <div class="designer-popup designer-popup-lg add-to-cart-popup">
        <div class="designer-popup-header">
          <div class="designer-popup-heading"><?php echo __( 'Add product to cart', 'vam-udesign' ); ?></div>
          <a href="#" class="designer-popup-close"><i class="icon-cancel-circled2"></i></a>
        </div>
        <div class="designer-popup-content">
          <div itemscope itemtype="Product" class="product design">
            <p class="msg-cart msg-cart-no-items" style="display:none">
              <i class="icon-cancel"></i>
              <span class="msg"><?php _e( 'Please select at least one item to add to cart', 'vam-udesign' ); ?></span>
            </p>
            <p class="msg-cart msg-cart-invalid-design" style="display:none">
              <i class="icon-cancel"></i>
              <span class="msg"><?php _e( 'Please add some elements in the designer first', 'vam-udesign' ); ?></span>
            </p>
            <p class="msg-cart msg-cart-invalid-items" style="display:none">
              <i class="icon-cancel"></i>
              <span class="msg"><?php _e( 'Item quantity must be a positive number', 'vam-udesign' ); ?></span>
            </p>
            <p class="msg-cart msg-cart-success" style="display:none">
              <i class="icon-ok"></i>
              <span class="msg"><?php _e( 'The selected items have been successfully added to cart', 'vam-udesign' ); ?></span>
            </p>
            <p class="msg-cart msg-cart-error" style="display:none"
              data-cart-default-error="<?php _e( 'The selected items could not be added to cart', 'vam-udesign' ); ?>"
              data-price-default-error="<?php _e( 'Could not calculate design price', 'vam-udesign' ); ?>">
              <i class="icon-cancel"></i>
              <span class="msg"></span>
            </p>

            <?php
              $add_to_cart_style = get_option( 'udesign_add_to_cart_style', 'bulk' );
              if ( $add_to_cart_style == 'classic' ) {
            ?>
              <div class="entry-summary">
                <?php do_action( 'vam_udesign_designer_product_summary' ); ?>
              </div>
            <?php } else { ?>
              <div class="variations-container"></div>
              <div class="cart-price">
                <span><?php _e( 'Price:', 'vam-udesign' ); ?></span>
                <span class="price-value" data-default-value="<?php echo htmlentities( wc_price( 0 ) ); ?>">
                  <?php echo wc_price( 0 ); ?>
                </span>
              </div>
              <button id="designer-update-price-button" class="button designer-button">
                <i class="icon-t-shirt"></i>
                <span><?php _e( 'Update price', 'vam-udesign' ); ?></span>
              </button>
              <button id="designer-add-to-cart-button" class="button designer-button">
                <i class="icon-cart"></i>
                <span><?php _e( 'Add to cart', 'vam-udesign' ); ?></span>
              </button>
            <?php } ?>

            <div class="clear"></div>
          </div><!-- .design -->
        </div>
      </div>
      <?php } ?>

      <?php if ( empty( $is_admin ) ) { ?>
      <div class="designer-footer">
        <?php if ( !empty( $show_branding ) ) { ?>
        <div class="udesign-branding">
          <a class="udesign-logo" href="https://vectorartmedia.com"
            target="_blank" title="<?php _e('T-shirt Design Software', 'vam-udesign') ?>">
            <span class="powered-by"><?php _e( 'Powered by', 'vam-udesign' ); ?></span>
            <img src="<?php echo VAM_UDesign()->plugin_url( '/assets/images/udesign.png' ); ?>"
              alt="<?php _e('T-shirt Design Software', 'vam-udesign') ?>" />
          </a>
        </div>
        <?php } ?>
      </div>
      <?php } ?>
    </div>
    <?php

    do_action( 'vam_udesign_after_designer' );
  }
}

new VAM_UDesign_Shortcode_Designer();
