<?php

class VAM_UDesign_WC_Swatch_Term {

  public $term_id;
  public $term;
  public $term_label;
  public $term_slug;
  public $term_description;
  public $taxonomy_slug;
  public $selected;
  public $type;
  public $thumbnail_src;
  public $thumbnail_id;
  public $size;
  public $width               = 26;
  public $height              = 26;
  public $color               = '#ffffff';
  public $attribute_meta_key  = 'swatches_id';

  public function __construct( $term_id, $taxonomy = '', $selected = false, $size = 'swatches_image_size' ) {

    $this->term_id = $term_id;
    $this->term = get_term( $term_id, $taxonomy );
    $this->term_label = $this->term->name;
    $this->term_slug = $this->term->slug;
    $this->term_description = $this->term->description;
    $this->taxonomy_slug = empty ( $taxonomy ) ?  $this->term->taxonomy : $taxonomy;
    $this->selected = $selected;
    $this->size = $size;

    $this->on_init();
  }

  public function on_init() {
    $type                 = get_woocommerce_term_meta( $this->term_id, $this->meta_key() . '_type', true );
    $color                = get_woocommerce_term_meta( $this->term_id, $this->meta_key() . '_color', true );
    $this->thumbnail_id   = get_woocommerce_term_meta( $this->term_id, $this->meta_key() . '_image', true );
    $this->thumbnail_src  = apply_filters( 'woocommerce_placeholder_img_src', WC()->plugin_url() . '/assets/images/placeholder.png' );

    $this->type = $type;
    if ( $type == 'image' ) {
      if ( $this->thumbnail_id ) {
        $imgsrc = wp_get_attachment_image_src( $this->thumbnail_id, $this->size );
        if ( $imgsrc && is_array( $imgsrc ) ) {
          $this->thumbnail_src = current( $imgsrc );
        }
      }
    } elseif ( $type == 'color' ) {
      $this->color = $color;
    }
  }

  public function get_html( $placeholder = true, $placeholder_src = 'default' ) {
    $href         = 'javascript:void(0)';
    $anchor_class = 'swatch-anchor';
    $image_class  = 'swatch-img';
    $image_alt    = 'thumbnail';
    $swatch_html  = '';
    if ( $this->type == 'image' ) {
      $swatch_html .= '<a href="' . $href . '" style="width:' . $this->width . 'px;height:' . $this->height . 'px;" title="' . esc_attr( $this->term_label ) . '" class="' . $anchor_class . '">';
      $swatch_html .= '<img src="' . $this->thumbnail_src . '" alt="' . $image_alt . '" class="wp-post-image swatch-image' . $this->meta_key() . ' ' . $image_class . '" width="' . $this->width . '" height="' . $this->height . '"/>';
      $swatch_html .= '</a>';
    } elseif ( $this->type == 'color' ) {
      list($r, $g, $b) = sscanf( $this->color, '#%02x%02x%02x' );
      $luminosity = ( ( 0.2126 * $r + 0.7152 * $g + 0.0722 * $b ) / 255 ) > 0.5 ? 'swatch-term-light' : 'swatch-term-dark';

      $swatch_html .= '<a href="' . $href . '" style="width:' . $this->width . 'px;height:' . $this->height . 'px;background-color:' . $this->color . ' !important;" data-color="' . esc_attr( $this->color ) . '" title="' . $this->term_label . '" class="pa_color ' . $anchor_class . ' ' . $luminosity . '"></a>';
    } elseif ( $this->type == 'label' ) {
      $swatch_html .= '<a href="' . $href . '" style="min-width:' . $this->width . 'px;height:' . $this->height . 'px;line-height:' . $this->height . 'px;" title="' . $this->term_description . '" class="pa_size ' . $anchor_class . '">' . $this->term_label . '</a>';
    } elseif ( $placeholder ) {
      if ( $placeholder_src == 'default' ) {
        $src = apply_filters( 'woocommerce_placeholder_img_src', WC()->plugin_url() . '/assets/images/placeholder.png' );
      } else {
        $src = $placeholder_src;
      }

      $swatch_html .= '<a href="' . $href . '" style="width:' . $this->width . 'px;height:' . $this->height . 'px;" title="' . esc_attr( $this->term_label ) . '"  class="' . $anchor_class . '">';
      $swatch_html .= '<img src="' . $src . '" alt="' . $image_alt . '" class="wp-post-image swatch-image' . $this->meta_key() . ' ' . $image_class . '" width="' . $this->width . '" height="' . $this->height . '"/>';
      $swatch_html .= '</a>';
    } else {
      return '';
    }

    $html = '<div class="select-option swatch-wrapper ' . esc_attr($this->taxonomy_slug) . ' ' . ($this->selected ? ' selected' : '') . '" data-attribute="' . esc_attr($this->taxonomy_slug) . '" data-value="' . esc_attr( $this->term_slug ) . '">';
    $html .= $swatch_html;
    $html .= '</div>';

    return $html;
  }

  public function get_type() {
    return $this->type;
  }

  public function get_color() {
    return $this->color;
  }

  public function get_image_src() {
    return $this->thumbnail_src;
  }

  public function get_image_id() {
    return $this->thumbnail_id;
  }

  public function get_width() {
    return $this->width;
  }

  public function get_height() {
    return $this->height;
  }

  public function meta_key() {
    return $this->taxonomy_slug . '_' . $this->attribute_meta_key;
  }

  public function get_name() {
    return $this->term->name;
  }
}
