<?php
/**
 * VAM_UDesign Clipart Functions
 *
 * Functions for clipart specific things.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

function vam_udesign_get_cliparts( $args = array() ) {
  // Base query arguments
  $query_args = array(
    'fields'         => 'ids',
    'udesign_query'  => 'clipart_query',
    'orderby'        => 'title',
    'order'          => 'ASC',
    'post_type'      => 'clipart',
    'post_status'    => 'publish',
    'posts_per_page' => -1,
    'tax_query'      => array(),
    'meta_query'     => array()
  );

  // Fields
  if ( !empty( $args['fields'] ) ) {
    $query_args['fields'] = $args['fields'];
  }

  // Pagination
  if ( !empty( $args['posts_per_page'] ) ) {
    $query_args['posts_per_page'] = $args['posts_per_page'];
  }
  if ( !empty( $args['paged'] ) ) {
    $query_args['paged'] = $args['paged'];
  }

  // Date filter
  if ( !empty( $args['m'] ) ) {
    $query_args['m'] = $args['m'];
  }

  // Sorting
  if ( !empty( $args['orderby'] ) ) {
    $query_args['orderby'] = $args['orderby'];
  }
  if ( !empty( $args['order'] ) ) {
    $query_args['order'] = $args['order'];
  }

  // From category
  if ( !empty( $args['category_id'] ) ) {
    $query_args['tax_query'][] = array(
      'taxonomy' => 'clipart_cat',
      'terms' => $args['category_id'],
    );
  }

  // Search query
  if ( !empty( $args['search_query'] ) ) {
    $query_args['meta_query'][] = array(
      'key'     => '_vam_udesign_search_field',
      'value'   => strtolower( sanitize_text_field( $args['search_query'] ) ),
      'compare' => 'LIKE'
    );
  }

  // Search by remote id
  if ( !empty( $args['remote_id'] ) ) {
    $remote_id = $args['remote_id'];
    if ( !is_array( $remote_id ) ) {
      $remote_id = array( $remote_id );
    }

    $query_args['meta_query'][] = array(
      'key'     => '_vam_udesign_remote_id',
      'value'   => $remote_id,
      'compare' => 'in'
    );
  }

  // Get cliparts
  $cliparts = new WP_Query( $query_args );
  wp_reset_postdata();

  return $cliparts;
}

function vam_udesign_get_clipart_image( $clipart_id, $size = 'full' ) {
  $attachment_id = get_post_meta( $clipart_id, '_clipart_image_id', true );
  if ( empty( $attachment_id ) ) {
    return false;
  }

  return wp_get_attachment_image_src( $attachment_id, $size );
}

function vam_udesign_get_clipart_thumb( $clipart_id, $size = 'full' ) {
  $attachment_id = get_post_thumbnail_id( $clipart_id );
  if ( empty( $attachment_id ) ) {
    return false;
  }

  return wp_get_attachment_image_src( $attachment_id, $size );
}

function vam_udesign_create_clipart( $args = array() ) {
  if ( empty( $args['name'] ) ) {
    return false;
  }

  if ( empty( $args['image'] ) ) {
    return false;
  }
  $image = $args['image'];

  // insert post
  $clipart = array(
    'post_title' => $args['name'],
    'post_content' => '',
    'post_status' => 'publish',
    'post_type' => 'clipart'
  );

  $clipart_id = wp_insert_post( $clipart );
  if ( empty( $clipart_id ) ) {
    return false;
  }

  // set image
  $image_id = vam_udesign_create_attachment( $image, $clipart_id );
  if ( empty( $image_id ) ) {
    return false;
  }

  update_post_meta( $clipart_id, '_clipart_image_id', $image_id );

  // set post thumb
  if ( !empty( $args['thumb'] ) ) {
    $thumb_id = vam_udesign_create_attachment( $args['thumb'], $clipart_id );
    if ( empty( $thumb_id ) ) {
      return false;
    }

    set_post_thumbnail( $clipart_id, $thumb_id );
  } else if ( !empty( $args['auto_thumb'] ) ) {
    if ( $image['type'] != 'image/svg+xml' ) {
      set_post_thumbnail( $clipart_id, $image_id );
    }
  }

  // set categories
  if ( !empty( $args['category_ids'] ) ) {
    wp_set_object_terms( $clipart_id, $args['category_ids'], 'clipart_cat' );
  }

  // set tags
  if ( !empty( $args['tag_ids'] ) ) {
    wp_set_object_terms( $clipart_id, $args['tag_ids'], 'clipart_tag' );
  }

  // set search field
  if ( !empty( $args['search_field'] ) ) {
    update_post_meta( $clipart_id, '_vam_udesign_search_field', $args['search_field'] );
  }

  // set remote id
  if ( !empty( $args['remote_id'] ) ) {
    update_post_meta( $clipart_id, '_vam_udesign_remote_id', $args['remote_id'] );
  }

  return $clipart_id;
}
