<?php
/**
 * VAM_UDesign Conditional Functions
 *
 * Functions for determining the current query/page.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * is_vam_udesign_page - Returns true if on a page which uses VAM_UDesign templates.
 * @return bool
 */
function is_vam_udesign_page() {
  return apply_filters( 'is_vam_udesign_page', ( is_design_archive() || is_design_taxonomy() || is_design() || is_designer_page() ) ? true : false );
}

if ( ! function_exists( 'is_design_archive' ) ) {

  /**
   * is_design_archive - Returns true when viewing the design type archive.
   * @return bool
   */
  function is_design_archive() {
    return ( is_post_type_archive( 'design' ) || is_page( vam_udesign_get_page_id( 'designs' ) ) );
  }
}

if ( ! function_exists( 'is_design_taxonomy' ) ) {

  /**
   * is_design_taxonomy - Returns true when viewing a design taxonomy archive.
   * @return bool
   */
  function is_design_taxonomy() {
    return is_tax( get_object_taxonomies( 'design' ) );
  }
}

if ( ! function_exists( 'is_design_category' ) ) {

  /**
   * is_design_category - Returns true when viewing a design category.
   * @param  string $term (default: '') The term slug your checking for. Leave blank to return true on any.
   * @return bool
   */
  function is_design_category( $term = '' ) {
    return is_tax( 'design_cat', $term );
  }
}

if ( ! function_exists( 'is_design_tag' ) ) {

  /**
   * is_design_tag - Returns true when viewing a design tag.
   * @param  string $term (default: '') The term slug your checking for. Leave blank to return true on any.
   * @return bool
   */
  function is_design_tag( $term = '' ) {
    return is_tax( 'design_tag', $term );
  }
}

if ( ! function_exists( 'is_design' ) ) {

  /**
   * is_design - Returns true when viewing a single design.
   * @return bool
   */
  function is_design() {
    return is_singular( array( 'design' ) );
  }
}

if ( ! function_exists( 'post_content_has_shortcode' ) ) {

  /**
   * Checks whether the content passed contains a specific short code.
   *
   * @param  string $tag Shortcode tag to check.
   * @return bool
   */
  function post_content_has_shortcode( $tag = '' ) {
    global $post;

    return is_a( $post, 'WP_Post' ) && has_shortcode( $post->post_content, $tag );
  }
}

if ( ! function_exists( 'is_designer_page' ) ) {

  /**
   * is_designer_page - Returns true when viewing the designer page.
   * @return bool
   */
  function is_designer_page() {
    return is_page( vam_udesign_get_page_id( 'designer' ) ) || post_content_has_shortcode( 'udesign_designer' );
  }
}
