<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * Tag Cloud Widget.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes/widgets
 * @author     uDesign Team <udesign@vectorartmedia.com>
 * @extends    WC_Widget
 */
class VAM_UDesign_Design_Tag_Cloud extends WC_Widget {

  /**
   * Constructor.
   */
  public function __construct() {
    $this->widget_cssclass    = 'woocommerce widget_product_tag_cloud udesign widget_design_tag_cloud';
    $this->widget_description = __( 'Your most used design tags in cloud format.', 'vam-udesign' );
    $this->widget_id          = 'vam_udesign_design_tag_cloud';
    $this->widget_name        = __( 'Design Tags', 'vam-udesign' );
    $this->settings           = array(
      'title'  => array(
        'type'  => 'text',
        'std'   => __( 'Design Tags', 'vam-udesign' ),
        'label' => __( 'Title', 'vam-udesign' )
      )
    );

    parent::__construct();
  }

  /**
   * Output widget.
   *
   * @see WP_Widget
   *
   * @since 1.0.0
   * @param array $args
   * @param array $instance
   */
  public function widget( $args, $instance ) {
    $current_taxonomy = $this->_get_current_taxonomy( $instance );

    if ( empty( $instance['title'] ) ) {
      $taxonomy = get_taxonomy( $current_taxonomy );
      $instance['title'] = $taxonomy->labels->name;
    }

    $this->widget_start( $args, $instance );

    echo '<div class="tagcloud">';

    wp_tag_cloud( apply_filters( 'vam_udesign_design_tag_cloud_widget_args', array(
      'taxonomy' => $current_taxonomy,
      'topic_count_text_callback' => array( $this, '_topic_count_text' ),
    ) ) );

    echo '</div>';

    $this->widget_end( $args );
  }

  /**
   * Return the taxonomy being displayed.
   *
   * @param  object $instance
   * @return string
   */
  public function _get_current_taxonomy( $instance ) {
    return 'design_tag';
  }

  /**
   * Retuns topic count text.
   *
   * @since 1.0.0
   * @param int $count
   * @return string
   */
  public function _topic_count_text( $count ) {
    /* translators: %s for design quantity, e.g. 1 design and 2 designs */
    return sprintf( _n( '%s design', '%s designs', $count, 'vam-udesign' ), number_format_i18n( $count ) );
  }
}
