<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

/**
 * List designs.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes/widgets
 * @author     uDesign Team <udesign@vectorartmedia.com>
 * @extends    WC_Widget
 */
class VAM_UDesign_Widget_Designs extends WC_Widget {

  /**
   * Constructor.
   */
  public function __construct() {
    $this->widget_cssclass    = 'woocommerce widget_products udesign widget_designs';
    $this->widget_description = __( 'Display a list of designs on your site.', 'vam-udesign' );
    $this->widget_id          = 'vam_udesign_designs';
    $this->widget_name        = __( 'Designs', 'vam-udesign' );
    $this->settings           = array(
      'title'  => array(
        'type'  => 'text',
        'std'   => __( 'Designs', 'vam-udesign' ),
        'label' => __( 'Title', 'vam-udesign' )
      ),
      'number' => array(
        'type'  => 'number',
        'step'  => 1,
        'min'   => 1,
        'max'   => '',
        'std'   => 5,
        'label' => __( 'Number of designs to show', 'vam-udesign' )
      ),
      'show' => array(
        'type'  => 'select',
        'std'   => '',
        'label' => __( 'Show', 'vam-udesign' ),
        'options' => array(
          ''         => __( 'All Designs', 'vam-udesign' ),
          'featured' => __( 'Featured Designs', 'vam-udesign' ),
        )
      ),
      'orderby' => array(
        'type'  => 'select',
        'std'   => 'date',
        'label' => __( 'Order by', 'vam-udesign' ),
        'options' => array(
          'date'   => __( 'Date', 'vam-udesign' ),
          'rand'   => __( 'Random', 'vam-udesign' ),
          'sales'  => __( 'Sales', 'woocommerce' ),
        )
      ),
      'order' => array(
        'type'  => 'select',
        'std'   => 'desc',
        'label' => _x( 'Order', 'Sorting order', 'vam-udesign' ),
        'options' => array(
          'asc'  => __( 'ASC', 'vam-udesign' ),
          'desc' => __( 'DESC', 'vam-udesign' ),
        )
      ),
    );

    parent::__construct();
  }

  /**
   * Query the designs and return them.
   * @param  array $args
   * @param  array $instance
   * @return WP_Query
   */
  public function get_designs( $args, $instance ) {
    $number  = ! empty( $instance['number'] ) ? absint( $instance['number'] ) : $this->settings['number']['std'];
    $show    = ! empty( $instance['show'] ) ? sanitize_title( $instance['show'] ) : $this->settings['show']['std'];
    $orderby = ! empty( $instance['orderby'] ) ? sanitize_title( $instance['orderby'] ) : $this->settings['orderby']['std'];
    $order   = ! empty( $instance['order'] ) ? sanitize_title( $instance['order'] ) : $this->settings['order']['std'];

    $query_args = array(
      'posts_per_page' => $number,
      'post_status'    => 'publish',
      'post_type'      => 'design',
      'no_found_rows'  => 1,
      'order'          => $order,
      'meta_query'     => array()
    );

    switch ( $show ) {
      case 'featured' :
        $query_args['meta_query'][] = array(
          'key'   => '_featured',
          'value' => 'yes'
        );
        break;
    }

    switch ( $orderby ) {
      case 'rand' :
        $query_args['orderby']  = 'rand';
        break;
      case 'sales' :
        $query_args['meta_key'] = 'total_sales';
        $query_args['orderby']  = 'meta_value_num';
        break;
      default :
        $query_args['orderby']  = 'date';
    }

    return new WP_Query( apply_filters( 'vam_udesign_designs_widget_query_args', $query_args ) );
  }

  /**
   * Output widget.
   *
   * @see WP_Widget
   *
   * @param array $args
   * @param array $instance
   */
  public function widget( $args, $instance ) {
    if ( $this->get_cached_widget( $args ) ) {
      return;
    }

    ob_start();

    if ( ( $designs = $this->get_designs( $args, $instance ) ) && $designs->have_posts() ) {
      $this->widget_start( $args, $instance );

      echo apply_filters( 'vam_udesign_before_widget_design_list', '<ul class="product_list_widget design_list_widget">' );

      while ( $designs->have_posts() ) {
        $designs->the_post();
        vam_udesign_get_template( 'content-widget-design.php' );
      }

      echo apply_filters( 'vam_udesign_after_widget_design_list', '</ul>' );

      $this->widget_end( $args );
    }

    wp_reset_postdata();

    echo $this->cache_widget( $args, ob_get_clean() );
  }
}
