<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/public
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

class VAM_UDesign_Public {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of the plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {
    $this->plugin_name = $plugin_name;
    $this->version = $version;
    $this->init_hooks();
  }

  /**
   * Hook into actions and filters.
   * @since  1.0.0
   */
  private function init_hooks() {
    add_action( 'init', array( $this, 'frontend_includes' ), 11 );
    add_action( 'after_setup_theme', array( $this, 'include_template_functions' ), 12 );
    add_action( 'after_setup_theme', array( $this, 'wc_template_hooks' ), 14 );
  }

  /**
   *
   * @since    1.0.0
   * @access   private
   */
  public function frontend_includes() {
    require_once 'vam-udesign-template-hooks.php';
    require_once 'class-vam-udesign-template-loader.php';
    require_once 'class-vam-udesign-frontend-scripts.php';
  }

  /**
   *
   * @since    1.0.0
   * @access   private
   */
  public function include_template_functions() {
    require_once 'vam-udesign-template-functions.php';
  }

  public function wc_template_hooks() {
    add_filter( 'woocommerce_dropdown_variation_attribute_options_args', 'vam_udesign_wc_swatches_variation_attribute_options_args', 10, 1 );
    add_filter( 'woocommerce_dropdown_variation_attribute_options_html', 'vam_udesign_wc_swatches_variation_attribute_options', 10, 2 );
  }

  /**
   * Register the stylesheets for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {

  }

  /**
   * Register the JavaScript for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {

  }
}
