<?php
/**
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/public/templates
 * @author     uDesign Team <udesign@vectorartmedia.com>
 * @version    1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly
}

global $design, $product;

$attribute_keys = array_keys( $attributes );

do_action( 'vam_udesign_before_add_to_cart_form' ); ?>

<form class="design_variations_form variations_form cart" method="post" enctype='multipart/form-data' data-design_id="<?php echo empty( $design ) ? 0 : absint( $design->id ); ?>" data-product_id="<?php echo absint( VAM_UDesign_WC_Compat::get_product_id( $product ) ); ?>" data-product_variations="<?php echo htmlspecialchars( json_encode( $available_variations ) ) ?>">
  <?php do_action( 'vam_udesign_before_variations_form' ); ?>

  <?php if ( empty( $available_variations ) && false !== $available_variations ) : ?>
    <p class="stock out-of-stock"><?php _e( 'This product is currently out of stock and unavailable.', 'vam-udesign' ); ?></p>
  <?php else : ?>
    <table class="variations" cellspacing="0">
      <tbody>
        <?php foreach ( $attributes as $attribute_name => $options ) : ?>
          <tr>
            <td class="label"><label for="<?php echo sanitize_title( $attribute_name ); ?>"><?php echo wc_attribute_label( $attribute_name ); ?></label></td>
            <td class="value">
              <?php
                $selected = isset( $_REQUEST[ 'attribute_' . sanitize_title( $attribute_name ) ] ) ? wc_clean( urldecode( $_REQUEST[ 'attribute_' . sanitize_title( $attribute_name ) ] ) ) : $product->get_variation_default_attribute( $attribute_name );
                wc_dropdown_variation_attribute_options( array( 'options' => $options, 'attribute' => $attribute_name, 'product' => $product, 'selected' => $selected ) );
                echo end( $attribute_keys ) === $attribute_name ? apply_filters( 'woocommerce_reset_variations_link', '<a class="reset_variations" href="#">' . __( 'Clear', 'vam-udesign' ) . '</a>' ) : '';
              ?>
            </td>
          </tr>
        <?php endforeach;?>
      </tbody>
    </table>

    <?php do_action( 'vam_udesign_before_add_to_cart_button' ); ?>

    <div class="single_variation_wrap">
      <?php
        /**
         * vam_udesign_before_single_variation Hook.
         */
        do_action( 'vam_udesign_before_single_variation' );

        /**
         * vam_udesign_single_variation hook. Used to output the cart button and placeholder for variation data.
         * @hooked vam_udesign_single_variation - 10 Empty div for variation data.
         * @hooked vam_udesign_single_design_add_to_cart_button - 20 Qty and cart button.
         */
        do_action( 'vam_udesign_single_variation' );

        /**
         * vam_udesign_after_single_variation Hook.
         */
        do_action( 'vam_udesign_after_single_variation' );
      ?>
    </div>

    <?php do_action( 'vam_udesign_after_add_to_cart_button' ); ?>
  <?php endif; ?>

  <?php do_action( 'vam_udesign_after_variations_form' ); ?>
</form>

<?php
do_action( 'vam_udesign_after_add_to_cart_form' );
