<?php
/**
 * VAM_UDesign Template Hooks
 *
 * Action/filter hooks used for VAM_UDesign functions/templates.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @subpackage VAM_UDesign/includes
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

add_filter( 'body_class', 'vam_udesign_body_class' );
add_filter( 'post_class', 'vam_udesign_design_post_class', 20, 3 );

/**
 * Sidebar.
 *
 * @see vam_udesign_get_sidebar()
 */
add_action( 'vam_udesign_sidebar', 'vam_udesign_get_sidebar', 10 );

/**
 * Archive descriptions.
 *
 * @see vam_udesign_taxonomy_archive_description()
 * @see vam_udesign_design_archive_description()
 */
add_action( 'vam_udesign_archive_description', 'vam_udesign_taxonomy_archive_description', 10 );
add_action( 'vam_udesign_archive_description', 'vam_udesign_design_archive_description', 10 );

/**
 * Designs Loop.
 *
 * @see vam_udesign_result_count()
 * @see vam_udesign_catalog_ordering()
 */
add_action( 'vam_udesign_before_shop_loop', 'vam_udesign_result_count', 20 );
add_action( 'vam_udesign_before_shop_loop', 'vam_udesign_catalog_ordering', 30 );

/**
 * Design Loop Items.
 *
 * @see vam_udesign_template_loop_design_link_open()
 * @see vam_udesign_template_loop_design_link_close()
 * @see vam_udesign_template_loop_add_to_cart()
 * @see vam_udesign_template_loop_design_thumbnail()
 * @see vam_udesign_template_loop_design_title()
 * @see vam_udesign_template_loop_category_link_open()
 * @see vam_udesign_template_loop_category_title()
 * @see vam_udesign_template_loop_category_link_close()
 */
add_action( 'vam_udesign_before_shop_loop_item', 'vam_udesign_template_loop_design_link_open', 10 );
add_action( 'vam_udesign_after_shop_loop_item', 'vam_udesign_template_loop_design_link_close', 5 );
add_action( 'vam_udesign_after_shop_loop_item', 'vam_udesign_template_loop_add_to_cart', 10 );
add_action( 'vam_udesign_before_shop_loop_item_title', 'vam_udesign_template_loop_design_thumbnail', 10 );
add_action( 'vam_udesign_shop_loop_item_title', 'vam_udesign_template_loop_design_title', 10 );
add_action( 'vam_udesign_before_subcategory', 'vam_udesign_template_loop_category_link_open', 10 );
add_action( 'vam_udesign_shop_loop_subcategory_title', 'vam_udesign_template_loop_category_title', 10 );
add_action( 'vam_udesign_after_subcategory', 'vam_udesign_template_loop_category_link_close', 10 );

/**
 * Subcategories.
 *
 * @see woocommerce_subcategory_thumbnail()
 */
add_action( 'vam_udesign_before_subcategory_title', 'woocommerce_subcategory_thumbnail', 10 );

/**
 * Before Single Designs Summary Div.
 *
 * @see vam_udesign_show_design_images()
 */

add_action( 'vam_udesign_before_single_design_summary', 'vam_udesign_show_design_images', 20 );

/**
 * After Single Design Summary Div.
 *
 * @see woocommerce_output_product_data_tabs()
 */
add_action( 'vam_udesign_after_single_design_summary', 'vam_udesign_output_design_data_tabs', 10 );
add_action( 'vam_udesign_after_single_design_summary', 'vam_udesign_output_related_designs', 20 );

/**
 * Design page tabs.
 */
add_filter( 'vam_udesign_design_tabs', 'vam_udesign_default_design_tabs' );
add_filter( 'vam_udesign_design_tabs', 'vam_udesign_sort_design_tabs', 99 );

/**
 * Design Summary Box.
 *
 * @see vam_udesign_template_single_title()
 * @see vam_udesign_template_single_products_gallery()
 * @see vam_udesign_template_single_products_gallery()
 * @see vam_udesign_template_single_excerpt()
 * @see vam_udesign_template_single_add_to_cart()
 * @see vam_udesign_template_single_meta()
 * @see vam_udesign_template_single_sharing()
 */
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_title', 10 );
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_excerpt', 20 );
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_products_filter', 30 );
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_products_gallery', 40 );
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_add_to_cart', 50 );
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_meta', 60 );
add_action( 'vam_udesign_single_design_summary', 'vam_udesign_template_single_sharing', 70 );

/**
 * Designer.
 *
 * @see vam_udesign_get_designer_product()
 * @see vam_udesign_template_single_add_to_cart()
 */
add_action( 'vam_udesign_designer_product_summary', 'vam_udesign_get_designer_product', 10 );
add_action( 'vam_udesign_designer_product_summary', 'vam_udesign_template_single_add_to_cart', 20 );
/**
 * Design Add to cart.
 *
 * @see vam_udesign_single_variation()
 * @see vam_udesign_single_design_add_to_cart_button()
 * @see vam_udesign_single_customize_button()
 */
add_action( 'vam_udesign_single_variation', 'vam_udesign_single_variation', 10 );
add_action( 'vam_udesign_single_variation', 'vam_udesign_single_design_add_to_cart_button', 20 );
add_action( 'vam_udesign_single_variation', 'vam_udesign_single_design_buttons', 30 );
add_action( 'woocommerce_single_variation', 'vam_udesign_single_design_buttons', 30 );

/**
 * Pagination after shop loops.
 *
 * @see woocommerce_pagination()
 */
add_action( 'vam_udesign_before_shop_loop', 'vam_udesign_setup_loop' );

/**
 * Setup loop.
 *
 * @see vam_udesign_setup_loop()
 */
add_action( 'vam_udesign_after_shop_loop', 'woocommerce_pagination', 10 );

/**
 * Notices.
 */
add_action( 'vam_udesign_before_shop_loop', 'wc_print_notices', 10 );
add_action( 'vam_udesign_before_single_design', 'wc_print_notices', 10 );
