<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://example.com
 * @since             1.0.0
 * @package           VAM_UDesign
 *
 * @wordpress-plugin
 * Plugin Name:       WooCommerce - uDesign Integration
 * Plugin URI:        https://vectorartmedia.com
 * Description:       WooCommerce - VectorArtMedia uDesign integration plugin
 * Version:           1.2.3
 * Author:            Vector Art Media Team
 * Author URI:        https://vectorartmedia.com
 * Requires at least: 4.4
 * Tested up to:      4.9.7
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       vam-udesign
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

define( 'VAM_UDESIGN_PLUGIN_SLUG', plugin_basename( __FILE__ ) );

/**
 * Required functions.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
if ( ! function_exists( 'is_woocommerce_active' ) ) {
  require_once( 'includes/woo-includes/woo-functions.php' );
}

/**
 * WooCommerce missing notice.
 *
 * @return string
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
function woocommerce_missing_notice_vam_udesign() {
  echo '<div class="error notice is-dismissible"><p>' . sprintf( __( 'WooCommerce - uDesign Integration Plugin requires WooCommerce to be installed and active. You can download %s here.', 'vam-udesign' ), '<a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce</a>' ) . '</p><button type="button" class="notice-dismiss"><span class="screen-reader-text">' . __('Dismiss this notice.', 'vam-udesign') . '</span></button></div>';
}

/**
 * WooCommerce active check.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
function maybe_self_deactivate_vam_udesign() {
  if ( ! is_woocommerce_active() ) {
    require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    deactivate_plugins( VAM_UDESIGN_PLUGIN_SLUG );
    unset( $_GET['activate'] );
    add_action( 'admin_notices', 'woocommerce_missing_notice_vam_udesign' );
  }
}
add_action( 'plugins_loaded', 'maybe_self_deactivate_vam_udesign' );

/**
 * Exit after WooCommerce missing notice.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
if ( ! is_woocommerce_active() ) {
  return;
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-vam-udesign-activator.php
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
function activate_vam_udesign() {
  require_once plugin_dir_path( __FILE__ ) . 'includes/class-vam-udesign-activator.php';
  VAM_UDesign_Activator::activate();
}
register_activation_hook( __FILE__, 'activate_vam_udesign' );

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-vam-udesign-deactivator.php
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
function deactivate_vam_udesign() {
  require_once plugin_dir_path( __FILE__ ) . 'includes/class-vam-udesign-deactivator.php';
  VAM_UDesign_Deactivator::deactivate();
}
register_deactivation_hook( __FILE__, 'deactivate_vam_udesign' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-vam-udesign.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since      1.0.0
 * @package    VAM_UDesign
 * @author     uDesign Team <udesign@vectorartmedia.com>
 */
function run_vam_udesign() {
	$plugin = VAM_UDesign::instance();
	$plugin->run();
}

run_vam_udesign();
